/********************************** <lalVerbatim file="ApplySearchCV">
Author: Julien Sylvestre
$Id: ApplySearch.c,v 1.5 2003/09/04 21:10:41 julien Exp $
**************************************************** </lalVerbatim> */

#include <lal/LALStdlib.h>
#include <lal/LALDatatypes.h>
#include <lal/Comm.h>
#include <lal/Inject.h>
#include <lal/Random.h>
#include <lal/SimulateCoherentGW.h>
#include <InitSearch.h>
#include <strings.h>
#include <unistd.h>
#include <BuildDB.h>
#include <signal.h>
#include <lal/DetResponse.h>
#include <lal/TimeDelay.h>
#include <lal/DetectorSite.h>
#include <lal/SkyCoordinates.h>
#include <time.h>
#include <math.h>
#include <LALWrapperConfig.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#define MAXCOMMENT 16384

NRCSID( APPLYSEARCHC, "power $Id: ApplySearch.c,v 1.5 2003/09/04 21:10:41 julien Exp $");

LALStatus *gStatus;
void BurstSigHandler(int);

void
LALApplySearch(
	       LALStatus             *status,
	       LALSearchOutput       *output,
	       LALSearchInput        *input,
	       LALApplySearchParams  *applyParams
	       )
{
  static BOOLEAN firsttime = 1;
  static UINT4 numSlaves = 0,
    activeSlaves = 0,
    jobsDone = 0,
    jobsTotal = 0,
    jobsGranularity = 0,
    jobsCounter = 0,
    zerobid = 0,
    jobsParams = 0;

  /*
  static FILE *fid = NULL;
  */
  static int fid = -1;

  INT4 i, myRank;
  BurstSearchParams *params = (BurstSearchParams*)(applyParams->searchParams);

  InitExchParams           initExchParams;

  INITSTATUS (status, "LALApplySearch", APPLYSEARCHC);
  ATTATCHSTATUSPTR (status);

#ifdef DEBUGBURST
  fprintf(stderr,"Entering ApplySearch\n");
  fflush(NULL);
#endif


  gStatus = status;

  MPI_Comm_rank( *(applyParams->mpiParams->comm), &myRank );

  /* check that the master is running on node rank zero */
  if ( params->searchMaster && myRank != 0 )
  {
    ABORT( status, INITSEARCHH_ERANK, INITSEARCHH_MSGERANK );
  }

  /* and that the slave is not running on node rank zero */
  if ( !params->searchMaster && myRank == 0 )
  {
    ABORT( status, INITSEARCHH_ERANK, INITSEARCHH_MSGERANK );
  }

  initExchParams.mpiComm   = *(applyParams->mpiParams->comm);
  initExchParams.myProcNum = myRank;
 
  if(firsttime) {

    if(params->searchMaster) {

      if(params->outputMethod == 2) {

#ifdef DEBUGBURST
	  fprintf(stderr,"file to open: %s\n",params->dsoOutput);
	  fflush(NULL);
#endif

	  fid = open(params->dsoOutput,O_WRONLY|O_CREAT|O_TRUNC,S_IRUSR|S_IRGRP|S_IROTH|S_IWOTH|S_IWGRP|S_IWUSR);

	  if(fid == -1) {
#ifdef DEBUGBURST
	    perror("ERROR from fopen:");
	    fflush(NULL);
#endif

	    ABORT(status,INITSEARCHH_EFILE,INITSEARCHH_MSGEFILE);
	  }

#ifdef DEBUGBURST
      fprintf(stderr,"MASTER: fid = %i\n",fid);
      fflush(NULL);
#endif
      
      }
    }

    firsttime = 0;

    zerobid = myRank * MAXNBURSTS;

    /* get the number of search slaves */
    numSlaves = 0;
    MPI_Comm_size( *(applyParams->mpiParams->comm), &numSlaves );
    numSlaves--;

    activeSlaves = numSlaves;

    if(params->MAXinj != 0) {
      jobsTotal = 1 + params->Nwaveforms * params->Ninj / params->Ninj_per_segment;
    } else {
      jobsTotal = 1 + params->Nwaveforms * params->Nwaveform_amplitude * params->Nalpha * params->Ndelta * params->Npsi * params->Ninj / params->Ninj_per_segment;
    }

    if(params->Nwaveforms && !jobsTotal) {
      ABORT(status, INITSEARCHH_EIN, INITSEARCHH_MSGEIN);
    }

    if(!jobsTotal) {
      jobsTotal = 1;
    }

    jobsParams = 1;
    for(i=0;i<params->Nparams;i++) {
      jobsTotal *= params->NNparams[i];
      jobsParams *= params->NNparams[i];
    }

    jobsGranularity = jobsTotal / numSlaves;
    if(jobsGranularity < 1) {
      jobsGranularity = 1;
    }
    if(jobsGranularity > 5) { /* to avoid timeouts */
      jobsGranularity = 5;
    }


  }


  if(params->searchMaster)  {

#ifdef DEBUGBURST
    fprintf(stderr,"MASTER: %u %u %u\n",jobsTotal,jobsDone,activeSlaves);
    fflush(NULL);
#endif

    if(jobsTotal > jobsDone || activeSlaves) {

      ExchParams *thisExch = NULL;
      UINT4Vector *u4v = NULL;
      UINT4 u4;

      /* wait for a message */
      LALInitializeExchange (status->statusPtr, &thisExch, NULL, &initExchParams);
      CHECKSTATUSPTR (status);
      
      /* case the exchange type */
      switch (thisExch->exchObjectType)
      {
      case ExchFileLock:
#ifdef DEBUGBURST
	fprintf(stderr,"MASTER: ExchFileLock\n");
	fflush(NULL);
#endif
	
	LALExchangeUINT4(status->statusPtr, &u4, thisExch);
	CHECKSTATUSPTR (status);

	{
	  CHARVector *cvec;
	  LALCHARCreateVector(status->statusPtr, &cvec, u4);
	  CHECKSTATUSPTR (status);
	  
	  LALExchangeCHARVector(status->statusPtr, cvec, thisExch);
	  CHECKSTATUSPTR (status);

	  /*
	  if(fwrite(cvec->data,1,cvec->length,fid) < u4) {
	  */
	  if(write(fid,cvec->data,cvec->length) < u4) {
#ifdef DEBUGBURST
	    perror("ERROR from fwrite:");
	    fflush(NULL);
#endif
	    
	    ABORT(status,INITSEARCHH_EFILE,INITSEARCHH_MSGEFILE);
	  }

	  LALCHARDestroyVector(status->statusPtr, &cvec);
	  CHECKSTATUSPTR (status);
	}

	break;

      case ExchParamList:
#ifdef DEBUGBURST
	fprintf(stderr,"MASTER: ExchParamList\n");
	fflush(NULL);
#endif
	/* send parameters id to slave */
	LALU4CreateVector(status->statusPtr, &u4v, jobsGranularity);
	CHECKSTATUSPTR (status);

	if(jobsCounter >= jobsTotal) {
	  u4v->length = 0;
	} else {

	  for(i=0; i<jobsGranularity; i++) {
	    u4v->data[i] = jobsCounter;
	    jobsCounter++;

	    if(jobsCounter == jobsTotal && i+1 != jobsGranularity) {
	      u4v->length = i+1;
	      break;
	    }
	  }

	}

#ifdef DEBUGBURST
	fprintf(stderr,"MASTER: u4v->length = %u, jobsGranularity = %u\n",u4v->length,jobsGranularity);
	fflush(NULL);
#endif

	LALExchangeUINT4(status->statusPtr, &(u4v->length), thisExch);
	CHECKSTATUSPTR (status);

	if(u4v->length) {
	  LALExchangeUINT4Vector (status->statusPtr, u4v, thisExch);
	  CHECKSTATUSPTR (status);

#ifdef DEBUGBURST
	  fprintf(stderr,"master sent unit4 vector:\n");
	  for(i=0;i<u4v->length;i++)
	    fprintf(stderr,"%u\n",u4v->data[i]);
	  fflush(NULL);
#endif

	}

	u4v->length = jobsGranularity;
	LALU4DestroyVector(status->statusPtr, &u4v);
	CHECKSTATUSPTR (status);

#ifdef DEBUGBURST
	fprintf(stderr,"MASTER: done ExchParamList\n");
	fflush(NULL);
#endif

	break;

      case ExchFinished:
#ifdef DEBUGBURST
	fprintf(stderr,"MASTER: ExchFinished\n");
	fflush(NULL);
#endif
	/* slave tells master how many jobs it has completed */
	LALExchangeUINT4 (status->statusPtr, &u4, thisExch);
	CHECKSTATUSPTR (status);

	jobsDone += u4;

	break;

      case ExchDeadSlave:
#ifdef DEBUGBURST
	fprintf(stderr,"MASTER: ExchDeadSlave\n");
	fflush(NULL);
#endif
	/* slave tells master it's shuting down */
	activeSlaves--;
	break;

      default:
#ifdef DEBUGBURST
	fprintf(stderr,"MASTER: unknown\n");
	fflush(NULL);
#endif
	ABORT ( status, INITSEARCHH_EIN, INITSEARCHH_MSGEIN );

      }

      LALFinalizeExchange (status->statusPtr, &thisExch);
      CHECKSTATUSPTR (status);

#ifdef DEBUGBURST
	fprintf(stderr,"MASTER: Leaving\n");
	fflush(NULL);
#endif

      output->numOutput = 0;
      output->result = NULL;
      output->fracRemaining = (REAL4)(jobsTotal-jobsDone) / (REAL4)jobsTotal;
      output->notFinished = 1;

    } else {
#ifdef DEBUGBURST
      fprintf(stderr,"MASTER: done.\n");
      fflush(NULL);
#endif

      if(params->outputMethod == 2) {

#ifdef DEBUGBURST
      fprintf(stderr,"MASTER: fid = %i\n",fid);
      fflush(NULL);
#endif

      if(fsync(fid)) {
#ifdef DEBUGBURST
	perror("ERROR from fsync:");
	fflush(NULL);
#endif
	  
	ABORT(status,INITSEARCHH_EFILE,INITSEARCHH_MSGEFILE);
      }


      {
	struct stat bstat;

	bzero(&bstat,sizeof(struct stat));

	fstat(fid,&bstat);

	if((bstat.st_mode &(S_IRUSR|S_IRGRP|S_IROTH|S_IWOTH|S_IWGRP|S_IWUSR)) != 0666) 
	  {

#ifdef DEBUGBURST
	    fprintf(stderr,"mode = %u, need to chmod!\n",bstat.st_mode &(S_IRUSR|S_IRGRP|S_IROTH|S_IWOTH|S_IWGRP|S_IWUSR));
	    fflush(NULL);
#endif

	    if(fchmod(fid,S_IRUSR|S_IRGRP|S_IROTH|S_IWOTH|S_IWGRP|S_IWUSR)) {
#ifdef DEBUGBURST
	      perror("ERROR from chmod:");
	      fflush(NULL);
#endif
	      
	      ABORT(status,INITSEARCHH_EFILE,INITSEARCHH_MSGEFILE);
	    }
	  }
      }

      if(close(fid)) {
#ifdef DEBUGBURST
	perror("ERROR from fclose:");
	fflush(NULL);
#endif
	  
	ABORT(status,INITSEARCHH_EFILE,INITSEARCHH_MSGEFILE);
      }
      }

      /* we're done */
      output->numOutput = 0;
      output->result = NULL;
      output->fracRemaining = 0;
      output->notFinished   = 0;
    }

  } else {
    /* I am a slave */

    UINT4Vector *u4v = NULL;
    UINT4 u4;

    ExchParams             *thisExch = NULL;
    ExchParams exchParamList,
      exchFinished,
      exchDeadSlave;

#ifdef DEBUGBURST
      fprintf(stderr,"Entering SLAVE\n");
      fflush(NULL);
#endif

    exchFinished.exchObjectType  = ExchFinished;
    exchFinished.send            = 1; 
    exchFinished.numObjects      = 1;
    exchFinished.partnerProcNum  = 0; 

    exchParamList.exchObjectType  = ExchParamList;
    exchParamList.send            = 0; 
    exchParamList.numObjects      = 1;
    exchParamList.partnerProcNum  = 0; 

    exchDeadSlave.exchObjectType  = ExchDeadSlave;
    exchDeadSlave.send            = 1; 
    exchDeadSlave.numObjects      = 0;
    exchDeadSlave.partnerProcNum  = 0; 

    LALInitializeExchange (status->statusPtr, &thisExch, 
			   &exchParamList, &initExchParams);
    CHECKSTATUSPTR (status);

    LALExchangeUINT4(status->statusPtr, &u4, thisExch);
    CHECKSTATUSPTR (status);

    if(u4) {

      LALU4CreateVector(status->statusPtr, &u4v, u4);
      CHECKSTATUSPTR (status);

      LALExchangeUINT4Vector (status->statusPtr, u4v, thisExch);
      CHECKSTATUSPTR (status);

#ifdef DEBUGBURST
      fprintf(stderr,"client got unit4 vector:\n");
      for(i=0;i<u4v->length;i++)
	fprintf(stderr,"%u\n",u4v->data[i]);
      fflush(NULL);
#endif

    }

    LALFinalizeExchange (status->statusPtr, &thisExch);
    CHECKSTATUSPTR (status);

    if(u4 == 0) {
      /* received nothing from master; die */
      LALInitializeExchange (status->statusPtr, &thisExch, 
			     &exchDeadSlave, &initExchParams);
      CHECKSTATUSPTR (status);
      
      LALFinalizeExchange (status->statusPtr, &thisExch);
      CHECKSTATUSPTR (status);

      output->numOutput     = 0;
      output->result        = NULL;
      output->fracRemaining = 0;
      output->notFinished   = 0;

#ifdef DEBUGBURST
      fprintf(stderr,"SLAVE: done.\n");
      fflush(NULL);
#endif

      DETATCHSTATUSPTR (status);
      RETURN (status);

    } else {

      EventIDColumn boutput, stdOutput;
      BurstParameter bparams;
      BurstParameter *bparamsptr, *bptr, *wave=NULL, *wamp;
      BurstParameterList *ETGparams;
      CHAR comment[MAXCOMMENT];
      REAL4 *wbuf = NULL;

      REAL4 wAmp, Alpha, Delta, Psi;
      REAL4Vector *wAmpv, *Alphav, *Deltav, *Psiv;

      UINT4 si,j, runp, runs;

      RandomParams *rpar = NULL;
      REAL4 arandomnumber;
      BOOLEAN got_wid = 0;

      LALCreateRandomParams(status->statusPtr, &rpar, myRank * clock());
      CHECKSTATUSPTR( status );

      /* initialize output */
      output->numOutput     = 0;
      output->result        = NULL;


      /* got a list of parameters; iterate and append to output */
      for(si=0;si<u4v->length;si++) {
	UINT4 wid, waid, aid, did, pid, Nid;
	UINT4 *Pid;

	wAmp = Alpha = Delta = Psi = 0.0;
	wAmpv = Alphav = Deltav = Psiv = NULL;

	wid = waid = aid = did = pid = Nid = 0;

	bzero(&boutput, sizeof(EventIDColumn));
	bzero(&stdOutput, sizeof(EventIDColumn));
	bzero(&bparams, sizeof(BurstParameter));

	if(u4v->data[si] < jobsParams) {
	  Nid = 0;
	  runs = u4v->data[si];
	  runp = jobsParams;	  
	} else {

	  u4v->data[si] -= jobsParams;

	  if(params->Nwaveforms) {
	    got_wid = 1;

	    runp = (jobsTotal - jobsParams) / params->Nwaveforms;
	    wid = u4v->data[si] / runp;

	    runs = u4v->data[si] - wid * runp;
	    runp /= params->Nwaveform_amplitude;
	    waid = runs / runp;
	  
	    wid++;

	    runs -= waid * runp;
	    runp /= params->Nalpha;
	    aid = runs / runp;

	    runs -= aid * runp;
	    runp /= params->Ndelta;
	    did = runs / runp;

	    runs -= did * runp;
	    runp /= params->Npsi;
	    pid = runs / runp;

	    runs -= pid * runp;
	    runp /= params->Ninj / params->Ninj_per_segment;
	    Nid = runs / runp;
	  } else {
	    /*
	    Nid = 0;
	    runs = u4v->data[si];
	    runp = jobsTotal;
	    */
	    ABORT(status, INITSEARCHH_EIN, INITSEARCHH_MSGEIN);
	  }
	}

#ifdef DEBUGBURST
	  fprintf(stderr,"wid = %u, waid = %u, aid = %u, did = %u, pid = %u, Nid = %u\n",wid,waid,aid,did,pid,Nid);
	  fflush(NULL);
#endif


	Pid = (UINT4 *)LALMalloc(params->Nparams * sizeof(UINT4));
	RASSERT(Pid, status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);

	{
	  UINT4 Mid = Nid;

	  for(i=0; i<params->Nparams; i++) {
	    runs -= Mid * runp;
	    runp /= params->NNparams[i];
	    Mid = Pid[i] = runs / runp;
	  }
	}

	if(wid) {
	  wave = &(params->waveforms);
	  for(i=0; i<wid; i++) {
	    wave = wave->next;
	  }

	  wamp = params->waveform_amplitudes.next;
	  if(wamp) {
	    if(wamp->random == 1) {
	      REAL4 lo,hi;
	      
	      if(wamp->real4_) {
		lo = *(wamp->real4_);
	      } else {
		lo = (REAL4)(*(wamp->int4_));
	      }
	      
	      wamp = wamp->next;
	    
	      if(wamp->real4_) {
		hi = *(wamp->real4_);
	      } else {
		hi = (REAL4)(*(wamp->int4_));
	      }

	      LALUniformDeviate(status->statusPtr, &arandomnumber, rpar);
	      CHECKSTATUSPTR( status );
	      wAmp = lo + (hi-lo)*arandomnumber;

	    } else {
	      if(params->MAXinj) {
		waid = Nid;
	      }
	      for(i=0; i<waid; i++) {
		wamp = wamp->next;
	      }
	      if(wamp->real4vector_) {
		wAmpv = wamp->real4vector_;
	      } else if(wamp->real4_) {
		wAmp = *(wamp->real4_);
	      } else {
		wAmp = (REAL4)(*(wamp->int4_));
	      }
	    }
	  }
	  
	  wamp = params->alpha.next;
	  if(wamp) {
	    if(wamp->random == 1) {
	      REAL4 lo,hi;
	      
	      if(wamp->real4_) {
		lo = *(wamp->real4_);
	      } else {
		lo = (REAL4)(*(wamp->int4_));
	      }

	      wamp = wamp->next;
	      
	      if(wamp->real4_) {
		hi = *(wamp->real4_);
	      } else {
		hi = (REAL4)(*(wamp->int4_));
	      }

	      LALUniformDeviate(status->statusPtr, &arandomnumber, rpar);
	      CHECKSTATUSPTR( status );
	      Alpha = lo + (hi-lo)*arandomnumber;

	    } else {
	      if(params->MAXinj) {
		aid = Nid;
	      }
	      for(i=0; i<aid; i++) {
		wamp = wamp->next;
	      }
	       if(wamp->real4vector_) {
		Alphav = wamp->real4vector_;
	      } else if(wamp->real4_) {
		Alpha = *(wamp->real4_);
	      } else {
		Alpha = (REAL4)(*(wamp->int4_));
	      }
	    }
	  }
	  
	  wamp = params->delta.next;
	  if(wamp) {
	    if(wamp->random == 1) {
	      REAL4 lo,hi;
	      
	      if(wamp->real4_) {
		lo = *(wamp->real4_);
	      } else {
		lo = (REAL4)(*(wamp->int4_));
	      }

	      wamp = wamp->next;

	      if(wamp->real4_) {
		hi = *(wamp->real4_);
	      } else {
		hi = (REAL4)(*(wamp->int4_));
	      }

	      LALUniformDeviate(status->statusPtr, &arandomnumber, rpar);
	      CHECKSTATUSPTR( status );
	      Delta = lo + (hi-lo)*arandomnumber;
	      
	    } else {
	      if(params->MAXinj) {
		did = Nid;
	      }
	      for(i=0; i<did; i++) {
		wamp = wamp->next;
	      }
	       if(wamp->real4vector_) {
		Deltav = wamp->real4vector_;
	      } else if(wamp->real4_) {
		Delta = *(wamp->real4_);
	      } else {
		Delta = (REAL4)(*(wamp->int4_));
	      }
	    }
	  }

	  wamp = params->psi.next;
	  if(wamp) {
	    if(wamp->random == 1) {
	      REAL4 lo,hi;
	      
	      if(wamp->real4_) {
		lo = *(wamp->real4_);
	      } else {
		lo = (REAL4)(*(wamp->int4_));
	      }
	      
	      wamp = wamp->next;
	    
	      if(wamp->real4_) {
		hi = *(wamp->real4_);
	      } else {
		hi = (REAL4)(*(wamp->int4_));
	      }

	      LALUniformDeviate(status->statusPtr, &arandomnumber, rpar);
	      CHECKSTATUSPTR( status );
	      Psi = lo + (hi-lo)*arandomnumber;
	      
	    } else {
	      if(params->MAXinj) {
		pid = Nid;
	      }
	      for(i=0; i<pid; i++) {
		wamp = wamp->next;
	      }
	       if(wamp->real4vector_) {
		Psiv = wamp->real4vector_;
	      } else if(wamp->real4_) {
		Psi = *(wamp->real4_);
	      } else {
		Psi = (REAL4)(*(wamp->int4_));
	      }
	    }
	  }
	}

#ifdef DEBUGBURST
  fprintf(stderr,"updating comment...\n");
  fflush(NULL);
#endif

	if(wid) {

	  sprintf(comment,"%s,",1+wave->char_);

#ifdef DEBUGBURST
  fprintf(stderr,"\t\twAmpv\n");
  fflush(NULL);
#endif

	  if(wAmpv) {
	    UINT4 kj;
	    sprintf(comment,"%s(%g",comment,wAmpv->data[0]);
	    for(kj=1;kj<wAmpv->length;kj++) {
	      sprintf(comment,"%s,%g",comment,wAmpv->data[kj]);
	    }
	    sprintf(comment,"%s),",comment);
	  } else {
	    sprintf(comment,"%s%g,",comment,wAmp);
	  }
	
#ifdef DEBUGBURST
  fprintf(stderr,"\t\tAlphav\n");
  fflush(NULL);
#endif
  
	  if(Alphav) {
	    UINT4 kj;
	    sprintf(comment,"%s(%g",comment,Alphav->data[0]);
	    for(kj=1;kj<Alphav->length;kj++) {
	      sprintf(comment,"%s,%g",comment,Alphav->data[kj]);
	    }
	    sprintf(comment,"%s),",comment);
	  } else {
	    sprintf(comment,"%s%g,",comment,Alpha);
	  }

#ifdef DEBUGBURST
  fprintf(stderr,"\t\tDeltav\n");
  fflush(NULL);
#endif

	  if(Deltav) {
	    UINT4 kj;
	    sprintf(comment,"%s(%g",comment,Deltav->data[0]);
	    for(kj=1;kj<Deltav->length;kj++) {
	      sprintf(comment,"%s,%g",comment,Deltav->data[kj]);
	    }
	    sprintf(comment,"%s),",comment);
	  } else {
	    sprintf(comment,"%s%g,",comment,Delta);
	  }

#ifdef DEBUGBURST
  fprintf(stderr,"\t\tPsiv\n");
  fflush(NULL);
#endif

	  if(Psiv) {
	    UINT4 kj;
	    sprintf(comment,"%s(%g",comment,Psiv->data[0]);
	    for(kj=1;kj<Psiv->length;kj++) {
	      sprintf(comment,"%s,%g",comment,Psiv->data[kj]);
	    }
	    sprintf(comment,"%s)",comment);
	  } else {
	    sprintf(comment,"%s%g",comment,Psi);
	  }

	} else {
	  sprintf(comment,",,,,,,");
	}


	/* injection code */
	if(wid) {
	  LALDetAMResponse F; /* response functions */
	  DetTimeAndASource dtS; /* for time delays */
	  LALPlaceAndGPS pGPS; /* for time delays */
	  LALDetAndSource dAs; /* for responses */
	  LALSource source; /* for responses */
	  SkyPosition position;  
	  LIGOTimeGPS t0;
	  REAL8 tDelay;
	  REAL4TimeSeries sig;
	  BurstParameter *injTime;
	  INT4 ndt;
	  BOOLEAN F1 = 0;
	  REAL4 lo,hi;
	  BOOLEAN injRandom;

	  INT4 *NDT = NULL;
	  INT4 indt, NDTN = 0;

#ifdef DEBUGBURST
  fprintf(stderr,"Beginning Injection...\n");
  fflush(NULL);
#endif

	  lo = hi = 0.0;

	  /* save a copy of input data */
	  wbuf = (REAL4 *)LALMalloc(params->data.data->data->vectorLength * sizeof(REAL4));
	  RASSERT(wbuf, status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
	  memcpy(wbuf, params->data.data->data->data, params->data.data->data->vectorLength * sizeof(REAL4));

	  sprintf(comment,"%s,(",comment);

	  sig.deltaT = params->data.data->deltaT;
	  sig.f0 = params->data.data->f0;
	  sig.sampleUnits = params->data.data->sampleUnits;

#ifdef DEBUGBURST
      fprintf(stderr,"**%i\t%i\n",wid,params->wave_pc[wid-1].data->vectorLength);
      fflush(NULL);
#endif 

	  sig.data = NULL;
	  LALCreateVector(status->statusPtr, &(sig.data), params->wave_pc[wid-1].data->vectorLength);
	  CHECKSTATUSPTR (status);


	  injTime = params->injTimes.next;

	  wamp = params->injTimes.next;
	  injRandom = 0;

	  if(wamp->random == 1) {

	    injRandom = 1;

	    if(wamp->real4_) {
	      lo = *(wamp->real4_);
	    } else {
	      lo = (REAL4)(*(wamp->int4_));
	    }

	    wamp = wamp->next;

	    if(wamp->real4_) {
	      hi = *(wamp->real4_);
	    } else {
	      hi = (REAL4)(*(wamp->int4_));
	    }
	  } else if(params->MAXinj ==0) {
	    for(i=0;i<Nid;i++) {
	      injTime = injTime->next; 
	    }
	  }


	  if(params->channel == strstr(params->channel,"H1")) {
	    pGPS.p_detector = dAs.pDetector = (LALDetector *)(&(lalCachedDetectors[LALDetectorIndexLHODIFF])); 
	  } else if(params->channel == strstr(params->channel,"H2")) {
	    pGPS.p_detector = dAs.pDetector = (LALDetector *)(&(lalCachedDetectors[LALDetectorIndexLHODIFF])); 
	  } else if(params->channel == strstr(params->channel,"L1")) {
	    pGPS.p_detector = dAs.pDetector = (LALDetector *)(&(lalCachedDetectors[LALDetectorIndexLLODIFF]));
	  } else {
	    ABORT(status, INITSEARCHH_EIN, INITSEARCHH_MSGEIN);
	  }

#ifdef DEBUGBURST
	    fprintf(stderr,"APPLYSEARCH: looping over %i\n",params->Ninj_per_segment);
	    fflush(NULL);
#endif

	  for(i=0; i<params->Ninj_per_segment; i++) { 

	    if(wAmpv) {
	      if(wAmpv->length != params->Ninj_per_segment) {
		ABORT(status, INITSEARCHH_EIN, INITSEARCHH_MSGEIN);
	      }
	      wAmp = wAmpv->data[i];
	    }

	    if(Alphav) {
	      if(Alphav->length != params->Ninj_per_segment) {
		ABORT(status, INITSEARCHH_EIN, INITSEARCHH_MSGEIN);
	      }
	      Alpha = Alphav->data[i];
	    }

	    if(Deltav) {
	      if(Deltav->length != params->Ninj_per_segment) {
		ABORT(status, INITSEARCHH_EIN, INITSEARCHH_MSGEIN);
	      }
	      Delta = Deltav->data[i];
	    }

	    if(Psiv) {
	      if(Psiv->length != params->Ninj_per_segment) {
		ABORT(status, INITSEARCHH_EIN, INITSEARCHH_MSGEIN);
	      }
	      Psi = Psiv->data[i];
	    }

	  position.longitude = Alpha;
	  position.latitude = Delta;
	  position.system = COORDINATESYSTEM_EQUATORIAL;
	  dtS.p_source = &position;

	  dAs.pSource = &source;
	  source.equatorialCoords = position;
	  source.orientation = Psi;

	  /* inject waveform: */

#ifdef DEBUGBURST
	    fprintf(stderr,"APPLYSEARCH: before LALUniformDeviate\n");
	    fflush(NULL);
#endif

	    if(injRandom) {
	      LALUniformDeviate(status->statusPtr, &arandomnumber, rpar);
	      CHECKSTATUSPTR( status );
	      ndt = (INT4)floor(lo + (hi-lo)*arandomnumber);

#ifdef DEBUGBURST
	      fprintf(stderr,"%i\t%g\t%i\n",i,arandomnumber,ndt);
	      fflush(NULL);
#endif

	    } else {
	      if(injTime->real4vector_) {
		if(injTime->real4vector_->length != params->Ninj_per_segment) {
		  ABORT(status, INITSEARCHH_EIN, INITSEARCHH_MSGEIN);
		}
		ndt = (INT4)(injTime->real4vector_->data[i]);
	      } else if(injTime->real4_) {
		ndt = (INT4)(*(injTime->real4_));
		injTime = injTime->next;	      
	      } else {
		ndt = *(injTime->int4_);
		injTime = injTime->next;
	      }
	    }

	    /* check no overlap of signals */
	    if(params->MAXinj == 0) {
	      for(indt=0;indt<NDTN;indt++) {
		if(abs(NDT[indt] - ndt) <= sig.data->length) {
		  break;
		}
	      }
	    } else {
	      indt = NDTN;
	    }
	    
	    if(indt == NDTN) {
	      
	      NDTN++;
	      NDT = (INT4 *)LALRealloc(NDT, NDTN * sizeof(INT4));
	      NDT[indt] = ndt;

	      if(F1) {
		sprintf(comment,"%s,%i",comment,ndt);
	      } else {
		sprintf(comment,"%s%i",comment,ndt);
		F1 = 1;
	      }
	    
	      {
		REAL8 dt = (REAL8)ndt * sig.deltaT;

		t0 = params->data.data->epoch;

		t0.gpsSeconds += (INT4)floor(dt);
		t0.gpsNanoSeconds += (INT4)floor(1E9 * (dt - floor(dt)));

		if(t0.gpsNanoSeconds >= 1000000000) {
		  (t0.gpsSeconds)++;
		  (t0.gpsNanoSeconds)-=1000000000;
		}
		
	      }

	      dtS.p_det_and_time = &pGPS;
	      pGPS.p_gps = &t0;

#ifdef DEBUGBURST
	      fprintf(stderr,"APPLYSEARCH: before LALTimeDelayFromEarthCenter\n");
	      fflush(NULL);
#endif

	      if(dtS.p_source->longitude < 0.0 ||
		 dtS.p_source->longitude > 6.2832 ||
		 dtS.p_source->latitude < -1.5708 ||
		 dtS.p_source->latitude > 1.5708) {
		tDelay = 0.0;
	      } else {
		LALTimeDelayFromEarthCenter( status->statusPtr,&tDelay, &dtS );
		CHECKSTATUSPTR (status);
	      }

	      {
		LALGPSandAcc wi;
		wi.gps = t0;
		wi.accuracy = LALLEAPSEC_STRICT;
		
#ifdef DEBUGBURST
		fprintf(stderr,"APPLYSEARCH: before LALComputeDetAMResponse\n");
		fflush(NULL);
#endif

		if(dtS.p_source->longitude < 0.0 ||
		   dtS.p_source->longitude > 6.2832 ||
		   dtS.p_source->latitude < -1.5708 ||
		   dtS.p_source->latitude > 1.5708) {
		  F.plus = F.cross = 1.0;
		} else {
		  LALComputeDetAMResponse(status->statusPtr,&F,&dAs,&wi);
		  CHECKSTATUSPTR (status);
		}
	      }

	      {
		REAL8 dt = tDelay;

		if(tDelay > 0.0) {
		  t0.gpsSeconds += (INT4)floor(dt);
		  t0.gpsNanoSeconds += (INT4)floor(1E9 * (dt - floor(dt)));
		} else {
		  dt = fabs(dt);
		  t0.gpsSeconds -= (INT4)floor(dt);
		  t0.gpsNanoSeconds -= (INT4)floor(1E9 * (dt - floor(dt)));
		}

		if(t0.gpsNanoSeconds >= 1000000000) {
		  (t0.gpsSeconds)++;
		  (t0.gpsNanoSeconds)-=1000000000;
		}

		if(t0.gpsNanoSeconds < 0) {
		  (t0.gpsSeconds)--;
		  (t0.gpsNanoSeconds) = 1000000000 + t0.gpsNanoSeconds;
		}
		
	      }	    
	      
	      sig.epoch = t0;
	      
	      bzero(sig.data->data, sig.data->length * sizeof(REAL4));
	    
	      for(j=0;j<sig.data->length;j++) {
		sig.data->data[j] += wAmp * (F.plus * params->wave_pc[wid-1].data->data[j] + F.cross * params->wave_pc[wid-1].data->data[j + sig.data->length]);
	      }
	    

#ifdef DEBUGBURST
	      fprintf(stderr,"APPLYSEARCH: before LALSSInjectTimeSeries\n");
	      fflush(NULL);
#endif

	      {
		REAL4TimeSeries ats;
		REAL4Vector av;
		ats.deltaT = params->data.data->deltaT;
		ats.f0 = params->data.data->f0;
		ats.epoch = params->data.data->epoch;
		ats.data = &av;
		av.length = params->data.data->data->vectorLength;
		av.data = params->data.data->data->data;

		LALSSInjectTimeSeries(status->statusPtr, &ats, &sig);
		CHECKSTATUSPTR (status);
	      }

#ifdef DEBUGBURST
	      fprintf(stderr,"APPLYSEARCH: after LALSSInjectTimeSeries\n");
	      fflush(NULL);
#endif

	    }
	  }

	  sprintf(comment,"%s)",comment);

	  LALFree(NDT);

	  LALDestroyVector(status->statusPtr, &(sig.data));
	  CHECKSTATUSPTR (status);
	}



	/* pack burst parameters */
	bparamsptr = &bparams;
	ETGparams = params->ETGparams.next;
	i=0;
	while(ETGparams) {

	  bptr = ETGparams->param.next;
	  for(j=0; j<Pid[i]; j++) {
	    bptr = bptr->next;
	  }

	  if(params->ETGparamsLOCK == 0) {

	    bparamsptr->next = (BurstParameter *)LALCalloc(1,sizeof(BurstParameter));
	    RASSERT(bparamsptr->next, status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
	    bparamsptr = bparamsptr->next;

	    if(bptr->random == 1) {
	      ABORT(status, INITSEARCHH_EIN, INITSEARCHH_MSGEIN);
	    }

	    if(bptr->char_ && !(bptr->real4vector_)) {
	      sprintf(comment,"%s,%s",comment,bptr->char_);

	      bparamsptr->char_ = (CHAR *)LALCalloc(1+strlen(bptr->char_),sizeof(CHAR));
	      RASSERT(bparamsptr->char_, status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
	      strcpy(bparamsptr->char_, bptr->char_);

	    } else {
	      if(bptr->real4_) {
		sprintf(comment,"%s,%g",comment,*(bptr->real4_));
		
		bparamsptr->real4_ = (REAL4 *)LALMalloc(sizeof(REAL4));
		RASSERT(bparamsptr->real4_, status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);

		*(bparamsptr->real4_) = *(bptr->real4_);

	      } else {
		if(bptr->int4_) {
		  sprintf(comment,"%s,%i",comment,*(bptr->int4_));

		  bparamsptr->int4_ = (INT4 *)LALMalloc(sizeof(INT4));
		  RASSERT(bparamsptr->int4_, status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);

		  *(bparamsptr->int4_) = *(bptr->int4_);
		  
		} else {
		  if(bptr->real4vector_) {
		    sprintf(comment,"%s,%s",comment,bptr->char_+1);

		    bparamsptr->real4vector_ = NULL;
		    LALCreateVector(status->statusPtr, &(bparamsptr->real4vector_), bptr->real4vector_->length);
		    CHECKSTATUSPTR (status);

		    memcpy(bparamsptr->real4vector_->data, bptr->real4vector_->data, bptr->real4vector_->length*sizeof(REAL4));

		  } else {
		    sprintf(comment,"%s,$",comment);
		  }
		}
	      }
	    }
	  } else {
	    /* have ETGparamsLOCK ! */
	    BurstParameter *bptr1;

#ifdef DEBUGBURST
	    fprintf(stderr,"SLAVE ETGParams: entering\n");
	    fflush(NULL);
#endif 

	    if(!(bptr->char_)) {
	      ABORT(status, INITSEARCHH_EIN, INITSEARCHH_MSGEIN);
	    }

	    memcpy(&bptr1, bptr->char_, sizeof(BurstParameter *));

	    bptr1 = bptr1->next;

	    while(bptr1) {

	      bparamsptr->next = (BurstParameter *)LALCalloc(1,sizeof(BurstParameter));
	      RASSERT(bparamsptr->next, status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
	      bparamsptr = bparamsptr->next;

	      if(bptr1->random == 1) {
		ABORT(status, INITSEARCHH_EIN, INITSEARCHH_MSGEIN);
	      }

	      if(bptr1->char_ && !(bptr1->real4vector_)) {

#ifdef DEBUGBURST
		fprintf(stderr,"SLAVE ETGParams: str: %s\n",bptr1->char_);
		fflush(NULL);
#endif 

		sprintf(comment,"%s,%s",comment,bptr1->char_);

		bparamsptr->char_ = (CHAR *)LALCalloc(1+strlen(bptr1->char_),sizeof(CHAR));
		RASSERT(bparamsptr->char_, status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
		strcpy(bparamsptr->char_, bptr1->char_);

	      } else {
		if(bptr1->real4_) {

#ifdef DEBUGBURST
		  fprintf(stderr,"SLAVE ETGParams: real4: %g\n",*(bptr1->real4_));
		  fflush(NULL);
#endif 

		  sprintf(comment,"%s,%g",comment,*(bptr1->real4_));
		
		  bparamsptr->real4_ = (REAL4 *)LALMalloc(sizeof(REAL4));
		  RASSERT(bparamsptr->real4_, status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);

		  *(bparamsptr->real4_) = *(bptr1->real4_);

		} else {
		  if(bptr1->int4_) {

#ifdef DEBUGBURST
		    fprintf(stderr,"SLAVE ETGParams: int4: %i\n",*(bptr1->int4_));
		    fflush(NULL);
#endif 

		    sprintf(comment,"%s,%i",comment,*(bptr1->int4_));

		    bparamsptr->int4_ = (INT4 *)LALMalloc(sizeof(INT4));
		    RASSERT(bparamsptr->int4_, status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);

		    *(bparamsptr->int4_) = *(bptr1->int4_);
		  
		  } else {
		    if(bptr1->real4vector_) {
		      sprintf(comment,"%s,%s",comment,bptr1->char_+1);

		      bparamsptr->real4vector_ = NULL;
		      LALCreateVector(status->statusPtr, &(bparamsptr->real4vector_), bptr1->real4vector_->length);
		      CHECKSTATUSPTR (status);

		      memcpy(bparamsptr->real4vector_->data, bptr1->real4vector_->data, bptr1->real4vector_->length*sizeof(REAL4));

		    } else {
		      sprintf(comment,"%s,$",comment);
		    }
		  }
		}
	      } 

	      bptr1 = bptr1->next;
	    }
	  }

	  ETGparams = ETGparams->next;
	  i++;
	}

	bparamsptr->next = NULL;

#ifdef DEBUGBURST
	fprintf(stderr,"%s\n",comment);
	fflush(NULL);
#endif

	/* run ETG */

	{

	  void (*oldSegFault)(int), (*oldSigBus)(int);

	  oldSegFault = signal(SIGSEGV, BurstSigHandler);
	  oldSigBus = signal(SIGBUS, BurstSigHandler);

#ifdef DEBUGBURST
	  fprintf(stderr,"Launching ETG function...\n");
	  fflush(NULL);

	  {
	    clock_t tic = clock(),
	      toc;
	    REAL4 *tdata = (REAL4 *)LALMalloc(params->data.data->data->vectorLength * sizeof(REAL4));

	    memcpy(tdata, params->data.data->data->data, params->data.data->data->vectorLength * sizeof(REAL4));

#endif

	  params->ETGfun(status->statusPtr,
			 &boutput,
			 params->data.data,
			 &bparams);
	  CHECKSTATUSPTR (status);

#ifdef DEBUGBURST
	  toc = clock();

	  fprintf(stderr,"Time in ETG function: %g s\n",(REAL8)(toc-tic)/(REAL8)CLOCKS_PER_SEC);

	  /*
	  {
	    UINT4 nbursts = 0;
	    EventIDColumn *optr = boutput.next;

	    fprintf(stderr,"*********************************************\n");
	    while(optr) {
	      nbursts++;
	      fprintf(stderr,"%s %s\t%s\t%u\t%u\t%g\t%g\t%g\n",optr->snglBurstTable->ifo, optr->snglBurstTable->search, optr->snglBurstTable->channel, optr->snglBurstTable->start_time.gpsSeconds, optr->snglBurstTable->start_time.gpsNanoSeconds, optr->snglBurstTable->duration, optr->snglBurstTable->central_freq, optr->snglBurstTable->bandwidth);
	      optr = optr->next;
	    }

	    fprintf(stderr,"%u bursts\n",nbursts);
	    fprintf(stderr,"*********************************************\n");
	  }
	  */

	  fflush(NULL);
#endif

	  signal(SIGSEGV, oldSegFault);
	  signal(SIGBUS, oldSigBus);

#ifdef DEBUGBURST

	  if(memcmp(tdata, params->data.data->data->data, params->data.data->data->vectorLength * sizeof(REAL4)) != 0) {
	    ABORT(status, INITSEARCHH_EETGDATA, INITSEARCHH_MSGEETGDATA);
	  }

	  LALFree(tdata);
	  }

	  fprintf(stderr,"ETG function is done...\n");
	  fflush(NULL);
#endif

	}

	if(!(params->noLALBurstOutput)) {
#ifdef DEBUGBURST
	  {
	    clock_t tic = clock(),
	      toc;
#endif

	    /* run standardized output function */
	    LALBurstOutput(status->statusPtr, &stdOutput, &boutput, &(params->oparams));
	    CHECKSTATUSPTR (status);

#ifdef DEBUGBURST
	    toc = clock();
	    fprintf(stderr,"Time in LALBurstOutput function: %g s\n",(REAL8)(toc-tic)/(REAL8)CLOCKS_PER_SEC);
	  }
#endif
	} else {
	  
	  stdOutput.next = boutput.next;

	}

	/* remove injected waveform */
	if(wid) {
	  memcpy(params->data.data->data->data, wbuf, params->data.data->data->vectorLength * sizeof(REAL4));
	  LALFree(wbuf);

	  if(!(params->noLALBurstOutput)) {
	    LALBurstOutput(status->statusPtr, NULL, NULL, NULL);
	    CHECKSTATUSPTR (status);
	  }
	}

	/* prepare output for LDAS */
	switch(params->outputMethod) {

	case 0:
	  BuildOutput(status->statusPtr, output, &stdOutput, comment, params->data.data->epoch.gpsSeconds, params->data.data->epoch.gpsNanoSeconds, params->data.data->deltaT * params->data.data->data->vectorLength, numSlaves, zerobid);
	  CHECKSTATUSPTR (status);
	  break;

	case 1:
	  BuildOutputBinary(status->statusPtr, output, &stdOutput, comment, params->data.data->epoch.gpsSeconds, params->data.data->epoch.gpsNanoSeconds, zerobid);
	  CHECKSTATUSPTR (status);
	  break;

	case 2:
	  BuildOutputFileBinary(status->statusPtr, params->dsoOutput, &stdOutput, comment, params->data.data->epoch.gpsSeconds, params->data.data->epoch.gpsNanoSeconds,&initExchParams);
	  CHECKSTATUSPTR (status);
	  break;

	default:
	  ABORT(status,INITSEARCHH_EIN,INITSEARCHH_MSGEIN);
	}

	/* clean up */
	LALFree(Pid);

	{
	  EventIDColumn *optr;

	  optr = boutput.next;
	  while(optr) {
	    EventIDColumn *optr2 = optr->next;

	    if(optr->snglTransdataTable) {
	      if(optr->snglTransdataTable->trans_data) {
		LALFree(optr->snglTransdataTable->trans_data);
	      }
	      LALFree(optr->snglTransdataTable);
	    }

	    if(optr->snglBurstTable) {
	      LALFree(optr->snglBurstTable);
	    }

	    LALFree(optr);
	    optr = optr2;
	  }

	  if(!(params->noLALBurstOutput)) {
	    optr = stdOutput.next;
	    while(optr) {
	      EventIDColumn *optr2 = optr->next;

	      /* no need to free trans_data since pointer copy of boutput */

	      if(optr->snglBurstTable) {
		LALFree(optr->snglBurstTable);
	      }

	      LALFree(optr);
	      optr = optr2;
	    }
	  }
	}

	bparamsptr = bparams.next;
	while(bparamsptr) {
	  BurstParameter *bptr = bparamsptr->next;

	  if(bparamsptr->char_) { 
	    LALFree(bparamsptr->char_); 
	  } 
	  if(bparamsptr->int4_) { 
	    LALFree(bparamsptr->int4_); 
	  } 
	  if(bparamsptr->real4_) { 
	    LALFree(bparamsptr->real4_); 
	  } 
	  if(bparamsptr->real4vector_) { 
	    LALDestroyVector(status->statusPtr, &(bparamsptr->real4vector_)); 
	    CHECKSTATUSPTR (status); 
	  }

	  LALFree(bparamsptr);
	  bparamsptr = bptr;
	}

      } /* end loop over u4v */


      if(!got_wid && !(params->noLALBurstOutput)) {
	LALBurstOutput(status->statusPtr, NULL, NULL, NULL);
	CHECKSTATUSPTR (status);
      }

      LALDestroyRandomParams(status->statusPtr, &rpar);
      CHECKSTATUSPTR (status);

      LALInitializeExchange (status->statusPtr, &thisExch, 
			     &exchFinished, &initExchParams);
      CHECKSTATUSPTR (status);
      
      LALExchangeUINT4(status->statusPtr, &(u4v->length), thisExch);
      CHECKSTATUSPTR (status);

      LALFinalizeExchange (status->statusPtr, &thisExch);
      CHECKSTATUSPTR (status);

      LALU4DestroyVector(status->statusPtr, &u4v);
      CHECKSTATUSPTR (status);

      output->fracRemaining = 0;
      output->notFinished   = 1;
    }

  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}





/***********************************************************************/
void BuildOutput(LALStatus *status,
		 LALSearchOutput *output,
		 EventIDColumn     *burstEventList,
		 CHAR               *tag,
		 UINT4 t0Sec,
		 UINT4 t0NanoSec,
		 REAL8 tDuration,
		 UINT4 numSlaves,
		 UINT4 zerobid
		 )
{

#ifndef WORDS_BIGENDIAN
  static void endian_swap(char * pdata, int dsize, int nelements);
#endif

  static UINT4 oldPos = 0;

  CHAR tabname[32]="sngl_burst";

  outPut               *outputArray;
  dataBase             *prev;
  dataBase             *dbEntry;

  UINT4 i, numOutput, ono;
  EventIDColumn *sbtptrG;
  SnglBurstTable *sbtptr;

  INITSTATUS (status, "BuildOutput", APPLYSEARCHC);
  ATTATCHSTATUSPTR (status);

  if(oldPos < zerobid) {
    oldPos = zerobid;
  }

  /*
  if(output->numOutput) {
    ABORT(status, INITSEARCHH_EIN, INITSEARCHH_MSGEIN);
  }
  */

  sbtptrG = burstEventList->next;
  numOutput = 0;
  while(sbtptrG) {
    numOutput++;
    sbtptrG = sbtptrG->next;
  }

  if(numOutput > MAXNBURSTS) {
    ABORT(status, INITSEARCHH_ENB, INITSEARCHH_MSGENB);
  }

#ifdef DEBUGBURST
  fprintf(stderr,"Got %u bursts\n",numOutput);
  fflush(NULL);
#endif

  if(!numOutput) {
    DETATCHSTATUSPTR (status);
    RETURN (status);
  }

  ono = output->numOutput;
  (output->numOutput) += numOutput;

#ifdef DEBUGBURST
  fprintf(stderr,"output->numOutput = %u\n",output->numOutput);
  fflush(NULL);
#endif

  outputArray = output->result = (outPut *) LALRealloc(output->result,  output->numOutput * sizeof(outPut) ); 
  RASSERT(output->result, status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);


  /* Events */
  for(i=ono, sbtptrG = burstEventList->next; i<output->numOutput; i++) {

    sbtptr = sbtptrG->snglBurstTable;

    bzero(outputArray + i, sizeof(outPut));

    outputArray[i].indexNumber = 0;
    outputArray[i].search = burst;
    outputArray[i].significant    = 1;
    outputArray[i].states         = NULL;
    outputArray[i].optional       = NULL;

    
    dbEntry = outputArray[i].results = (dataBase *) LALMalloc( sizeof(dataBase) );
    
    dbEntry->previous = NULL;

    /* zeroth entry: event id */
    strcpy( dbEntry->tableName, "sngl_burst");
    sprintf( dbEntry->columnName, "event_id" );

    dbEntry->type       = char_s_ptr;
    dbEntry->numberRows = 1;
    dbEntry->rows.chars = (CHAR *)LALMalloc(128 * sizeof(CHAR));
    sprintf(dbEntry->rows.chars,"sngl_burstgroup:sngl_burst:%u",oldPos);
    dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
    *(dbEntry->rowDimensions) = strlen(dbEntry->rows.chars);
    
    prev = dbEntry;
    
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;


    /* copy  name for IFO */
    strcpy( dbEntry->tableName, tabname );
    sprintf( dbEntry->columnName, "ifo" );

    dbEntry->type       = char_s_ptr;
    dbEntry->numberRows = 1;
    dbEntry->rows.chars = (CHAR *) LALMalloc( LIGOMETA_IFO_MAX * dbEntry->numberRows * sizeof(CHAR) );
    strncpy(dbEntry->rows.chars, sbtptr->ifo, LIGOMETA_IFO_MAX);
    dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
    *(dbEntry->rowDimensions) = strlen(sbtptr->ifo);

    prev = dbEntry;
    
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;

    strcpy( dbEntry->tableName, tabname );  
    sprintf( dbEntry->columnName, "search" );

    dbEntry->type       = char_s_ptr;
    dbEntry->numberRows = 1;
    dbEntry->rows.chars = (CHAR *) LALMalloc( LIGOMETA_SEARCH_MAX * dbEntry->numberRows * sizeof(CHAR) );
    strncpy(dbEntry->rows.chars, sbtptr->search, LIGOMETA_SEARCH_MAX);
    dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
    *(dbEntry->rowDimensions) = strlen(sbtptr->search);

    prev = dbEntry;

    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;

    strcpy( dbEntry->tableName, tabname );  
    sprintf( dbEntry->columnName, "channel" );

    dbEntry->type       = char_s_ptr;
    dbEntry->numberRows = 1;
    dbEntry->rows.chars = (CHAR *) LALMalloc( LIGOMETA_CHANNEL_MAX * dbEntry->numberRows * sizeof(CHAR) );
    strncpy(dbEntry->rows.chars, sbtptr->channel, LIGOMETA_CHANNEL_MAX);
    dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
    *(dbEntry->rowDimensions) = strlen(sbtptr->channel);

    prev = dbEntry;

    /* entry: start_time */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;
    
    strcpy( dbEntry->tableName, tabname );
    sprintf( dbEntry->columnName, "start_time" );

    dbEntry->type       = int_4s;
    dbEntry->numberRows = 1;
    dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.int4s) = sbtptr->start_time.gpsSeconds;

    prev = dbEntry;

    /* entry: start_time_ns */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;

    strcpy( dbEntry->tableName, tabname );
    sprintf( dbEntry->columnName, "start_time_ns" );

    dbEntry->type       = int_4s;
    dbEntry->numberRows = 1;
    dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.int4s) = sbtptr->start_time.gpsNanoSeconds;

    prev = dbEntry;

    /* entry: duration */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;

    strcpy( dbEntry->tableName, tabname );
    sprintf( dbEntry->columnName, "duration" );

    dbEntry->type       = real_4;
    dbEntry->numberRows = 1;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.real4) = sbtptr->duration;

    prev = dbEntry;

    
    /* entry: central frequency */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;

    strcpy( dbEntry->tableName, tabname );
    sprintf( dbEntry->columnName, "central_freq" );
  
    dbEntry->type       = real_4;
    dbEntry->numberRows = 1;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.real4) = sbtptr->central_freq;

    prev = dbEntry;

    
    /* entry: bandwidth  */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;

    strcpy( dbEntry->tableName, tabname );
    sprintf( dbEntry->columnName, "bandwidth" );

    dbEntry->type       = real_4;
    dbEntry->numberRows = 1;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.real4) = sbtptr->bandwidth;

    prev = dbEntry;


    /* entry: snr */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;

    strcpy( dbEntry->tableName, tabname );
    sprintf( dbEntry->columnName, "snr" );
   
    dbEntry->type       = real_4;
    dbEntry->numberRows = 1;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.real4) = sbtptr->snr;

    prev = dbEntry;


    /* entry: amplitude */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev; 

    strcpy( dbEntry->tableName, tabname );
    sprintf( dbEntry->columnName, "amplitude" );
      
    dbEntry->type       = real_4;
    dbEntry->numberRows = 1;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.real4) = sbtptr->amplitude;


    prev = dbEntry;

    /* entry: confidence */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev; 

    strcpy( dbEntry->tableName, tabname );
    sprintf( dbEntry->columnName, "confidence" );
      
    dbEntry->type       = real_4;
    dbEntry->numberRows = 1;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.real4) = sbtptr->confidence;



    /** TAG **/
    {
      UINT4 tdl;

#ifndef WORDS_BIGENDIAN
      endian_swap((char *)(tag), sizeof(UCHAR), strlen(tag));
#endif

      prev = dbEntry;
      
      /* sngl_transdata: event_id */
      dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      dbEntry->previous = prev;

      strcpy( dbEntry->tableName, "sngl_mime");
      sprintf( dbEntry->columnName, "event_id" );

      dbEntry->type       = char_s_ptr;
      dbEntry->numberRows = 1;
      dbEntry->rows.chars = (CHAR *)LALMalloc(128 * sizeof(CHAR));

      sprintf(dbEntry->rows.chars,"sngl_burstgroup:sngl_burst:%u",oldPos);
      dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
      *(dbEntry->rowDimensions) = strlen(dbEntry->rows.chars);

      prev = dbEntry;

      /* copy IFO */
      dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      dbEntry->previous = prev;
      
      strcpy( dbEntry->tableName, "sngl_mime");
      sprintf( dbEntry->columnName, "ifo" );
      
      dbEntry->type       = char_s_ptr;
      dbEntry->numberRows = 1;
      dbEntry->rows.chars = (CHAR *) LALMalloc( LIGOMETA_IFO_MAX * dbEntry->numberRows * sizeof(CHAR) );
      strncpy(dbEntry->rows.chars, sbtptr->ifo, LIGOMETA_IFO_MAX);
      dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
      *(dbEntry->rowDimensions) = strlen(sbtptr->ifo);


      prev = dbEntry;

      /* length */
      dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      dbEntry->previous = prev;
      
      strcpy( dbEntry->tableName, "sngl_mime");
      sprintf( dbEntry->columnName, "mimedata_length");

      dbEntry->type       = int_4s;
      dbEntry->numberRows = 1;
      dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
      dbEntry->rowDimensions = NULL;
      tdl = *(dbEntry->rows.int4s) = strlen(tag);
    
      if(tdl >= 1000000) {
	ABORT( status, INITSEARCHH_EMAXBLOB, INITSEARCHH_MSGEMAXBLOB);
      }

      prev = dbEntry;

      /* data (bigEndian) */
      dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      dbEntry->previous = prev;
      
      strcpy( dbEntry->tableName, "sngl_mime");
      sprintf( dbEntry->columnName, "mimedata");
      
      dbEntry->type       = char_u_ptr;
      dbEntry->numberRows = 1;
      dbEntry->rows.charu = (UCHAR *)LALMalloc((1+tdl) * sizeof(CHAR));
      strncpy(dbEntry->rows.charu, tag, 1+tdl);
      dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
      *(dbEntry->rowDimensions) = tdl;

      prev = dbEntry;
      
      /* data_type */
      dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      dbEntry->previous = prev;
      
      strcpy( dbEntry->tableName, "sngl_mime");
      sprintf( dbEntry->columnName, "mimetype");

      dbEntry->type       = char_s_ptr;
      dbEntry->numberRows = 1;
      dbEntry->rows.chars = (CHAR *) LALMalloc( 7 * sizeof(CHAR) );
      strcpy(dbEntry->rows.chars,"string");
      dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
      *(dbEntry->rowDimensions) = 6;

      prev = dbEntry;
      
      /* event_table */
      dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      dbEntry->previous = prev;
      
      strcpy( dbEntry->tableName, "sngl_mime");
      sprintf( dbEntry->columnName, "event_table");

      dbEntry->type       = char_s_ptr;
      dbEntry->numberRows = 1;
      dbEntry->rows.chars = (CHAR *) LALMalloc( (1+strlen(tabname)) * sizeof(CHAR) );
      strcpy(dbEntry->rows.chars,tabname);
      dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
      *(dbEntry->rowDimensions) = strlen(tabname);

    }


    /***********************************************************/
    /*************************BLOB******************************/
    if(sbtptrG->snglTransdataTable)
    {
      UINT4 tdl;
      SnglTransdataTable *trans_data = sbtptrG->snglTransdataTable;

      prev = dbEntry;
      
      /* sngl_transdata: event_id */
      dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      dbEntry->previous = prev;

      strcpy( dbEntry->tableName, "sngl_transdata");
      sprintf( dbEntry->columnName, "event_id" );

      dbEntry->type       = char_s_ptr;
      dbEntry->numberRows = 1;
      dbEntry->rows.chars = (CHAR *)LALMalloc(128 * sizeof(CHAR));

      sprintf(dbEntry->rows.chars,"sngl_burstgroup:sngl_burst:%u",oldPos);
      dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
      *(dbEntry->rowDimensions) = strlen(dbEntry->rows.chars);

      prev = dbEntry;
      
      
      /* sngl_transdata: event_table */
      dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      dbEntry->previous = prev;
      
      strcpy( dbEntry->tableName, "sngl_transdata");
      sprintf( dbEntry->columnName, "event_table" );
      
      dbEntry->type       = char_s_ptr;
      dbEntry->numberRows = 1;
      dbEntry->rows.chars = (CHAR *) LALMalloc((1 + strlen(tabname)) * dbEntry->numberRows * sizeof(CHAR) );
      strcpy(dbEntry->rows.chars, tabname);
      dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
      *(dbEntry->rowDimensions) = strlen(tabname);
      
      prev = dbEntry;

      /* copy IFO */
      dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      dbEntry->previous = prev;
      
      strcpy( dbEntry->tableName, "sngl_transdata");
      sprintf( dbEntry->columnName, "ifo" );
      
      dbEntry->type       = char_s_ptr;
      dbEntry->numberRows = 1;
      dbEntry->rows.chars = (CHAR *) LALMalloc( LIGOMETA_IFO_MAX * dbEntry->numberRows * sizeof(CHAR) );
      strncpy(dbEntry->rows.chars, trans_data->ifo, LIGOMETA_IFO_MAX);
      dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
      *(dbEntry->rowDimensions) = strlen(trans_data->ifo);
      
      prev = dbEntry;
      
      /* transdata_name */
      dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      dbEntry->previous = prev;
      
      strcpy( dbEntry->tableName, "sngl_transdata");
      sprintf( dbEntry->columnName, "transdata_name");
      
      dbEntry->type       = char_s_ptr;
      dbEntry->numberRows = 1;
      dbEntry->rows.chars = (CHAR *) LALMalloc( LIGOMETA_TRANSDATA_NAME_MAX * dbEntry->numberRows * sizeof(CHAR) );
      strncpy(dbEntry->rows.chars,trans_data->name,LIGOMETA_TRANSDATA_NAME_MAX);
      dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
      *(dbEntry->rowDimensions) = strlen(trans_data->name);
      
      prev = dbEntry;
      
      /* dimensionality */
      dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      dbEntry->previous = prev;

      strcpy( dbEntry->tableName, "sngl_transdata");
      sprintf( dbEntry->columnName, "dimensions");
      
      dbEntry->type       = int_4s;
      dbEntry->numberRows = 1;
      dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
      dbEntry->rowDimensions = NULL;
      *(dbEntry->rows.int4s) = trans_data->dimensions; 
      
      prev = dbEntry;
      
      /* number of x bins */
      dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      dbEntry->previous = prev;

      strcpy( dbEntry->tableName, "sngl_transdata");
      sprintf( dbEntry->columnName, "x_bins");
      
      dbEntry->type       = int_4s;
      dbEntry->numberRows = 1;
      dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
      dbEntry->rowDimensions = NULL;
      *(dbEntry->rows.int4s) = trans_data->x_bins;
      
      prev = dbEntry;

      /* x limits */
      dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      dbEntry->previous = prev;

      strcpy( dbEntry->tableName, "sngl_transdata");
      sprintf( dbEntry->columnName, "x_start");
      dbEntry->type       = real_8;
      dbEntry->numberRows = 1;
      dbEntry->rows.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
      dbEntry->rowDimensions = NULL;
      *(dbEntry->rows.real8) = trans_data->x_start;
      
      prev = dbEntry;

      dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      dbEntry->previous = prev;

      strcpy( dbEntry->tableName, "sngl_transdata");
      sprintf( dbEntry->columnName, "x_end");
      dbEntry->type       = real_8;
      dbEntry->numberRows = 1;
      dbEntry->rows.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
      dbEntry->rowDimensions = NULL;
      *(dbEntry->rows.real8) = trans_data->x_end;
      
      prev = dbEntry;

      /* x units */
      dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      dbEntry->previous = prev;

      strcpy( dbEntry->tableName, "sngl_transdata");
      sprintf( dbEntry->columnName, "x_units");
      
      dbEntry->type       = char_s_ptr;
      dbEntry->numberRows = 1;
      dbEntry->rows.chars = (CHAR *) LALMalloc( LIGOMETA_TRANSDATA_UNITS_MAX * dbEntry->numberRows * sizeof(CHAR) );
      strncpy(dbEntry->rows.chars, trans_data->x_units,LIGOMETA_TRANSDATA_UNITS_MAX);
      dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
      *(dbEntry->rowDimensions) = strlen(trans_data->x_units);
      
      prev = dbEntry;


      /* number of y bins */
      dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      dbEntry->previous = prev;
      
      strcpy( dbEntry->tableName, "sngl_transdata");
      sprintf( dbEntry->columnName, "y_bins");
      
      dbEntry->type       = int_4s;
      dbEntry->numberRows = 1;
      dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
      dbEntry->rowDimensions = NULL;
      *(dbEntry->rows.int4s) = trans_data->y_bins;
      
      prev = dbEntry;
      
      /* frequency limits */
      dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      dbEntry->previous = prev;
      
      strcpy( dbEntry->tableName, "sngl_transdata");
      sprintf( dbEntry->columnName, "y_start");
      dbEntry->type       = real_8;
      dbEntry->numberRows = 1;
      dbEntry->rows.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
      dbEntry->rowDimensions = NULL;
      *(dbEntry->rows.real8) = trans_data->y_start;
      
      prev = dbEntry;
      
      dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      dbEntry->previous = prev;
      
      strcpy( dbEntry->tableName, "sngl_transdata");
      sprintf( dbEntry->columnName, "y_end");
      dbEntry->type       = real_8;
      dbEntry->numberRows = 1;
      dbEntry->rows.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
      dbEntry->rowDimensions = NULL;
      *(dbEntry->rows.real8) = trans_data->y_end;
      
      prev = dbEntry;
      
      /* units */
      dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      dbEntry->previous = prev;
      
      strcpy( dbEntry->tableName, "sngl_transdata");
      sprintf( dbEntry->columnName, "y_units");
      
      dbEntry->type       = char_s_ptr;
      dbEntry->numberRows = 1;
      dbEntry->rows.chars = (CHAR *) LALMalloc( LIGOMETA_TRANSDATA_UNITS_MAX * dbEntry->numberRows * sizeof(CHAR) );
      strncpy(dbEntry->rows.chars,trans_data->y_units,LIGOMETA_TRANSDATA_UNITS_MAX);
      dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
      *(dbEntry->rowDimensions) = strlen(trans_data->y_units);
      
      prev = dbEntry;
      
      /* data_type */
      dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      dbEntry->previous = prev;
      
      strcpy( dbEntry->tableName, "sngl_transdata");
      sprintf( dbEntry->columnName, "data_type");

      dbEntry->type       = char_s_ptr;
      dbEntry->numberRows = 1;
      dbEntry->rows.chars = (CHAR *) LALMalloc( LIGOMETA_TRANSDATA_DATA_MAX * dbEntry->numberRows * sizeof(CHAR) );
      strncpy(dbEntry->rows.chars,trans_data->data_type,LIGOMETA_TRANSDATA_DATA_MAX);
      dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
      *(dbEntry->rowDimensions) = strlen(trans_data->data_type);

      prev = dbEntry;

      /* data_units */
      dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      dbEntry->previous = prev;

      strcpy( dbEntry->tableName, "sngl_transdata");
      sprintf( dbEntry->columnName, "data_units");

      dbEntry->type       = char_s_ptr;
      dbEntry->numberRows = 1;
      dbEntry->rows.chars = (CHAR *) LALMalloc( LIGOMETA_TRANSDATA_UNITS_MAX * dbEntry->numberRows * sizeof(CHAR) );
      strncpy(dbEntry->rows.chars,trans_data->data_units,LIGOMETA_TRANSDATA_UNITS_MAX);
      dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
      *(dbEntry->rowDimensions) = strlen(trans_data->data_units);

      prev = dbEntry;

      /* length */
      dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      dbEntry->previous = prev;
      
      strcpy( dbEntry->tableName, "sngl_transdata");
      sprintf( dbEntry->columnName, "transdata_length");

      dbEntry->type       = int_4s;
      dbEntry->numberRows = 1;
      dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
      dbEntry->rowDimensions = NULL;
      tdl = *(dbEntry->rows.int4s) = trans_data->transdata_length;
    
      if(tdl >= 1000000) {
	ABORT( status, INITSEARCHH_EMAXBLOB, INITSEARCHH_MSGEMAXBLOB);
      }

      prev = dbEntry;

      /* data (bigEndian) */
      dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      dbEntry->previous = prev;
      
      strcpy( dbEntry->tableName, "sngl_transdata");
      sprintf( dbEntry->columnName, "transdata");
      
      dbEntry->type       = char_u_ptr;
      dbEntry->numberRows = 1;
      dbEntry->rows.charu = trans_data->trans_data;
      dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
      *(dbEntry->rowDimensions) = tdl;

    } /* end of BLOB */

    oldPos++;

    /* LAST entry */
    dbEntry->next = NULL;

    sbtptrG = sbtptrG->next;

  } /* loop over nclusters */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}


void BurstSigHandler(int s) {

  switch(s) {
  case SIGSEGV:
    fprintf(stderr,"Got SIGSEGV from ETG function!\n");
    break;
  case SIGBUS:
    fprintf(stderr,"Got SIGBUS from ETG function!\n");
    break;
  default:
    fprintf(stderr,"Got unknown signal %i from ETG function\n",s);
    break;
  }

  fflush(NULL);

#ifdef DEBUGBURST
  exit(-1);
#endif

  ABORT(gStatus,INITSEARCHH_EETG,INITSEARCHH_MSGEETG);

}


#ifndef WORDS_BIGENDIAN
static void endian_swap(char * pdata, int dsize, int nelements)

{

        int i,j,indx;
        char tempbyte;

        if (dsize <= 1) return;

        for (i=0; i<nelements; i++)
        {
                indx = dsize;
                for (j=0; j<dsize/2; j++)
                {
                        tempbyte = pdata[j];
                        indx = indx - 1;
                        pdata[j] = pdata[indx];
                        pdata[indx] = tempbyte;
                }

                pdata = pdata + dsize;
        }

        return;

}
#endif




/***********************************************************************/
void BuildOutputBinary(LALStatus *status,
		       LALSearchOutput *output,
		       EventIDColumn     *burstEventList,
		       CHAR               *tag,
		       UINT4 t0Sec,
		       UINT4 t0NanoSec,
		       UINT4 zerobid
		       )
{

#ifndef WORDS_BIGENDIAN
  static void endian_swap(char * pdata, int dsize, int nelements);
#endif

  static UINT4 oldPos = 0;

  CHAR tabname[32]="sngl_burst";
  
  outPut               *outputArray;
  dataBase             *prev;
  dataBase             *dbEntry;

  UINT4 i, numOutput, ono;
  SnglBurstTable *sbtptr;
  EventIDColumn *sbtptrG;

  UCHAR *data;
  UINT4 datalength, size1, tdl, datalength_;
  CHAR ifo[LIGOMETA_IFO_MAX];
  CHAR name[LIGOMETA_SEARCH_MAX];

  size_t off;

  INITSTATUS (status, "BuildOutput", APPLYSEARCHC);
  ATTATCHSTATUSPTR (status);

  if(oldPos < zerobid) {
    oldPos = zerobid;
  }

  /*
  if(output->numOutput) {
    ABORT(status, INITSEARCHH_EIN, INITSEARCHH_MSGEIN);
  }
  */

  sbtptrG = burstEventList->next;
  numOutput = 0;
  while(sbtptrG) {
    numOutput++;
    sbtptrG = sbtptrG->next;
  }

#ifdef DEBUGBURST
  fprintf(stderr,"Got %u bursts\n",numOutput);
  fflush(NULL);
#endif

  if(!numOutput) {
    DETATCHSTATUSPTR (status);
    RETURN (status);
  }

  size1 = (LIGOMETA_IFO_MAX + LIGOMETA_SEARCH_MAX + LIGOMETA_CHANNEL_MAX) * sizeof(CHAR) + 2*sizeof(INT4) + 6 * sizeof(REAL4);
  datalength_ = datalength = numOutput * size1 + sizeof(UINT4) + strlen(tag) + 1;
  data = (UCHAR *)LALMalloc( datalength );
  if(!data) {
    ABORT(status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
  }

#ifndef WORDS_BIGENDIAN
  endian_swap((char *)(tag), sizeof(UCHAR), strlen(tag));
#endif
  strcpy(data,tag);

  memcpy(data + strlen(tag) + 1, &numOutput, sizeof(UINT4));
#ifndef WORDS_BIGENDIAN
  endian_swap((char *)(data + strlen(tag) + 1), sizeof(UINT4), 1);
#endif

  off = sizeof(UINT4) + strlen(tag) + 1;

  i = 0;
  sbtptrG = burstEventList->next;
  strncpy(ifo,burstEventList->next->snglBurstTable->ifo,LIGOMETA_IFO_MAX);
  strncpy(name,burstEventList->next->snglBurstTable->search,LIGOMETA_SEARCH_MAX);
  while(sbtptrG) {
    UCHAR *p1 = data + i * size1 + off,
      *pst, *pr4;

    if(sbtptrG->snglTransdataTable) {
      ABORT(status,INITSEARCHH_EUI,INITSEARCHH_MSGEUI);
    }

    sbtptr = sbtptrG->snglBurstTable;

    memcpy(p1, sbtptr->ifo,LIGOMETA_IFO_MAX * sizeof(CHAR));

    p1 += LIGOMETA_IFO_MAX * sizeof(CHAR);
    memcpy(p1, sbtptr->search,LIGOMETA_SEARCH_MAX * sizeof(CHAR));

    p1 += LIGOMETA_SEARCH_MAX * sizeof(CHAR);
    memcpy(p1, sbtptr->channel, LIGOMETA_CHANNEL_MAX * sizeof(CHAR));

    pst = p1 + LIGOMETA_CHANNEL_MAX * sizeof(CHAR);
    memcpy(pst, &(sbtptr->start_time.gpsSeconds), sizeof(INT4));
    memcpy(pst + sizeof(INT4), &(sbtptr->start_time.gpsNanoSeconds), sizeof(INT4));

    pr4 = pst + 2*sizeof(INT4);
    memcpy(pr4, &(sbtptr->duration), sizeof(REAL4));

    p1 = pr4 + sizeof(REAL4);
    memcpy(p1, &(sbtptr->central_freq), sizeof(REAL4));

    p1 += sizeof(REAL4);
    memcpy(p1, &(sbtptr->bandwidth), sizeof(REAL4));

    p1 += sizeof(REAL4);
    memcpy(p1, &(sbtptr->amplitude), sizeof(REAL4));
    
    p1 += sizeof(REAL4);
    memcpy(p1, &(sbtptr->snr), sizeof(REAL4));

    p1 += sizeof(REAL4);
    memcpy(p1, &(sbtptr->confidence), sizeof(REAL4));

#ifndef WORDS_BIGENDIAN
    endian_swap((char *)(pst), sizeof(INT4), 2);
    endian_swap((char *)(pr4), sizeof(REAL4), 6);
#endif

    sbtptrG = sbtptrG->next;
    i++;
  }

  ono = output->numOutput;
  (output->numOutput) += 1 + datalength / MAXBLOB;

#ifdef DEBUGBURST
  fprintf(stderr,"output->numOutput = %u\n",output->numOutput);
  fflush(NULL);
#endif

  outputArray = output->result = (outPut *) LALRealloc(output->result,  output->numOutput * sizeof(outPut) ); 
  RASSERT(output->result, status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);


  /* Events */
  for(i=ono; i<output->numOutput; i++) {

    bzero(outputArray + i, sizeof(outPut));

    outputArray[i].indexNumber = 0;
    outputArray[i].search = burst;
    outputArray[i].significant    = 1;
    outputArray[i].states         = NULL;
    outputArray[i].optional       = NULL;

    
    dbEntry = outputArray[i].results = (dataBase *) LALMalloc( sizeof(dataBase) );
    
    dbEntry->previous = NULL;

    /* sngl_transdata: event_id */
    strcpy( dbEntry->tableName, "sngl_transdata");
    sprintf( dbEntry->columnName, "event_id" );

    dbEntry->type       = char_s_ptr;
    dbEntry->numberRows = 1;
    dbEntry->rows.chars = (CHAR *)LALMalloc(128 * sizeof(CHAR));

    sprintf(dbEntry->rows.chars,"sngl_burstgroup:sngl_burst:%u",oldPos);
    dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
    *(dbEntry->rowDimensions) = strlen(dbEntry->rows.chars);

    prev = dbEntry;
      
      
    /* sngl_transdata: event_table */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;
      
    strcpy( dbEntry->tableName, "sngl_transdata");
    sprintf( dbEntry->columnName, "event_table" );
      
    dbEntry->type       = char_s_ptr;
    dbEntry->numberRows = 1;
    dbEntry->rows.chars = (CHAR *) LALMalloc((1 + strlen(tabname)) * dbEntry->numberRows * sizeof(CHAR) );
    strcpy(dbEntry->rows.chars, tabname);
    dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
    *(dbEntry->rowDimensions) = strlen(tabname);
      
    prev = dbEntry;

    /* copy IFO */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;
      
    strcpy( dbEntry->tableName, "sngl_transdata");
    sprintf( dbEntry->columnName, "ifo" );
      
    dbEntry->type       = char_s_ptr;
    dbEntry->numberRows = 1;
    dbEntry->rows.chars = (CHAR *) LALMalloc( LIGOMETA_IFO_MAX * dbEntry->numberRows * sizeof(CHAR) );
    strncpy(dbEntry->rows.chars, ifo, LIGOMETA_IFO_MAX);
    dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
    *(dbEntry->rowDimensions) = strlen(ifo);
      
    prev = dbEntry;
      
    /* transdata_name */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;
      
    strcpy( dbEntry->tableName, "sngl_transdata");
    sprintf( dbEntry->columnName, "transdata_name");
      
    dbEntry->type       = char_s_ptr;
    dbEntry->numberRows = 1;
    dbEntry->rows.chars = (CHAR *) LALMalloc( LIGOMETA_TRANSDATA_NAME_MAX * dbEntry->numberRows * sizeof(CHAR) );
    strncpy(dbEntry->rows.chars,name,LIGOMETA_TRANSDATA_NAME_MAX);
    dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
    *(dbEntry->rowDimensions) = strlen(name);
      
    prev = dbEntry;
      
    /* dimensionality */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;

    strcpy( dbEntry->tableName, "sngl_transdata");
    sprintf( dbEntry->columnName, "dimensions");
      
    dbEntry->type       = int_4s;
    dbEntry->numberRows = 1;
    dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.int4s) = datalength_;
      
    prev = dbEntry;
      
    /* number of x bins */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;

    strcpy( dbEntry->tableName, "sngl_transdata");
    sprintf( dbEntry->columnName, "x_bins");
      
    dbEntry->type       = int_4s;
    dbEntry->numberRows = 1;
    dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.int4s) = t0Sec;
      
    prev = dbEntry;

    /* x limits */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;

    strcpy( dbEntry->tableName, "sngl_transdata");
    sprintf( dbEntry->columnName, "x_start");
    dbEntry->type       = real_8;
    dbEntry->numberRows = 1;
    dbEntry->rows.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.real8) = 0.0;
      
    prev = dbEntry;

    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;

    strcpy( dbEntry->tableName, "sngl_transdata");
    sprintf( dbEntry->columnName, "x_end");
    dbEntry->type       = real_8;
    dbEntry->numberRows = 1;
    dbEntry->rows.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.real8) = 1.0;
      
    prev = dbEntry;

    /* x units */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;

    strcpy( dbEntry->tableName, "sngl_transdata");
    sprintf( dbEntry->columnName, "x_units");
      
    dbEntry->type       = char_s_ptr;
    dbEntry->numberRows = 1;
    dbEntry->rows.chars = (CHAR *) LALMalloc( LIGOMETA_TRANSDATA_UNITS_MAX * dbEntry->numberRows * sizeof(CHAR) );
    sprintf(dbEntry->rows.chars,"n/a");
    dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
    *(dbEntry->rowDimensions) = 3;
      
    prev = dbEntry;

    /* number of y bins */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;
      
    strcpy( dbEntry->tableName, "sngl_transdata");
    sprintf( dbEntry->columnName, "y_bins");
      
    dbEntry->type       = int_4s;
    dbEntry->numberRows = 1;
    dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.int4s) = t0NanoSec;
      
    prev = dbEntry;
      
    /* frequency limits */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;
      
    strcpy( dbEntry->tableName, "sngl_transdata");
    sprintf( dbEntry->columnName, "y_start");
    dbEntry->type       = real_8;
    dbEntry->numberRows = 1;
    dbEntry->rows.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.real8) = 0.0;
      
    prev = dbEntry;
      
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;
      
    strcpy( dbEntry->tableName, "sngl_transdata");
    sprintf( dbEntry->columnName, "y_end");
    dbEntry->type       = real_8;
    dbEntry->numberRows = 1;
    dbEntry->rows.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.real8) = 0.0;
      
    prev = dbEntry;
      
    /* units */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;
      
    strcpy( dbEntry->tableName, "sngl_transdata");
    sprintf( dbEntry->columnName, "y_units");
      
    dbEntry->type       = char_s_ptr;
    dbEntry->numberRows = 1;
    dbEntry->rows.chars = (CHAR *) LALMalloc( 4 * dbEntry->numberRows * sizeof(CHAR) );
    sprintf(dbEntry->rows.chars,"n/a");
    dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
    *(dbEntry->rowDimensions) = 3;
      
    prev = dbEntry;
      
    /* data_type */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;
      
    strcpy( dbEntry->tableName, "sngl_transdata");
    sprintf( dbEntry->columnName, "data_type");

    dbEntry->type       = char_s_ptr;
    dbEntry->numberRows = 1;
    dbEntry->rows.chars = (CHAR *) LALMalloc( LIGOMETA_TRANSDATA_DATA_MAX * dbEntry->numberRows * sizeof(CHAR) );
    sprintf(dbEntry->rows.chars,"n/a");
    dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
    *(dbEntry->rowDimensions) = 3;

    prev = dbEntry;

    /* data_units */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;

    strcpy( dbEntry->tableName, "sngl_transdata");
    sprintf( dbEntry->columnName, "data_units");

    dbEntry->type       = char_s_ptr;
    dbEntry->numberRows = 1;
    dbEntry->rows.chars = (CHAR *) LALMalloc( LIGOMETA_TRANSDATA_UNITS_MAX * dbEntry->numberRows * sizeof(CHAR) );
    sprintf(dbEntry->rows.chars,"n/a");
    dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
    *(dbEntry->rowDimensions) = 3;

    prev = dbEntry;

    /* length */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;
      
    strcpy( dbEntry->tableName, "sngl_transdata");
    sprintf( dbEntry->columnName, "transdata_length");

    dbEntry->type       = int_4s;
    dbEntry->numberRows = 1;
    dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
    dbEntry->rowDimensions = NULL;
    tdl = *(dbEntry->rows.int4s) = datalength;
    
    if(*(dbEntry->rows.int4s) > MAXBLOB) {
      tdl = *(dbEntry->rows.int4s) = MAXBLOB;
      datalength -= MAXBLOB;
    }

    prev = dbEntry;

    /* data (bigEndian) */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;
      
    strcpy( dbEntry->tableName, "sngl_transdata");
    sprintf( dbEntry->columnName, "transdata");
      
    dbEntry->type       = char_u_ptr;
    dbEntry->numberRows = 1;
    dbEntry->rows.charu = (UCHAR *)LALMalloc(tdl * sizeof(UCHAR));
    RASSERT(tdl,status,INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);

    memcpy(dbEntry->rows.charu, data + (i-ono)*MAXBLOB, tdl * sizeof(UCHAR));

    dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
    *(dbEntry->rowDimensions) = tdl;

    oldPos++;

    /* LAST entry */
    dbEntry->next = NULL;

  }

  LALFree(data);

  DETATCHSTATUSPTR (status);
  RETURN (status);
}







/***********************************************************************/
void BuildOutputFileBinary(LALStatus *status,
			   CHAR *file,
			   EventIDColumn     *burstEventList,
			   CHAR               *tag,
			   UINT4 t0Sec,
			   UINT4 t0NanoSec,
			   InitExchParams *initExchParams
			   )
{

#ifndef WORDS_BIGENDIAN
  static void endian_swap(char * pdata, int dsize, int nelements);
#endif

  /*
  FILE *fid;
  */

  /*
  struct flock lock;
  */

  ExchParams             *thisExch = NULL;
  ExchParams exchFileLock;

  SnglBurstTable *sbtptr;
  EventIDColumn *sbtptrG;

  UCHAR *data;
  UINT4 i,numOutput,datalength, size1, datalength_;

  size_t off;

  INITSTATUS (status, "BuildOutput", APPLYSEARCHC);
  ATTATCHSTATUSPTR (status);

  sbtptrG = burstEventList->next;
  numOutput = 0;
  while(sbtptrG) {
    numOutput++;
    sbtptrG = sbtptrG->next;
  }

#ifdef DEBUGBURST
  fprintf(stderr,"Got %u bursts\n",numOutput);
  fflush(NULL);
#endif

  size1 = (LIGOMETA_IFO_MAX + LIGOMETA_SEARCH_MAX + LIGOMETA_CHANNEL_MAX) * sizeof(CHAR) + 2*sizeof(INT4) + 6 * sizeof(REAL4);
  datalength_ = datalength = numOutput * size1 + sizeof(UINT4) + strlen(tag) + 1 + 3*sizeof(UINT4);
  data = (UCHAR *)LALMalloc( datalength );
  if(!data) {
    ABORT(status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
  }

  memcpy(data,&t0Sec,sizeof(UINT4));
  memcpy(data+sizeof(UINT4),&t0NanoSec,sizeof(UINT4));
  memcpy(data+2*sizeof(UINT4),&datalength_,sizeof(UINT4));

#ifndef WORDS_BIGENDIAN
  endian_swap((char *)(data), sizeof(UINT4), 3);
#endif

  strcpy(data+3*sizeof(UINT4),tag);

#ifndef WORDS_BIGENDIAN
  endian_swap((char *)(data+3*sizeof(UINT4)), sizeof(UCHAR), 1+strlen(tag));
#endif

  memcpy(data + strlen(tag) + 1 + 3*sizeof(UINT4), &numOutput, sizeof(UINT4));
#ifndef WORDS_BIGENDIAN
  endian_swap((char *)(data + strlen(tag) + 1 + 3*sizeof(UINT4)), sizeof(UINT4), 1);
#endif

  off = sizeof(UINT4) + strlen(tag) + 1 + 3*sizeof(UINT4);

  i = 0;
  sbtptrG = burstEventList->next;

  while(sbtptrG) {
    UCHAR *p1 = data + i * size1 + off,
      *pst, *pr4;

    if(sbtptrG->snglTransdataTable) {
      ABORT(status,INITSEARCHH_EUI,INITSEARCHH_MSGEUI);
    }

    sbtptr = sbtptrG->snglBurstTable;

    memcpy(p1, sbtptr->ifo,LIGOMETA_IFO_MAX * sizeof(CHAR));

    p1 += LIGOMETA_IFO_MAX * sizeof(CHAR);
    memcpy(p1, sbtptr->search,LIGOMETA_SEARCH_MAX * sizeof(CHAR));

    p1 += LIGOMETA_SEARCH_MAX * sizeof(CHAR);
    memcpy(p1, sbtptr->channel, LIGOMETA_CHANNEL_MAX * sizeof(CHAR));

    pst = p1 + LIGOMETA_CHANNEL_MAX * sizeof(CHAR);
    memcpy(pst, &(sbtptr->start_time.gpsSeconds), sizeof(INT4));
    memcpy(pst + sizeof(INT4), &(sbtptr->start_time.gpsNanoSeconds), sizeof(INT4));

    pr4 = pst + 2*sizeof(INT4);
    memcpy(pr4, &(sbtptr->duration), sizeof(REAL4));

    p1 = pr4 + sizeof(REAL4);
    memcpy(p1, &(sbtptr->central_freq), sizeof(REAL4));

    p1 += sizeof(REAL4);
    memcpy(p1, &(sbtptr->bandwidth), sizeof(REAL4));

    p1 += sizeof(REAL4);
    memcpy(p1, &(sbtptr->amplitude), sizeof(REAL4));
    
    p1 += sizeof(REAL4);
    memcpy(p1, &(sbtptr->snr), sizeof(REAL4));

    p1 += sizeof(REAL4);
    memcpy(p1, &(sbtptr->confidence), sizeof(REAL4));

#ifndef WORDS_BIGENDIAN
    endian_swap((char *)(pst), sizeof(INT4), 2);
    endian_swap((char *)(pr4), sizeof(REAL4), 6);
#endif

    sbtptrG = sbtptrG->next;
    i++;
  }

  exchFileLock.exchObjectType  = ExchFileLock;
  exchFileLock.send = 1;
  exchFileLock.numObjects = 1;
  exchFileLock.partnerProcNum = 0;

  LALInitializeExchange (status->statusPtr, &thisExch, 
			 &exchFileLock, initExchParams);
  CHECKSTATUSPTR (status);

  {
    CHARVector cvec;
    cvec.length = datalength;
    cvec.data = data;
    
    LALExchangeUINT4(status->statusPtr, &(datalength), thisExch);
    CHECKSTATUSPTR (status);
    LALExchangeCHARVector(status->statusPtr, &cvec, thisExch);
    CHECKSTATUSPTR (status);
  }

  /*
  {
    UINT4 u4 = 0;
    LALExchangeUINT4(status->statusPtr, &u4, thisExch);
    CHECKSTATUSPTR (status);
  }

  if(!(fid = fopen(file,"a"))) {
#ifdef DEBUGBURST
    perror("ERROR from fopen:");
    fflush(NULL);
#endif

    ABORT(status,INITSEARCHH_EFILE,INITSEARCHH_MSGEFILE);
  }
  */
  /*
  bzero(&lock,sizeof(struct flock));
  lock.l_type = F_WRLCK;
  lock.l_whence = SEEK_SET;

  if(fcntl(fileno(fid), F_SETLKW, &lock)) {
#ifdef DEBUGBURST
    perror("ERROR from fcntl:");
    fflush(NULL);
#endif

    ABORT(status,INITSEARCHH_EFILE,INITSEARCHH_MSGEFILE);
  }
  */

  /*
  if(fchmod(fileno(fid),S_IRUSR|S_IRGRP|S_IROTH|S_IWOTH|S_IWGRP|S_IWUSR)) {
#ifdef DEBUGBURST
    perror("ERROR from fchmod:");
    fflush(NULL);
#endif

    ABORT(status,INITSEARCHH_EFILE,INITSEARCHH_MSGEFILE);
  }

  if(fwrite(data,1,datalength_,fid) < datalength_) {
#ifdef DEBUGBURST
    perror("ERROR from fwrite:");
    fflush(NULL);
#endif

    ABORT(status,INITSEARCHH_EFILE,INITSEARCHH_MSGEFILE);
  }
  */

  /*
  bzero(&lock,sizeof(struct flock));
  lock.l_type = F_UNLCK;
  lock.l_whence = SEEK_SET;
  if(fcntl(fileno(fid), F_SETLK, &lock)) {
#ifdef DEBUGBURST
    perror("ERROR from fcntl (unlck):");
    fflush(NULL);
#endif

    ABORT(status,INITSEARCHH_EFILE,INITSEARCHH_MSGEFILE);
  }
  */
  /*
  if(fclose(fid)) {
#ifdef DEBUGBURST
    perror("ERROR from fclose:");
    fflush(NULL);
#endif

    ABORT(status,INITSEARCHH_EFILE,INITSEARCHH_MSGEFILE);
  }

  {
    UINT4 u4 = 0;
    LALExchangeUINT4(status->statusPtr, &u4, thisExch);
    CHECKSTATUSPTR (status);
  }
  */

  LALFinalizeExchange (status->statusPtr, &thisExch);
  CHECKSTATUSPTR (status);

  LALFree(data);

  DETATCHSTATUSPTR (status);
  RETURN (status);
}
