/********************************** <lalVerbatim file="ConditionDataCV">
Author: Julien Sylvestre
$Id: ConditionData.c,v 1.6 2003/09/04 21:10:41 julien Exp $
**************************************************** </lalVerbatim> */

#include <lal/LALStdlib.h>
#include <lal/AVFactories.h>
#include <lal/SeqFactories.h>
#include <lal/Random.h>
#include <lal/LALMalloc.h>
#include <InitSearch.h>

#include <lal/Calibration.h>
#include <ExtractSeries.h>

#include <strings.h>

NRCSID( CONDITIONDATAC , "power $Id: ConditionData.c,v 1.6 2003/09/04 21:10:41 julien Exp $" );

const LALUnit strainPerCount = {0,{0,0,0,0,0,1,-1},{0,0,0,0,0,0,0}};

void
LALConditionData(
    LALStatus             *status,
    LALSearchInput        *inout,
    void                  *searchParams,
    LALMPIParams          *mpiParams
                )
{
  BurstSearchParams *params;
  BurstParameter *ptr;
  BurstParameterList *ptrl;
  UINT4 si, Nmat;
  CHAR *cptr;
  CHAR ccptr[8192];

  INITSTATUS (status, "LALConditionData", CONDITIONDATAC);
  ATTATCHSTATUSPTR (status);

  RASSERT (searchParams, status, INITSEARCHH_ENULL, INITSEARCHH_MSGENULL);

  params = (BurstSearchParams *) searchParams;

  /*
   *
   * condition the data on nodes 
   *
   */

  /*
  if(!(params->searchMaster)) {
  */

  {
    INT4 i = 0;
    INT4 ki; 

    REAL4Vector *ptr_p, *ptr_c;


    ptr = params->ETGparams.next->param.next;
    if(ptr) {
      if(ptr->random == 2) {
	params->ETGparamsLOCK = 1;

	cptr = strtok(ptr->char_,"_");

	for(si=0;si<inout->numberSequences;si++) {
	  if(strstr(inout->sequences[si].name, cptr)) { 

	    CHAR *cbuf;

	    params->NNparams[0] = 0;

	    LALFree(ptr->char_);
	    LALFree(ptr);

	    ptr = &(params->ETGparams.next->param);
	    bzero(ptr, sizeof(BurstParameter));

	    /* process ilwd file, create burstparams lists */
	    if(inout->sequences[si].type != char_s) { 
	      ABORT ( status, INITSEARCHH_EIN, INITSEARCHH_MSGEIN ); 
	    } 

	    if(inout->sequences[si].numberDimensions != 2) { 
	      ABORT ( status, INITSEARCHH_EIN, INITSEARCHH_MSGEIN ); 
	    } 

	    cbuf = (CHAR *)calloc(1+inout->sequences[si].dimensions[1], sizeof(CHAR));
	    RASSERT(cbuf, status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);

	    for(ki=0;ki<inout->sequences[si].dimensions[0];ki++) { 
	      BurstParameter *tmp;

#ifdef DEBUGBURST
	      fprintf(stderr,"ETGParams: entering loop\n");
	      fflush(NULL);
#endif 

	      (params->NNparams[0])++;

	      tmp = (BurstParameter *)calloc(1,sizeof(BurstParameter));
	      RASSERT(tmp, status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);

	      ptr->next = (BurstParameter *)LALCalloc(1,sizeof(BurstParameter));
	      RASSERT(ptr->next, status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
	      ptr = ptr->next;
	      ptr->char_ = (CHAR *)LALCalloc(1, sizeof(BurstParameter *));
	      RASSERT(ptr->char_, status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);

	      memcpy(ptr->char_, &tmp, sizeof(BurstParameter *));

	      bzero(cbuf, inout->sequences[si].dimensions[1] * sizeof(CHAR));
	      memcpy(cbuf, inout->sequences[si].data.chars + inout->sequences[si].dimensions[1] * ki, inout->sequences[si].dimensions[1] * sizeof(CHAR));


#ifdef DEBUGBURST
	      fprintf(stderr,"ETGParams: processing string\n%s\n",cbuf);
	      fflush(NULL);
#endif 

	      /* process string */
	      {
		CHAR *arg, *s3;
		UINT4 Npa;

		arg = strtok_r(cbuf,",",&s3);

		while(arg) {
#ifdef DEBUGBURST
		  fprintf(stderr,"ETGParams: start ParseParameter for %s\n",arg);
		  fflush(NULL);
#endif 
		  Npa = 0;

		  ParseParameter(status->statusPtr, tmp, arg, &Npa);
		  CHECKSTATUSPTR (status);

		  tmp = tmp->next;

#ifdef DEBUGBURST
		  fprintf(stderr,"ETGParams: done ParseParameter (%u)\n",Npa);
		  fprintf(stderr,"ETGParams: %s\n%s\n",cbuf,arg);
		  fflush(NULL);
#endif 

		  arg = strtok_r(NULL,",",&s3);

#ifdef DEBUGBURST
		  fprintf(stderr,"ETGParams: %s\n%s\n",cbuf,arg);
		  fflush(NULL);
#endif 
		}
	      }
	    }

	    free(cbuf);

	    break;

	  }

	}

	if(si == inout->numberSequences) { 
	  ABORT( status, INITSEARCHH_ENULL, INITSEARCHH_MSGENULL); 
	} 

      }
    }

#ifdef DEBUGBURST
    fprintf(stderr,"ETGParams: done parsing\n");
    fflush(NULL);
#endif 

#define DOMATRIX(par_) \
    Nmat = 0; \
    ptr = params->par_.next; \
    if(ptr) { \
      if(ptr->random == 2) { \
	cptr = strtok(ptr->char_,"_"); \
\
	for(si=0;si<inout->numberSequences;si++) { \
	  if(strstr(inout->sequences[si].name, cptr)) { \
	    \
	    size_t off; \
\
	    LALFree(ptr->char_); \
	    LALFree(ptr); \
	    ptr = &(params->par_); \
	    \
	    if(inout->sequences[si].type != real_4) { \
	      ABORT ( status, INITSEARCHH_EIN, INITSEARCHH_MSGEIN ); \
	    } \
	    if(inout->sequences[si].numberDimensions != 2) { \
	      ABORT ( status, INITSEARCHH_EIN, INITSEARCHH_MSGEIN ); \
	    } \
\
	    for(ki=0, off=0;ki<inout->sequences[si].dimensions[0];ki++) { \
	      \
	      ptr->next = (BurstParameter *)LALCalloc(1,sizeof(BurstParameter)); \
	      RASSERT(ptr->next, status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC); \
	      ptr = ptr->next; \
              Nmat++; \
	      LALCreateVector(status->statusPtr, &(ptr->real4vector_), inout->sequences[si].dimensions[1]); \
	      CHECKSTATUSPTR (status); \
\
	      memcpy(ptr->real4vector_->data, inout->sequences[si].data.real4 + off, inout->sequences[si].dimensions[1] * sizeof(REAL4)); \
\
	      off += inout->sequences[si].dimensions[1]; \
              if(params->MAXinj == 0) {params->MAXinj = inout->sequences[si].dimensions[1];} \
\
	  } \
         params->Ninj_per_segment = params->MAXinj; \
         break; \
	} \
}\
	if(si == inout->numberSequences) { \
	  ABORT( status, INITSEARCHH_ENULL, INITSEARCHH_MSGENULL); \
	} \
      \
      } \
    } \

    DOMATRIX(waveform_amplitudes)
      /*
    if(params->MAXinj) {
      params->Nwaveform_amplitude = Nmat;
    }
      */
    DOMATRIX(alpha)
    DOMATRIX(delta)
    DOMATRIX(psi)
    DOMATRIX(injTimes)

#undef DOMATRIX

    if(params->MAXinj) {
      params->Nwaveform_amplitude = params->Nalpha = params->Ndelta = params->Npsi = 1;
    }

    ptr = params->waveforms.next;

#ifdef DEBUGBURST
for(si=0;si<inout->numberSequences;si++) {
      fprintf(stderr,"**%i:%s\n",si,inout->sequences[si].name);
      fflush(NULL);
}
#endif 

    if(params->Nwaveforms) {
      params->wave_pc = (REAL4TimeVectorSeries *)LALCalloc(params->Nwaveforms, sizeof(REAL4TimeVectorSeries));
      RASSERT(params->wave_pc, status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
    }

    while(ptr) {

      cptr = strtok(ptr->char_,"_");

      strcpy(ccptr,cptr);
      strcat(ccptr,"_p");

      for(si=0;si<inout->numberSequences;si++) {
	if(strstr(inout->sequences[si].name, ccptr)) {

#ifdef DEBUGBURST
	  fprintf(stderr,"Using waveforms %s for %s\n",inout->sequences[si].name,ccptr);
	  fflush(NULL);
#endif
	  ptr_p = NULL;

	  LALCreateVector(status->statusPtr, &ptr_p, inout->sequences[si].dimensions[0]);
	  CHECKSTATUSPTR (status);

	  if(inout->sequences[si].type != real_4) {
	    ABORT ( status, INITSEARCHH_EIN, INITSEARCHH_MSGEIN );
	  }
	  
	  memcpy(ptr_p->data, inout->sequences[si].data.real4,ptr_p->length * sizeof(REAL4)); 
	  
	  break;
	}
      }
      
      if(si == inout->numberSequences) {
	ABORT( status, INITSEARCHH_ENULL, INITSEARCHH_MSGENULL);
      }
      

     

      cptr = strtok(ptr->char_,"_");
      strcpy(ccptr,cptr);
      strcat(ccptr,"_c");

      for(si=0;si<inout->numberSequences;si++) {
	if(strstr(inout->sequences[si].name, ccptr)) {

#ifdef DEBUGBURST
	  fprintf(stderr,"Using waveforms %s for %s\n",inout->sequences[si].name,ccptr);
	  fflush(NULL);
#endif

	  ptr_c = NULL;
	  LALCreateVector(status->statusPtr, &ptr_c, inout->sequences[si].dimensions[0]);
	  CHECKSTATUSPTR (status);

	  if(inout->sequences[si].type != real_4) {
	    ABORT ( status, INITSEARCHH_EIN, INITSEARCHH_MSGEIN );
	  }
	  
	  memcpy(ptr_c->data, inout->sequences[si].data.real4,ptr_c->length * sizeof(REAL4)); 
	  
	  break;
	}
      }
      
      if(si == inout->numberSequences) {
	ABORT( status, INITSEARCHH_ENULL, INITSEARCHH_MSGENULL);
      }


      /* pack into TimeVectorSeries */
      RASSERT(ptr_c->length == ptr_p->length, status, INITSEARCHH_EIN, INITSEARCHH_MSGEIN );

      params->wave_pc[i].data = (REAL4VectorSequence *)LALMalloc(sizeof(REAL4VectorSequence));
      RASSERT(params->wave_pc[i].data, status, INITSEARCHH_EIN, INITSEARCHH_MSGEIN );

      params->wave_pc[i].data->length = 2;
      params->wave_pc[i].data->vectorLength = ptr_c->length;

#ifdef DEBUGBURST
      fprintf(stderr,"**%i\t%i\n",i,params->wave_pc[i].data->vectorLength);
      fflush(NULL);
#endif 

      params->wave_pc[i].data->data = (REAL4 *)LALMalloc(2 * ptr_c->length * sizeof(REAL4));
      RASSERT(params->wave_pc[i].data->data, status, INITSEARCHH_EIN, INITSEARCHH_MSGEIN );

      memcpy(params->wave_pc[i].data->data, ptr_p->data, ptr_p->length * sizeof(REAL4)); 

      memcpy(params->wave_pc[i].data->data + ptr_p->length, ptr_c->data, ptr_c->length * sizeof(REAL4)); 
      
      ptr = ptr->next;
      i++;

      LALDestroyVector(status->statusPtr, &ptr_p);
      CHECKSTATUSPTR (status);

      LALDestroyVector(status->statusPtr, &ptr_c);
      CHECKSTATUSPTR (status);

    }
  

    ptrl = params->ETGparams.next;
    while(ptrl) {
      ptr = ptrl->param.next;
      while(ptr) {
	
	if(ptr->real4vector_) {
	  
	  cptr = strtok(ptr->char_,"_");
	  
	  for(si=0;si<inout->numberSequences;si++) {
	    if(strstr(inout->sequences[si].name, cptr)) {

	      ptr->real4vector_->length = inout->sequences[si].dimensions[0];
	      ptr->real4vector_->data = (REAL4 *)LALMalloc(ptr->real4vector_->length * sizeof(REAL4));
	      RASSERT(ptr->real4vector_->data, status, INITSEARCHH_EIN, INITSEARCHH_MSGEIN );
	      
	      if(inout->sequences[si].type != real_4) {
		ABORT ( status, INITSEARCHH_EIN, INITSEARCHH_MSGEIN );
	      }
	      
	      memcpy(ptr->real4vector_->data, inout->sequences[si].data.real4,ptr->real4vector_->length * sizeof(REAL4)); 
	      
	      break;
	    }
	  }

	  if(si == inout->numberSequences) {
	    ABORT( status, INITSEARCHH_ENULL, INITSEARCHH_MSGENULL);
	  }

	}

	ptr = ptr->next;
      }
      ptrl = ptrl->next;
    }


    /* get GW data */
    for(si=0;si<inout->numberSequences;si++) {
      if(strstr(inout->sequences[si].name, GW_STRAIN_DATA)) {
	
	if(inout->sequences[si].dimensions[0] != params->Ndata) {
	  ABORT ( status, INITSEARCHH_EIN, INITSEARCHH_MSGEIN );
	}

	params->data.data = (REAL4TimeVectorSeries *)LALCalloc(1, sizeof(REAL4TimeVectorSeries));
	RASSERT(params->data.data, status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);

	params->data.data->data = NULL;

	{
	  CreateVectorSequenceIn vsin;
	  
	  vsin.length = 1;
	  vsin.vectorLength = inout->sequences[si].dimensions[0];

	  LALCreateVectorSequence(status->statusPtr, &(params->data.data->data), &vsin);
	  CHECKSTATUSPTR (status);
	}

	if(inout->sequences[si].type != real_4) {
	  ABORT ( status, INITSEARCHH_EIN, INITSEARCHH_MSGEIN );
	}

	memcpy(params->data.data->data->data, inout->sequences[si].data.real4,params->data.data->data->vectorLength * sizeof(REAL4)); 

	/* set time */
	params->data.data->epoch.gpsSeconds = inout->sequences[si].range.dTime.startSec;
	params->data.data->epoch.gpsNanoSeconds = inout->sequences[si].range.dTime.startNan;

	/* set sampling frequency */
	params->data.data->deltaT = inout->sequences[si].range.dTime.timeStepSize;

      break;

      }

    }

    if(si == inout->numberSequences) {
      ABORT( status, INITSEARCHH_ENULL, INITSEARCHH_MSGENULL);
    }


   /* get PSD from datacond */
    for(si=0;si<inout->numberSequences;si++) {
      if(strstr(inout->sequences[si].name, GW_STRAIN_PSD)) {

	params->data.spec = (REAL4FrequencySeries *)LALCalloc(1,sizeof(REAL4FrequencySeries));
	RASSERT(params->data.spec, status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
	
	params->data.spec->data = NULL;
	
	LALCreateVector(status->statusPtr, &(params->data.spec->data), inout->sequences[si].dimensions[0]);
	CHECKSTATUSPTR (status);
	
	if(inout->sequences[si].type != real_4) {
	  ABORT ( status, INITSEARCHH_EIN, INITSEARCHH_MSGEIN );
	}

	memcpy(params->data.spec->data->data, inout->sequences[si].data.real4, inout->sequences[si].dimensions[0] * sizeof(REAL4)); 

      break;

      }

    }

    if(si == inout->numberSequences) {
      ABORT( status, INITSEARCHH_ENULL, INITSEARCHH_MSGENULL);
    }


    /* set up calibration */
    /* code stolen from inspiral dso */
    {
      REAL4TimeSeries              ats, *chanPtr = &ats;
      REAL4Vector av;
      COMPLEX8FrequencySeries      *respPtr;
      CHAR                          ifo[3];

      chanPtr->f0 =  params->data.data->f0;
      chanPtr->deltaT =  params->data.data->deltaT;
      chanPtr->epoch =  params->data.data->epoch;
      chanPtr->f0 =  params->data.data->f0;
      chanPtr->data = &av;
      chanPtr->data->length = params->data.data->data->vectorLength;
      chanPtr->data->data = params->data.data->data->data;

      respPtr = params->data.resp = (COMPLEX8FrequencySeries *)LALCalloc(1, sizeof(COMPLEX8FrequencySeries));
      RASSERT(respPtr, status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);

      strncpy(ifo,params->channel,2);
      ifo[2] = 0;

#ifdef DEBUGBURST
      fprintf(stderr,"ifo = %s\n",ifo);
      fflush(NULL);
#endif 

      /* set the start epoch for the response function from the data */
      respPtr->epoch.gpsSeconds = chanPtr->epoch.gpsSeconds;
      respPtr->epoch.gpsNanoSeconds = chanPtr->epoch.gpsNanoSeconds;

      /* set the frequency step size of the response from the psd */
      respPtr->f0 = 0.0;
      respPtr->deltaF = 1.0 / 
        ( ((REAL8) chanPtr->data->length) * chanPtr->deltaT );

      /* set the units of the desired response function */
      respPtr->sampleUnits = strainPerCount;

      /* allocate memory for the vector in the response function */
      respPtr->data = (COMPLEX8Vector *)LALCalloc(1, sizeof(COMPLEX8Vector));
      RASSERT(respPtr->data, status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);

      respPtr->data->length = chanPtr->data->length / 2 + 1;
      respPtr->data->data = (COMPLEX8 *) 
        LALCalloc( respPtr->data->length, sizeof(COMPLEX8) );

      /* extract it */
      LALExtractResponse( status->statusPtr, respPtr, inout, ifo );
      RCHECKSTATUSPTR( status );

    }

  }

  DETATCHSTATUSPTR (status);
  RETURN (status);

}
