/********************************** <lalVerbatim file="StochasticHV">
Author: Anderson, W.G., Romano, J.D., and Whelan, J.T.
$Id: SGWB.h,v 1.3 2003/09/29 20:11:29 kaice Exp $
**************************************************** </lalVerbatim> */

/******************************* <lalLaTeX file="StochasticH">
\section{Header \texttt{Stochastic.h}}
\label{s:Stochastic.h}

Provides structures and definitions global to the stochastic search,
notably \texttt{StochasticSearchParams}.

\subsection*{Synopsis}
\begin{verbatim}
#include "Stochastic.h"
\end{verbatim}

\subsection*{Error conditions}

There are no error conditions associated with this header file.

\subsection*{Structures and Unions}

*************************************************** </lalLaTeX> */

#ifndef _SGWB_H
#define _SGWB_H

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <unistd.h>
#include <lal/StochasticCrossCorrelation.h>
#include "LALWrapperInterface.h"
#include "SGWBData.h"

/* turn these on to enable debugging tools */
#undef STOCHASTIC_SO_DEBUG    /* print debug information */
/* #undef STOCHASTIC_SO_ATTACH_DDD */ /* sleep for a few seconds to attach ddd */ 
/* #define STOCHASTIC_SO_DEBUG  */      /* print debug information */
#define STOCHASTIC_SO_ATTACH_DDD  /* sleep for a few seconds to attach ddd */

#define STOCHASTICH_CCSTATDBBIT 1
#define STOCHASTICH_CCSIGMDBBIT 2
#define STOCHASTICH_CCSPECDBBIT 4
#define STOCHASTICH_CCSPECFRBIT 8

#ifdef  __cplusplus
extern "C" {
#endif

NRCSID (SGWBH, "stochastic $Id: SGWB.h,v 1.3 2003/09/29 20:11:29 kaice Exp $");

#define STOCHASTICH_FALSE 0
#define STOCHASTICH_TRUE  1

/******************************* <lalLaTeX file="StochasticH">

\subsubsection*{Union \texttt{StochasticFFTPlan}}
\index{\texttt{StochasticFFTPlan}}

\noindent
This holds a pointer to an FFTPlan, with the type being determined by
whether or not the time series data for the search are heterodyned.
The fields are:
\begin{description}
  \item[\texttt{RealFFTPlan *realPlan}]
  \item[\texttt{ComplexFFTPlan *complexPlan}]
\end{description}

*************************************************** </lalLaTeX> */

typedef union
tagStochasticFFTPlan
{
  RealFFTPlan                      *realPlan;
  ComplexFFTPlan                   *complexPlan;
}
StochasticFFTPlan;

/******************************* <lalLaTeX file="StochasticH">

\subsubsection*{Structure \texttt{StochasticSearchParams}}
\index{\texttt{StochasticSearchParams}}

\noindent
The fields are:
\begin{description}
  \item[\texttt{BOOLEAN searchMaster}]
  \item[\texttt{BOOLEAN haveData}]
  \item[\texttt{UINT4 currentSegment}]
  \item[\texttt{COMPLEX8FrequencySeries *optimalFilter}]
  \item[\texttt{REAL4WithUnits *sigma}]
  \item[\texttt{REAL4Vector *window1}]
  \item[\texttt{REAL4Vector *window2}]
  \item[\texttt{StochasticFFTPlan fftPlan}]
  \item[\texttt{StochasticInitParams *initParams}]
  \item[\texttt{StochasticDataSegmentVector *stochasticSegVec}]
  \item[\texttt{gpsTimeInterval                 timeInterval}]
  \item[\texttt{REAL4 segmentDuration}]   
\end{description}

*************************************************** </lalLaTeX> */

  typedef struct
  tagStochasticSearchParams
  {
    BOOLEAN                         searchMaster;  
    BOOLEAN                         haveData;
    UINT4                           *numSlaves;
    UINT4                           currentSegment;
    UINT4                           masterCounter;
    UINT4                           slaveCounter;
    COMPLEX8FrequencySeries*        optimalFilter;
    REAL4FrequencySeries*           overlapReduction;
    REAL4FrequencySeries*           frequencyMask;
    REAL4WithUnits*                 sigma;
    REAL4Vector*                    window1;
    REAL4Vector*                    window2;
    StochasticDataSegmentVector     stochasticJobVec;
    StochasticDataSegmentVector     stocahsticMCVec;
    StochasticFFTPlan               fftPlan;
    StochasticInitParams*           initParams;
    StochasticDataSegmentVector     stochasticSegVec;
    gpsTimeInterval                 timeInterval;
    REAL4                           segmentDuration;
    UINT4                           jobLength;
    UINT4                           fftLength;

  }
  StochasticSearchParams;

#ifdef  __cplusplus
}
#endif /* C++ protection */

#endif /* _SGWB_H  */


/******************************* <lalLaTeX file="StochasticH">
  
\vfill{\footnotesize\input{StochasticHV}}

*************************************************** </lalLaTeX> */
