#include "CastToFunctions.h"


/*  static  */
void CastToReal4 ( LALStatus *status, REAL4 *output,
			  const multiDimData *input,
			  const UINT4 offset, const UINT4 length )
{
  REAL4     *sPtr, *sStopPtr;
  REAL8     *dPtr;
  INT2      *i2Ptr;
  UINT2     *u2Ptr;
  INT4      *i4Ptr;
  UINT4     *u4Ptr;
  INT8      *i8Ptr;
  UINT8     *u8Ptr;
  COMPLEX8  *cPtr;
  COMPLEX16 *zPtr;

  /* printf ("%d %d\n", input->type, real_4); */
  switch (input->type)
  {
    case real_4:
      sPtr = 
	memcpy ( output, input->data.real4 + offset,
		 length * sizeof(REAL4) );
      if (sPtr != output)
      {
        ABORT (status, 
               STOCHASTICCONDITIONDATAH_EALOC, 
               STOCHASTICCONDITIONDATAH_MSGEALOC);
      }
      break;
    case real_8:
      sStopPtr = output + length;
      for ( sPtr = output, dPtr = input->data.real8 + offset;
	    sPtr < sStopPtr; ++sPtr, ++dPtr )
      {
	*sPtr = (REAL4) *dPtr;
      }
    break;
    case int_2s:
      sStopPtr = output + length;
      for ( sPtr = output, i2Ptr = input->data.int2s + offset;
	    sPtr < sStopPtr; ++sPtr, ++i2Ptr )
      {
	*sPtr = (REAL4) *i2Ptr;
      }
    break;
    case int_2u:
      sStopPtr = output + length;
      for ( sPtr = output, u2Ptr = input->data.int2u + offset;
	    sPtr < sStopPtr; ++sPtr, ++u2Ptr )
      {
	*sPtr = (REAL4) *u2Ptr;
      }
    break;
    case int_4u:
      sStopPtr = output + length;
      for ( sPtr = output, u4Ptr = input->data.int4u + offset;
	    sPtr < sStopPtr; ++sPtr, ++u4Ptr )
      {
	*sPtr = (REAL4) *u4Ptr;
      }
    break;
    case int_4s:
      sStopPtr = output + length;
      for ( sPtr = output, i4Ptr = input->data.int4s + offset;
	    sPtr < sStopPtr; ++sPtr, ++i4Ptr )
      {
	*sPtr = (REAL4) *i4Ptr;
      }
    break;
    case int_8u:
      sStopPtr = output + length;
      for ( sPtr = output, u8Ptr = input->data.int8u + offset;
	    sPtr < sStopPtr; ++sPtr, ++u8Ptr )
      {
	*sPtr = (REAL4) *u8Ptr;
      }
    break;
    case int_8s:
      sStopPtr = output + length;
      for ( sPtr = output, i8Ptr = input->data.int8s + offset;
	    sPtr < sStopPtr; ++sPtr, ++i8Ptr )
      {
	*sPtr = (REAL4) *i8Ptr;
      }
    break;
    case complex_8:
      sStopPtr = output + length;
      for ( sPtr = output, cPtr = input->data.complex8 + offset;
	    sPtr < sStopPtr; ++sPtr, ++cPtr )
      {
	*sPtr = (REAL4) cPtr->re;
      }
    break;
    case complex_16:
      sStopPtr = output + length;
      for ( sPtr = output, zPtr = input->data.complex16 + offset;
	    sPtr < sStopPtr; ++sPtr, ++zPtr )
      {
	*sPtr = (REAL4) zPtr->re;
      }
    default:
      ABORT (status, 
	     STOCHASTICCONDITIONDATAH_ETYPE, 
	     STOCHASTICCONDITIONDATAH_MSGETYPE);
  }
}

/*  static  */
void CastToComplex8 ( LALStatus *status, COMPLEX8 *output,
			     const multiDimData *input,
			     const UINT4 offset, const UINT4 length )
{
  REAL4     *sPtr;
  REAL8     *dPtr;
  INT2      *i2Ptr;
  UINT2     *u2Ptr;
  INT4      *i4Ptr;
  UINT4     *u4Ptr;
  INT8      *i8Ptr;
  UINT8     *u8Ptr;
  COMPLEX8  *cPtr, *cStopPtr;
  COMPLEX16 *zPtr;

  /* printf ("%d %d\n", input->type, complex_8); */
  switch (input->type)
  {
    case complex_8:
      cPtr = 
	memcpy ( output, input->data.complex8 + offset,
		 length * sizeof(COMPLEX8) );
      if (cPtr != output)
      {
        ABORT (status, 
               STOCHASTICCONDITIONDATAH_EALOC, 
               STOCHASTICCONDITIONDATAH_MSGEALOC);
      }
    break;
    case complex_16:
      cStopPtr = output + length;
      for ( cPtr = output, zPtr = input->data.complex16 + offset;
	    cPtr < cStopPtr; ++cPtr, ++zPtr )
      {
	cPtr->re = (REAL4) zPtr->re;
	cPtr->im = (REAL4) zPtr->im;
      }
    break;
    case real_4:
      cStopPtr = output + length;
      for ( cPtr = output, sPtr = input->data.real4 + offset;
	    cPtr < cStopPtr; ++cPtr, ++dPtr )
      {
	cPtr->re = *sPtr;
	cPtr->im = 0.0;
      }
    break;
    case real_8:
      cStopPtr = output + length;
      for ( cPtr = output, dPtr = input->data.real8 + offset;
	    cPtr < cStopPtr; ++cPtr, ++dPtr )
      {
	cPtr->re = (REAL4) *dPtr;
	cPtr->im = 0.0;
      }
    break;
    case int_2s:
      cStopPtr = output + length;
      for ( cPtr = output, i2Ptr = input->data.int2s + offset;
	    cPtr < cStopPtr; ++cPtr, ++i2Ptr )
      {
	cPtr->re = (REAL4) *i2Ptr;
	cPtr->im = 0.0;
      }
    break;
    case int_2u:
      cStopPtr = output + length;
      for ( cPtr = output, u2Ptr = input->data.int2u + offset;
	    cPtr < cStopPtr; ++cPtr, ++u2Ptr )
      {
	cPtr->re = (REAL4) *u2Ptr;
	cPtr->im = 0.0;
      }
    break;
    case int_4u:
      cStopPtr = output + length;
      for ( cPtr = output, u4Ptr = input->data.int4u + offset;
	    cPtr < cStopPtr; ++cPtr, ++u4Ptr )
      {
	cPtr->re = (REAL4) *u4Ptr;
	cPtr->im = 0.0;
      }
    break;
    case int_4s:
      cStopPtr = output + length;
      for ( cPtr = output, i4Ptr = input->data.int4s + offset;
	    cPtr < cStopPtr; ++cPtr, ++i4Ptr )
      {
	cPtr->re = (REAL4) *i4Ptr;
	cPtr->im = 0.0;
      }
    break;
    case int_8u:
      cStopPtr = output + length;
      for ( cPtr = output, u8Ptr = input->data.int8u + offset;
	    cPtr < cStopPtr; ++cPtr, ++u8Ptr )
      {
	cPtr->re = (REAL4) *u8Ptr;
	cPtr->im = 0.0;
      }
    break;
    case int_8s:
      cStopPtr = output + length;
      for ( cPtr = output, i8Ptr = input->data.int8s + offset;
	    cPtr < cStopPtr; ++cPtr, ++i8Ptr )
      {
	cPtr->re = (REAL4) *i8Ptr;
	cPtr->im = 0.0;
      }
    break;
    default:
      ABORT (status, 
	     STOCHASTICCONDITIONDATAH_ETYPE, 
	     STOCHASTICCONDITIONDATAH_MSGETYPE);
  }
}

/* static */ /* inline */ 

INT4 ctoi (CHAR c)
{
  switch(c)
  {
  case '0':
    return 0;
  case '1':
    return 1;
  case '2':
    return 2;
  case '3':
    return 3;
  case '4':
    return 4;
  case '5':
    return 5;
  case '6':
    return 6;
  case '7':
    return 7;
  case '8':
    return 8;
  case '9':
    return 9;
  default:
    return -1;
  }
}
