#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <lal/LALStdlib.h>
#include <lal/LALDatatypes.h>
#include <lal/Comm.h>
#include <lal/AVFactories.h>
#include <lal/TimeFreqFFT.h>
#include <lal/CoarseGrainFrequencySeries.h>
#include "SGWBApplySearch.h"
#include "SGWB.h"
#include "CastToFunctions.h"
#include "SGWBAuxFunctions.h"

NRCSID (SGWBCOREAPPLYSEARCHC, "$Id: SGWBCoreApplySearch.c,v 1.3 2003/09/15 00:43:02 kaice Exp $");


void LALSGWBCoreApplySearch ( LALStatus *status,
			      LALSearchOutput *output,
			      LALSearchInput *input,
			      StochasticSearchParams *params)
{
  
  SZeroPadAndFFTParameters            szpfftParams;
  CZeroPadAndFFTParameters            czpfftParams;
  COMPLEX8FrequencySeries             fseries1;
  COMPLEX8FrequencySeries             fseries2;
  COMPLEX8FrequencySeries             ccSpectrum;
  COMPLEX8FrequencySeries             ccSpectrumOut;
  REAL4WithUnits                      ccStatReal;
  COMPLEX8WithUnits                   ccStatComplex;
  StochasticCrossCorrelationInput     ccInput;
  FrequencySamplingParams             cgParams;
  COMPLEX8                            *cPtr;
  COMPLEX8                            *cStopPtr;

  LALUnitPair                         unitPair;

  UINT4                               numFTPoints;

  StochasticSearchOutput              searchOutput;

  StochasticRealDataSegment*          sDummySegment = NULL;
  StochasticComplexDataSegment*       cDummySegment = NULL;

  REAL4FrequencySeries           *spec1Ptr, *spec2Ptr;

  FrequencySamplingParams         freqParams;

  StochasticOmegaGWParameters     omegaGWParams;
  StochasticInverseNoiseInput     invNoiseInput1;
  StochasticInverseNoiseInput     invNoiseInput2;
  StochasticInverseNoiseOutput    invNoiseOutput1;
  StochasticInverseNoiseOutput    invNoiseOutput2;

  StochasticOptimalFilterInput    optimalInput;

  StochasticOptimalFilterNormalizationInput       optNormInput;
  StochasticOptimalFilterNormalizationParameters  optNormParams;
  StochasticOptimalFilterNormalizationOutput      optNormOutput;

  REAL4WithUnits                  lambda;
  LALUnit                         tmpUnit;
  RAT4                            power;


  REAL4FrequencySeries           *spec1, *spec2;

  SequenceIndices                 sI;

  COMPLEX8FrequencySeries        *resp1, *resp2;
  
  REAL4FrequencySeries            calibratedInvNoise1;
  REAL4FrequencySeries            calibratedInvNoise2;
  COMPLEX8FrequencySeries         halfCalibratedInvNoise1;
  COMPLEX8FrequencySeries         halfCalibratedInvNoise2;

  REAL4FrequencySeries            omegaGW;

  INITSTATUS (status, "LALCoreApplySearch", SGWBCOREAPPLYSEARCHC);
  ATTATCHSTATUSPTR (status);
  
      
  fprintf(stdout,"CORE slavecount %d nodenumsegs %d currentseg %d \n", params->slaveCounter, params->initParams->nodeNumSegs, params->currentSegment); 
  fflush( stdout ); 
  

  TRY( LALFindCheckMDSequences(status->statusPtr,
			       input,
			       params,
			       &sI),
       status );
    
  TRY( LALAllocateParamFieldMemory( status->statusPtr,
				    params,
				    &resp1,
				    &resp2,
				    &spec1, 
				    &spec2,
				    &spec1Ptr,
				    &spec2Ptr),
       status );
  
  /*
   *
   * Fill in frequency sampling parameters from initparams
   *
   */

  freqParams.f0 = params->initParams->f0;
  freqParams.deltaF = params->initParams->deltaF;
  freqParams.length = params->initParams->numFrequencies;

  TRY( LALCalculateSpec1Spec2( status->statusPtr,
			       params,
			       &spec1, 
			       &spec2,
			       &spec1Ptr,
			       &spec2Ptr,
			       freqParams),
       status );
        

  TRY( LALCalculateResp1Resp2( status->statusPtr,
			       input,
			       params,
			       &resp1, 
			       &resp2,
			       freqParams,
			       sI),
       status );

  /* Begin move calculate inverse noise psds from ConditionData */

  /* 
   *
   * calculate inverse noise psds 
   *
   */

  /* spec1 */
  /* allocate memory for StochasticInverseNoiseOutput fields */
  calibratedInvNoise1.data = NULL;
  TRY( LALCreateVector (status->statusPtr,
			&(calibratedInvNoise1.data),
			params->initParams->numFrequencies),
       status );
    
  halfCalibratedInvNoise1.data = NULL;
  TRY( LALCCreateVector (status->statusPtr,
			 &(halfCalibratedInvNoise1.data),
			 params->initParams->numFrequencies),
       status );
    
  /* fill in StochasticInverseNoiseInput fields */
  invNoiseInput1.unCalibratedNoisePSD = spec1;
  invNoiseInput1.responseFunction  = resp1;
    
  /* fill in StochasticInverseNoiseOutput fields */
  invNoiseOutput1.calibratedInverseNoisePSD   = &calibratedInvNoise1;
  invNoiseOutput1.halfCalibratedInverseNoisePSD = &halfCalibratedInvNoise1;

  /* calculate inverse noise outputs */
  TRY( LALStochasticInverseNoise (status->statusPtr,
				  &invNoiseOutput1,
				  &invNoiseInput1),
       status );
     
  /* spec2 */
  /* allocate memory for StochasticInverseNoiseOutput fields */
  calibratedInvNoise2.data = NULL;
  TRY( LALCreateVector (status->statusPtr,
			&(calibratedInvNoise2.data),
			params->initParams->numFrequencies),
       status );
    
  halfCalibratedInvNoise2.data = NULL;
  TRY( LALCCreateVector (status->statusPtr,
			 &(halfCalibratedInvNoise2.data),
			 params->initParams->numFrequencies),
       status );
    
  /* fill in StochasticInverseNoiseInput fields */
  invNoiseInput2.unCalibratedNoisePSD = spec2;
  invNoiseInput2.responseFunction  = resp2;
    
  /* fill in StochasticInverseNoiseOutput fields */
  invNoiseOutput2.calibratedInverseNoisePSD   = &calibratedInvNoise2;
  invNoiseOutput2.halfCalibratedInverseNoisePSD = &halfCalibratedInvNoise2;

  /* calculate inverse noise outputs */
  TRY( LALStochasticInverseNoise (status->statusPtr,
				  &invNoiseOutput2,
				  &invNoiseInput2),
       status );
  /* End move calculate inverse noise psds from ConditionData */

  /* Begin move omega_GW from ConditionData */


  /*
   *
   * set omega_GW = const 
   *
   */
    
  /* allocate memory for frequency series */
  omegaGW.data = NULL;
  TRY( LALCreateVector (status->statusPtr,
			&(omegaGW.data),
			params->initParams->numFrequencies),
       status );

  /* fill in StochasticOmegaGWParameters structure */
  omegaGWParams.alpha    = 0;
  omegaGWParams.fRef     = params->initParams->f0
    + ( ( params->initParams->numFrequencies - 1 ) / 2
	* params->initParams->deltaF );
  omegaGWParams.omegaRef = 1;
  omegaGWParams.length   = params->initParams->numFrequencies;
  omegaGWParams.f0       = params->initParams->f0;
  omegaGWParams.deltaF   = params->initParams->deltaF;

  /* calculate omegaGW */
  TRY( LALStochasticOmegaGW (status->statusPtr,
			     &omegaGW,
			     &omegaGWParams),
       status );

  /* End move omega_GW from ConditionData */

  /* Begin calculate optimal filter normalization from ConditionData */
    
  /*
   *
   * calculate optimal filter normalization
   *
   */
    
  /* fill in StochasticOptimalFilterNormalizationInput fields */
  optNormInput.overlapReductionFunction = params->overlapReduction;
  optNormInput.omegaGW = &omegaGW;
  optNormInput.inverseNoisePSD1 = &calibratedInvNoise1;
  optNormInput.inverseNoisePSD2 = &calibratedInvNoise2;

  /* fill in StochasticOptimalFilterNormalizationParameters fields */
  optNormParams.fRef = omegaGWParams.fRef;

  /* check base frequency for heterodyned data */
  optNormParams.heterodyned = (BOOLEAN) params->initParams->baseFreq;

  /* Specify window functions for normalization */
  optNormParams.window1 = params->window1;
  optNormParams.window2 = params->window2;

  /* fill in StochasticOptimalFilterNormalizationParameters fields */
  if ( params->optimalFilter ) {
    optNormOutput.normalization = &lambda;
  }
  else
    {
      optNormOutput.normalization = NULL;
    }

  /* params->sigma is null if sigma has not been requested */
  optNormOutput.variance = params->sigma;

  /* calculate optimal filter normalization */

  TRY( LALStochasticOptimalFilterNormalization (status->statusPtr,
						&optNormOutput,
						&optNormInput,
						&optNormParams),
       status );

  /* This is sigma^2/T, so we need to convert it to sigma */ 
  /* Corresponding to a single segment */

  if (params->sigma)
    {
      params->sigma->value = sqrt(params->sigma->value
				  * params->segmentDuration );
      unitPair.unitOne = &(params->sigma->units);
      unitPair.unitTwo = &lalSecondUnit;
      TRY( LALUnitMultiply(status->statusPtr,
			   &tmpUnit,
			   &unitPair),
	   status );
      power.numerator = 1;
      power.denominatorMinusOne = 1;
      TRY( LALUnitRaise(status->statusPtr,
			&(params->sigma->units),
			&tmpUnit,
			&power),
	   status );
    }

  if ( params->optimalFilter ) {
    /*
     *
     * calculate optimal filter 
     *
     */
      
    /* fill in StochasticOptimalFilterInput fields */
    optimalInput.overlapReductionFunction = params->overlapReduction;
    optimalInput.omegaGW = &omegaGW;
    optimalInput.halfCalibratedInverseNoisePSD1 = &halfCalibratedInvNoise1;
    optimalInput.halfCalibratedInverseNoisePSD2 = &halfCalibratedInvNoise2;
      
    /* calculate optimal filter values */
      
    TRY( LALStochasticOptimalFilter (status->statusPtr,
				     params->optimalFilter,
				     &optimalInput,
				     &lambda),
	 status );
  }
  else
    {
      params->optimalFilter = NULL;
    }


  /*        printf("\n\n ****** \n");  */
  /*        for (j = 0; j <  params->optimalFilter->data->length; j++) {   */
  /*  	printf(" %e %e %i %i  %i \n", (params->optimalFilter->data->data[j].re), (params->optimalFilter->data->data[j].im), params->currentSegment, params->optimalFilter->data->length, j); */
  /*        }  */
  /*        printf("\n ******  \n\n"); */


  /* End calculate optimal filter normalization from ConditionData */

  /**************************************************************

  ****** Begining of original ApplySearch Routine  **************

  ***************************************************************/


  /* Assign searchOutput pointers depending on outputs requested */

  if ( ( STOCHASTICH_CCSPECDBBIT | STOCHASTICH_CCSPECFRBIT )
       & params->initParams->outputType ) 
    {
      searchOutput.ccSpec = &ccSpectrumOut;
    }
  else
    {
      searchOutput.ccSpec = NULL;
    }

  if ( STOCHASTICH_CCSTATDBBIT & params->initParams->outputType ) 
    {
      if (!(params->initParams->baseFreq))
	{ /* real option */
	  searchOutput.ccStatReal = &ccStatReal;
	  searchOutput.ccStatComplex = NULL;
	}
      else
	{
	  searchOutput.ccStatReal = NULL;
	  searchOutput.ccStatComplex = &ccStatComplex; 
	}
    }
  else
    {
      searchOutput.ccStatReal = NULL;
      searchOutput.ccStatComplex = NULL;
    }

  if (params->optimalFilter)
    { /* 
       *
       * allocate temporary memory for the frequency data 
       *
       */
       
      numFTPoints = ( params->initParams->baseFreq 
		      ? ( params->initParams->fullLength )
		      : params->initParams->fullLength / 2 + 1
		      );
       
      fseries1.data = NULL;
      TRY( LALCCreateVector (status->statusPtr, 
			     &(fseries1.data),
			     numFTPoints),
	   status );

       
      fseries2.data = NULL;
      TRY( LALCCreateVector (status->statusPtr, 
			     &(fseries2.data), 
			     numFTPoints),
	   status );
       
      if (searchOutput.ccSpec)
	{
	  ccSpectrum.data = NULL;
	  TRY( LALCCreateVector (status->statusPtr, 
				 &(ccSpectrum.data), 
				 params->initParams->numFrequencies),
	       status );
	 
	  ccSpectrumOut.data = NULL;
	  TRY( LALCCreateVector (status->statusPtr, 
				 &(ccSpectrumOut.data), 
				 params->initParams->outputNumFreq),
	       status );
	}
       
      /* 
       * zero-pad and FFT the time-series data
       */
       
      if (!(params->initParams->baseFreq))
	{ /* real option */
	  /* point dummySegment to the current segment to analyze */

	  /*** KTR ***/
	  /* move this bit and the one below in to ApplySearch 
	     and pass them in */
	  sDummySegment 
	    = params->stochasticSegVec.realVec->data + params->currentSegment;
	  /*** KTR ***/
	 
	  szpfftParams.fftPlan = params->fftPlan.realPlan;
	  szpfftParams.length  = params->initParams->fullLength;
	  szpfftParams.window  = params->window1;
	  TRY( LALSZeroPadAndFFT (status->statusPtr,
				  &fseries1,
				  sDummySegment->data1,
				  &szpfftParams),
	       status );
	 
	  szpfftParams.window = params->window2;
	  TRY( LALSZeroPadAndFFT (status->statusPtr,
				  &fseries2,
				  sDummySegment->data2,
				  &szpfftParams),
	       status );
	 
	} /* end real option */
      else
	{ /* complex option */
	  /* point dummySegment to the current segment to analyze */
	  cDummySegment 
	    = params->stochasticSegVec.complexVec->data + params->currentSegment;
	 
	  czpfftParams.fftPlan = params->fftPlan.complexPlan;
	  czpfftParams.length  = params->initParams->fullLength;
	  czpfftParams.window  = params->window1;
	  TRY( LALCZeroPadAndFFT (status->statusPtr,
				  &fseries1,
				  cDummySegment->data1,
				  &czpfftParams),
	       status );
	 
	  czpfftParams.window  = params->window2;
	  TRY( LALCZeroPadAndFFT (status->statusPtr,
				  &fseries2,
				  cDummySegment->data2,
				  &czpfftParams),
	       status );
	}/* end complex option */

      /* 
       *
       * build stochastic cross-correlation input structure 
       *
       */
       
      ccInput.hBarTildeOne  = &fseries1;
      ccInput.hBarTildeTwo  = &fseries2;
      ccInput.optimalFilter = params->optimalFilter;
       
      if ( searchOutput.ccSpec ) 
	{ /* 
	   * calculate the cross-correlation spectrum 
	   */
	  TRY( LALStochasticCrossCorrelationSpectrum(status->statusPtr,
						     &ccSpectrum,
						     &ccInput,
						     params->initParams->epochsMatch),
	       status );
 
	  cgParams.length = params->initParams->outputNumFreq;
	  cgParams.f0     = params->initParams->outputF0;
	  cgParams.deltaF = params->initParams->outputDeltaF;
	 
	  /* Coarse-grain the output to the desired resolution */
	  TRY( LALCCoarseGrainFrequencySeries(status->statusPtr,
					      &ccSpectrumOut,
					      &ccSpectrum,
					      &cgParams),
	       status);
	 
	  /* 
	   *
	   * build output for metadata database or trend frames 
	   *
	   */
	 
	  if ( searchOutput.ccStatReal )
	    { /* real option */
	      if (ccSpectrum.f0 == 0) 
		{
		  /* DC contribution */
		  ccStatReal.value 
		    = ccSpectrum.data->data->re / 2.0;
	     
		  /* We might want to check that the imaginary parts of the DC
		     components of all the series vanish */
	     
		  /* initialize pointers */
		  cPtr = ccSpectrum.data->data + 1;
		} /* if f0 == 0 */
	      else 
		{
		  ccStatReal.value  = 0.0;
	     
		  /* initialize pointers */
		  cPtr = ccSpectrum.data->data;
		}    
	      cStopPtr    = ccSpectrum.data->data + ccSpectrum.data->length;
	      /* contributions from positive and (negative) frequency components */
	      for ( ; cPtr < cStopPtr; ++cPtr ) 
		{
		  ccStatReal.value  += cPtr->re;
		}
	   
	      /* normalize */
	      ccStatReal.value  *= ccSpectrum.deltaF * 2.0 ;
	   
	      /* Set output units */  
	      unitPair.unitOne = &ccSpectrum.sampleUnits;
	      unitPair.unitTwo =  &lalHertzUnit;
	      TRY( LALUnitMultiply(status->statusPtr,
				   &(ccStatReal.units), 
				   &unitPair),
		   status );
	    } /* end real option */
	  else if ( searchOutput.ccStatComplex )
	    { /* complex option */
	   
	      ccStatComplex.value.re = ccStatComplex.value.im = 0.0;
	   
	      /* initialize pointers */
	      cPtr = ccSpectrum.data->data;
	   
	      cStopPtr    = ccSpectrum.data->data + ccSpectrum.data->length;
	      /* contributions from positive and (negative) frequency components */
	      for ( ; cPtr < cStopPtr; ++cPtr ) 
		{
		  ccStatComplex.value.re += cPtr->re;
		  ccStatComplex.value.im += cPtr->im;
		}
	   
	      /* normalize */
	      ccStatComplex.value.re *= ccSpectrum.deltaF;
	      ccStatComplex.value.im *= ccSpectrum.deltaF;
	   
	      /* Set output units */  
	      unitPair.unitOne = &ccSpectrum.sampleUnits;
	      unitPair.unitTwo =  &lalHertzUnit;
	      TRY( LALUnitMultiply(status->statusPtr,
				   &(ccStatComplex.units),
				   &unitPair),
		   status );
	    }

	  /* 
	   * remove temporary memory for the frequency data 
	   */
	 
	  TRY( LALCDestroyVector (status->statusPtr, &(ccSpectrum.data)),
	       status );
	}
      else if ( searchOutput.ccStatReal )
	{ /* real option */
	  TRY( LALStochasticCrossCorrelationStatistic(status->statusPtr,
						      &ccStatReal,
						      &ccInput,
						      params->initParams->epochsMatch),
	       status );
	} /* end real option */
      else if  ( searchOutput.ccStatComplex )
	{ /* complex option */
	  TRY( LALStochasticHeterodynedCrossCorrelationStatistic(status->statusPtr,
								 &ccStatComplex,
								 &ccInput,
								 params->initParams->epochsMatch),
	       status );
	}
     
      /* 
       * remove temporary memory for the frequency data 
       */
       
      TRY( LALCDestroyVector (status->statusPtr, &(fseries2.data)), status );
       
      TRY( LALCDestroyVector (status->statusPtr, &(fseries1.data)), status );
    }

  /* KTR: This if statement prevents the DSO from writing identical
     values of the theoretical error to the database. In the past this
     would avoid writing the same value of sigma for each segment of
     a given job. I am going to remove the if because we are going
     to want a sigma for each segment of the data segments. sigma will
     now vary for each segment within a job. */
  /*       if ( params->currentSegment == 0 ) */
  /* 	{ */
  searchOutput.sigma = params->sigma;
  /* 	} */
  /*       else */
  /* 	{ */
  /* 	  searchOutput.sigma = NULL; */
  /* 	} */

  searchOutput.outputType = params->initParams->outputType;

  TRY( StochasticBuildOutput(status->statusPtr,
			     output,
			     &searchOutput,
			     params),
       status );

  if ( searchOutput.ccSpec )
    {
      TRY( LALCDestroyVector (status->statusPtr, &(ccSpectrumOut.data)),
	   status );
    }
     
  /*
   *
   * clean up
   *
   */

  TRY( LALCDestroyVector (status->statusPtr,
			  &(halfCalibratedInvNoise2.data)),
       status );
   
  TRY( LALDestroyVector (status->statusPtr, &(calibratedInvNoise2.data)),
       status );
   
  TRY( LALCDestroyVector (status->statusPtr,
			  &(halfCalibratedInvNoise1.data)),
       status );
   
  TRY( LALDestroyVector (status->statusPtr, &(calibratedInvNoise1.data)),
       status );

  TRY( LALDestroyVector (status->statusPtr, &(omegaGW.data)),
       status );

  /* moved from below */ 

  /*
   *
   * clean up
   *
   */
   
   
   
  TRY( LALDestroyVector (status->statusPtr, &(spec1->data)),
       status );
   
  LALFree(spec1);
   
  TRY( LALDestroyVector (status->statusPtr, &(spec2->data)),
       status );
   
  LALFree(spec2);
   
  TRY( LALDestroyVector (status->statusPtr, &(spec1Ptr->data)),
       status );
   
  LALFree(spec1Ptr);
   
  TRY( LALDestroyVector (status->statusPtr, &(spec2Ptr->data)),
       status );
   
  LALFree(spec2Ptr);

  TRY( LALCDestroyVector (status->statusPtr, &(resp1->data)),
       status );
   
  LALFree(resp1);
   
  TRY( LALCDestroyVector (status->statusPtr, &(resp2->data)),
       status );
   
  LALFree(resp2);
  
 printf("\n\n *************** This Far????????? ************** %i \n\n", params->currentSegment);

 
  /* normal exit */ 
  DETATCHSTATUSPTR (status);
  RETURN (status);

}
