/* -*- mode: c; c-basic-offset: 2; -*- */
/******** <lalVerbatim file="SlopeApplySearchMasterC"> ********
Author: Daw, E
$Id: SlopeApplySearchMaster.c,v 1.5 2003/02/06 21:48:56 edaw Exp $
******** </lalVerbatim> ********/

#ifdef DEBUG_SLOPE
#include <stdio.h>
#endif
#include <math.h>

/* lal standard library */
#include <lal/LALStdlib.h>

/* lal MPI interface */
#include <lal/Comm.h>

/* lal slope filters */
#include <lal/SlopeDetectorFilter.h>

#include <BuildDB.h> 

/* structures and definitions for this code segment */
#include <SlopeDatatypes.h>
#include <SlopeAddTimeToGpsSecNan.h>
#include <SlopeAddEventToLinkedList.h>
#include <SlopeApplySearchMaster.h>

NRCSID( SLOPEAPPLYSEARCHMASTERC, "$Id: ");

void
SlopeApplySearchMaster(LALStatus*          status,
		       LALSearchOutput*    output,
		       SlopeSearchParams*  params) 
{
  REAL4Vector dataIn;
  REAL4Vector dataSendOut;
  REAL4* dataOut;
  UINT4 dataSearchCount = 0;
  UINT4 nSearch;
  UINT4 nDataOut = 0;
  UINT4 peakBin = 0;
  UINT4 peakTimeSec;
  UINT4 peakTimeNan;
  UINT4 lastBigger;
  UINT4 nZeroCrossings = 0;
  UINT4 peakBinAtStart = 0;
  REAL4 peakAmplitude = 0.0;
  REAL4 peakSNR = 0.0;
  REAL4 peakAmpAtStart = 0.0;
  REAL4 peakDuration = 0.0;
  REAL4 secAfterStartStretch;
  REAL4 searchidentifier;
  REAL4 sumSq = 0.0;
  REAL4 rmsSeg = 0.0;
  UINT4 sumsqCount = 0;
  SLOPEFilterParams* filterp;
  SLOPEFilterParams filterp_local;
  UINT4 hcount;
  UINT4 triggersInThisBatch=0;
  const char* slopesearchname="slope";
  const char* slopesearchcomment="none";

#ifdef SLDEBUG
  UINT4 i;
#endif  

#ifdef SLDEBUG
  printf("Starting apply search master.\n");
#endif

  INITSTATUS( status, "SlopeApplySearchMaster", SLOPEAPPLYSEARCHMASTERC );
  ATTATCHSTATUSPTR(status);

  /* set pointer for filter parameters substructure of */
  /* the search parameters array */
  filterp = (SLOPEFilterParams*)params;

  /* get the length of the data output array */
  nDataOut = params->nData - params->forder + 1 ;
#ifdef SLDEBUG
  printf("number of bins in output array: %u\n",nDataOut);
#endif
  
  /* get the number of bins to search */
  nSearch = params->nData - 
    params->forder -
    params->peakWidth + 2;

  /* construct data vector for input data array. */
  dataIn.length = params->nData;
  dataIn.data = (REAL4*)params->dataNew;

  searchidentifier = (REAL4)params->function_select;

  if(params->dataScanned==0){

    /* allocate memory for output data */
    dataOut = LALCalloc( nDataOut , sizeof(REAL4) );
  
    /* make structure containing output data array and size */
    dataSendOut.length = nDataOut;
    dataSendOut.data = dataOut;
    params->filteredData = dataOut;

#ifdef SLDEBUG
    printf("Starting filtration\n");
#endif

    /* set the frequency equal to the peak identifier */
    /* so I can identify which search turned up a trigger */
    /* this is a temporary kluge for E7 */

    switch( params->function_select ) {
    case FILTER_OUTPUT_PRELIM:
      /* filter the data using the 'old' slope detector filter */
      LALSlopeDetectorFilter(status->statusPtr,
			     &dataSendOut,
			     &dataIn,
			     params->forder);
      break;
    case FILTER_OUTPUT_SLOPE:
      LALSlopeLineFitFilter(status->statusPtr,
			    &dataSendOut,
			    &dataIn,
			    *(filterp));
      break;
    case FILTER_OUTPUT_OFFSET:
      LALSlopeLineFitFilter(status->statusPtr,
			    &dataSendOut,
			    &dataIn,
			    *(filterp));
      break;
    case FILTER_OUTPUT_ALF:
      LALSlopeLineFitFilter(status->statusPtr,
			    &dataSendOut,
			    &dataIn,
			    *(filterp));
      break;
    case FILTER_OUTPUT_TAPS_SET_BOXCAR:
      if((*(filterp->taps_set)) == 0) {
	LALSlopeConvolutionFilter(status->statusPtr,
				  &dataSendOut,
				  &dataIn,
				  *(filterp));
      }
      filterp_local.forder = filterp->forder;
      filterp_local.tap = filterp->tap;
      filterp_local.history_allocated = filterp->history_allocated;
      filterp_local.taps_set = filterp->taps_set;
      filterp_local.history = filterp->history;
      filterp_local.function_select = FILTER_OUTPUT_CONVOLVE;
      filterp_local.waveform_offset = filterp->waveform_offset;
      filterp_local.sampling_period_s = filterp->sampling_period_s;
      LALSlopeConvolutionFilter(status->statusPtr,
				&dataSendOut,
				&dataIn,
				filterp_local);
      break;
    case FILTER_OUTPUT_TAPS_SET_GAUSSIAN:
      if((*(filterp->taps_set)) == 0) {
	LALSlopeConvolutionFilter(status->statusPtr,
				  &dataSendOut,
				  &dataIn,
				  *(filterp));
      }
      filterp_local.forder = filterp->forder;
      filterp_local.tap = filterp->tap;
      filterp_local.history_allocated = filterp->history_allocated;
      filterp_local.taps_set = filterp->taps_set;
      filterp_local.history = filterp->history;
      filterp_local.function_select = FILTER_OUTPUT_CONVOLVE;
      filterp_local.waveform_offset = filterp->waveform_offset;
      filterp_local.sampling_period_s = filterp->sampling_period_s;
      LALSlopeConvolutionFilter(status->statusPtr,
				&dataSendOut,
				&dataIn,
				filterp_local);
      break;
    case FILTER_OUTPUT_TAPS_SET_SINE:
      if((*(filterp->taps_set)) == 0) {
	LALSlopeConvolutionFilter(status->statusPtr,
				  &dataSendOut,
				  &dataIn,
				  *(filterp));
      }
      filterp_local.forder = filterp->forder;
      filterp_local.tap = filterp->tap;
      filterp_local.history_allocated = filterp->history_allocated;
      filterp_local.taps_set = filterp->taps_set;
      filterp_local.history = filterp->history;
      filterp_local.function_select = FILTER_OUTPUT_CONVOLVE;
      filterp_local.waveform_offset = filterp->waveform_offset;
      filterp_local.sampling_period_s = filterp->sampling_period_s;
      LALSlopeConvolutionFilter(status->statusPtr,
				&dataSendOut,
				&dataIn,
				filterp_local);
      break;
    default:
      ABORT(status, APPLYSEARCH_NOSUCHFILTER, APPLYSEARCH_NOSUCHFILTERMESG );
    }

#ifdef SLDEBUG
    printf("Starting peak search.\n");
#endif

    /* write the latter end of the output data to the output history buffer */
    for(hcount=0 ; hcount < (params->peakWidth - 1) ; ++hcount) {
      params->peakHistBuf[hcount] = dataOut[nSearch + hcount];
    }

  } /* end if(scannedData==0) */
  else {
    dataOut=params->filteredData;
  }
  
  /* adapt threshold to reflect current rms if required. If the rms */
  /* is zero, set the threshold to params->triggerThreshold. The rms */
  /* being zero indicates that there is no nonzero input data, so setting */
  /* the threshold nonzero will result in no triggers being generated, */
  /* which is the desired result */
  if ( params->dataNormalized == 0 ) {
    if ( params->thresholdAdaptive == 1 ) {
      sumSq = 0; 
      for ( sumsqCount = 0; sumsqCount < nSearch; ++sumsqCount ) {
        sumSq += ( dataOut[sumsqCount] * dataOut[sumsqCount] );
      }
      rmsSeg = sqrt(sumSq / nSearch);
      if ( rmsSeg > 0 ) {
        params->thresholdSet = ( params->triggerThreshold * rmsSeg );
      } else {
        params->thresholdSet = params->triggerThreshold;
      }
    } else {
      params->thresholdSet = params->triggerThreshold;
    }
  }	
  params->dataNormalized = 1;  

  /* do a peak search on the filtered data */
  dataSearchCount = params->dataScanned;

  while(dataSearchCount < nSearch && triggersInThisBatch < params->batchSize) {

    /* test if the next bin exceeds the threshold */
    if( dataOut[dataSearchCount] > params->thresholdSet ) {

      /* increment counter for detected events */
      ++params->numberOfTriggersSoFar;
      triggersInThisBatch++;

      /* set event amplitude */
      peakAmplitude = dataOut[dataSearchCount];

      /* set bin where peak starts */
      peakBin = dataSearchCount;

      /* look over the (peakWidth) bins after the one exceeding */
      /* the threshold. The peak is positioned at the tallest one */
      lastBigger = 1;
      peakBinAtStart = peakBin;
      peakAmpAtStart = peakAmplitude;
      while(dataSearchCount < ( peakBin + 
			     params->peakWidth - 1 )) {
	++dataSearchCount;
	if( ( dataOut[dataSearchCount] > peakAmpAtStart ) ||
	    ( ( -dataOut[dataSearchCount]  ) > peakAmpAtStart ) ) {
	  lastBigger = dataSearchCount - peakBinAtStart;
	}
	if( dataOut[dataSearchCount] > peakAmplitude ) {
	  peakAmplitude = dataOut[dataSearchCount];
	  peakBin = dataSearchCount;
	}
      }
      peakDuration = ((REAL4)lastBigger)/((REAL4)params->samplingRate);
      
      /* count the zero crossings within the peak */
      nZeroCrossings = 0;
      dataSearchCount = peakBinAtStart;
      while( dataSearchCount < ( peakBin + lastBigger ) ) {
	++dataSearchCount;
	if( ( ( dataOut[ dataSearchCount ] > 0 ) &&
	      ( dataOut[ dataSearchCount - 1 ] < 0 ) ) ||
	    ( ( dataOut[ dataSearchCount ] < 0 ) &&
	      ( dataOut[ dataSearchCount - 1 ] > 0 ) ) ) {
	  ++nZeroCrossings;
	}
      }
      dataSearchCount = peakBinAtStart + params->peakWidth - 1;
      
      /* get peak snr */
      /* if adaptive thresholding was not set, peakSNR is just the */
      /* same as peak amplitude. If it was set peakSNR is peak */
      /* amplitude divided by the rms measured over the dataset */
      peakSNR = peakAmplitude /
	        (params->thresholdSet / params->triggerThreshold);

      /* get peak time */
      secAfterStartStretch = (REAL4)peakBin/((REAL4)params->samplingRate);
      peakTimeSec = params->startTimeSec;
      peakTimeNan = params->startTimeNan;
      SlopeAddTimeToGpsSecNan(&peakTimeSec, 
			      &peakTimeNan, secAfterStartStretch);
      
#ifdef SLDEBUG
      printf("Adding event at bin %u\n",peakBin); 
#endif
      
      SlopeAddEventToLinkedList(status->statusPtr,
				peakTimeSec,
				peakTimeNan,
				peakAmplitude,
                                peakSNR,
				peakDuration,
				nZeroCrossings/(2*peakDuration),
				params->thresholdSet,
				&(params->currentEvent),
				&(params->firstEvent) );

#ifdef SLDEBUG
      printf("Added event. Data search count is %u / %u\n",
	     dataSearchCount,nDataOut);
      /* print out search information */
      printf("next data in %f\n",
	     dataIn.data[dataSearchCount]);
      printf("next data out %f\n",dataOut[dataSearchCount]);
#endif
      

    } else {
      ++dataSearchCount;
    }

    /* update progress in search */
    output->fracRemaining = ( (REAL4)(nSearch - dataSearchCount) / nSearch );
  }

  params->dataScanned=dataSearchCount;
  params->numberOfTriggersInThisSearch=triggersInThisBatch;
#ifdef SLDEBUG
  printf("Finished search.\n");

#endif

  if(params->dataScanned >= nSearch ){
    SearchSummaryInfo summary;
    LIGOTimeGPS startTime;
    LIGOTimeGPS endTime;
#ifdef SLDEBUG
    dataBase* db=NULL;
#endif

    startTime.gpsSeconds=params->startTimeSec;
    startTime.gpsNanoSeconds=params->startTimeNan;

    endTime.gpsSeconds=params->stopTimeSec;
    endTime.gpsNanoSeconds=params->stopTimeNan;

    LALFree(dataOut);
#ifdef SLDEBUG
    printf("Freed dataOut.\n");
#endif

    summary.dsoName=slopesearchname;
    summary.comment=slopesearchcomment;
    summary.inputStartTime=startTime;
    summary.inputEndTime=endTime;
    summary.outputStartTime=startTime;
    summary.outputEndTime=endTime;
    summary.numEvents=params->numberOfTriggersSoFar;
    summary.numNodes=2;

    LALCreateSearchSummaryDataBase(status->statusPtr, &params->summary, &summary);

/*      params->numberOfTriggersInThisSearch++; */
#ifdef SLDEBUG
    db=params->summary;
    while(db!=NULL){
      printf("------>>> %s, %s, %d, %p, %p\n",db->tableName,db->columnName,
	     db->numberRows,
	     db->next, db->previous);
      fflush(stdout);
      db=db->next;
    }
#endif


    /* Fake event is inserted here in order to have some entry in the tables
       even if no events were found. As a result one should always substract 1
       from the total number of events to get the number of real events
    */

/*      SlopeAddEventToLinkedList(status->statusPtr, */
/*  				(INT4)(-1), */
/*  				(INT4)(-1), */
/*  				(REAL4)(-1.0), */
/*  				(REAL4)(-1.0), */
/*  				searchidentifier, */
/*  				(REAL4)(-1.0), */
/*  				&(params->currentEvent), */
/*  				&(params->firstEvent) ); */

/*        ++params->numberOfTriggersSoFar; */
/*        triggersInThisBatch++; */
/*        params->numberOfTriggersInThisSearch=triggersInThisBatch; */

    


  }

  CHECKSTATUSPTR(status);
  DETATCHSTATUSPTR(status);
  RETURN(status);
}






