/* -*- mode: c; c-basic-offset: 2; -*- */
/******** <lalVerbatim file="SlopeLALFinalizeSearchC"> ********
Author: Daw, E
$Id: SlopeLALFinalizeSearch.c,v 1.2 2002/07/15 22:04:34 edaw Exp $
********* </lalVerbatim> ********/

#ifdef DEBUG_SLOPE
#include <stdio.h>
#endif
#include <math.h>

/* functions for memory allocation */
#include <lal/LALStdlib.h>

/* functions for vector creation and manipulation */
/* #include <lal/AVFactories.h> */

/* functions for LAL MPI interface */
#include <lal/Comm.h>

/* functions for slope filtering and peak searching data */

/* structures and definitions for this wrapper code */
#include <SlopeDatatypes.h>

NRCSID( SLOPELALFINALIZESEARCHC, "$Id: SlopeLALFinalizeSearch.c,v 1.2 2002/07/15 22:04:34 edaw Exp $");

/******** local static function declarations ********/

/****************************************************/
/** note that currently my code is designed to run **/
/** on a single machine.                           **/
/****************************************************/

/******** wrapperAPI function definitions */
/* free up memory dynamically assigned during analysis run */
/* note that I DO NOT free up memory allocated to the subfields */
/* of LALSearchOutput* output. There is no way to do this as it */
/* is not passed as an argument to LALFinalizeSearch. I am assuming */
/* that wrapperAPI tidies this up as necessary */

void
LALFinalizeSearch(LALStatus*             status,
		  void**                 psearchParams)
{
  SlopeSearchParams* searchParams = 0x0;
  SlopeDetections* pslope = 0x0;

  INITSTATUS( status, "LALFinalizeSearch", SLOPELALFINALIZESEARCHC );
  ATTATCHSTATUSPTR(status);

  searchParams = (SlopeSearchParams*)*psearchParams;

  /* if a linked list of events was created, flush the list */

  if( searchParams->numberOfTriggersInThisSearch > 0 ) {
    pslope = searchParams->firstEvent;
    while( pslope!=0x0 && (pslope->next) != 0x0 ) {
      pslope = pslope->next;
      LALFree(pslope->previous);
    }
    if(pslope!=0x0) LALFree( pslope );
  }

  if( ((searchParams->function_select == FILTER_OUTPUT_TAPS_SET_BOXCAR) ||
       (searchParams->function_select == FILTER_OUTPUT_TAPS_SET_GAUSSIAN)) ||
      (searchParams->function_select == FILTER_OUTPUT_TAPS_SET_SINE ) ) {
    LALFree( searchParams->taps_set );
    LALFree( searchParams->tap );
  }

/*  #ifdef SLDEBUG */
/*    if(searchParams->summary!=NULL){ */
/*      printf("summary's memory was not released\n"); */
/*      printf("%s \n",searchParams->summary->columnName); */
/*    } */
/*  #endif */

  LALFree( searchParams->history_allocated );
  LALFree( searchParams->dataNew );
  LALFree( searchParams->filtHistBuf );
  LALFree( searchParams->peakHistBuf );
  LALFree( searchParams );
  
  DETATCHSTATUSPTR(status);
  RETURN(status);
}





