/******** <lalVerbatim file="StackSlideCV"> ********
Author: Mendell, G. and Landry, M.
$Id: StackSlide.c,v 1.1 2003/09/25 19:41:59 jolien Exp $
********* </lalVerbatim> ********/

/* Notes: */
/* BLK = one Block of frequency domain data */
/* STK = one Stack of frequency domain data; Block are coherenty combined to make Stacks. */
/* SUM = Summed Stacks after Sliding */

/* ??? Code first submitted to lalwrapper cvs. */

/* TO DO LIST: */
/* Get units from input data.  For now these are hard coded based on input data type. */
/* Currently inputDataTypeFlag == 0 = SFTs is the only supported input data type. Need to add types 1 = PSDs, 2 = F-stat */
/* Currently calibration and normalization code only works when input data is SFTs */
/* Currently normalizationFlag == 1 only type of normalization supported */
/* Check that all allocated memory is deallocated */
/* Code currently assumes params->numSUMsPerParamSpacePt = params->duration/params->tSUM = 1; need to upgrade to more general case */
/* Compute Correct Power Stats, snr, confidence, etc... */
/* Code is hardwired to search to look for "sft" in comment of inout-sequence: if (strstr(inout->sequences[i].comment, "sft")); should use fdomain channel name */

/* REVISIONS: */

/* #define INCLUDE_DEMOD_CODE */ /* GET CODE WORKING WITH STKS = PSDs FIRST */
#define INCLUDE_BUILDSUMFRAME_CODE

/* #define DEBUG_STACKSLIDE */
/* #define DEBUG_PARAMETERS */
/* #define DEBUG_BLK_CODE */
/* #define DEBUG_DEMOD_CODE */
/* #define DEBUG_BUILDOUTPUT_CODE */
/* #define DEBUG_BUILDSUMFRAMEOUTPUT_CODE */
/* #define DEBUG_FRACREMAINING_CODE */
/* #define DEBUG_INPUTBLK_CODE */
/* #define DEBUG_LALWRAPPERINITBARYCENTER */
/* #define DEBUG_NORMALIZEBLKS */
/* #define DEBUG_CALIBRATEBLKS */

/* UNCOMMENT ONLY ONE OF INCLUDE_LALINITBARYCENTER OR INCLUDE_LALWRAPPERINITBARYCENTER AT A TIME */
/* UNCOMMENT INCLUDE_LALINITBARYCENTER FOR TEST PURPOSES IN STANDALONE MODE; OTHERWISE UNCOMMENT INCLUDE_LALWRAPPERINITBARYCENTER */
/* NOTE: when INCLUDE_LALINITBARYCENTER is uncommeted one must hard code input earth and sun file names (see below). */
/* #define INCLUDE_LALINITBARYCENTER */
#define INCLUDE_LALWRAPPERINITBARYCENTER

#include <stdio.h>
#include <math.h>
#include <lal/LALStdlib.h>
#include <lal/AVFactories.h>
/* Next two are used when calibrating BLKs */
#include <ExtractSeries.h>
#include <lal/VectorOps.h>
#include <lal/RealFFT.h>
#include <lal/LALConstants.h>
/* #include <lal/LALInitBarycenter.h> */ /* Need to include this in stand alone mode only when INCLUDE_LALINITBARYCENTER uncommeted above. */
#include <lal/LALDemod.h>
#include <lal/Units.h>
#include <StackSlide.h>
/* #include <BuildDB.h> */ /* Not using LALCreateSearchSummaryDataBase */
#include <lal/BandPassTimeSeries.h>

NRCSID( STACKSLIDEC, "$Id: StackSlide.c,v 1.1 2003/09/25 19:41:59 jolien Exp $");

/******************************************/
/*                                        */
/* START SECTION: prototype declarations  */
/*                                        */
/******************************************/

#ifdef INCLUDE_LALINITBARYCENTER
 /* Copied from LALInitBarycenter.c in the LAL support package.  Author: Curt Cutler */
 void LALInitBarycenter(LALStatus *stat, EphemerisData *edat);
#endif
#ifdef INCLUDE_LALWRAPPERINITBARYCENTER
 /* Function to transfer ephemeris data input as ilwd from LALSearchInput structure into LAL structures; based on LALInitBarycenter. */
 void LALWRAPPERInitBarycenter(LALStatus *stat, LALSearchInput *inout, EphemerisData *edat);
#endif

static void TimeToFloat(REAL8 *f, LIGOTimeGPS *tgps); /* Copied from LALDemodTest.c */
static void FloatToTime(LIGOTimeGPS *tgps, REAL8 *f); /* Copied from LALDemodTest.c */
void ComputePowerStats( const REAL4Vector *fsVec, FreqSeriesPowerStats *stats, REAL8 fStart, REAL8 df );

static void BuildOutput(LALStatus*            const status,
	    		const StackSlideSearchParams* const params,
	    		LALSearchOutput*      const output);

#ifdef INCLUDE_BUILDSUMFRAME_CODE
static void BuildSUMFrameOutput(LALStatus* const status,
	    const StackSlideSearchParams*    const params,
	    LALSearchOutput*            const output);
#endif

/******************************************/
/*                                        */
/* END SECTION: prototype declarations    */
/*                                        */
/******************************************/

/******************************************/
/*                                        */
/* START FUNCTION: LALInitSearch          */
/*                                        */
/******************************************/
void
LALInitSearch(
    LALStatus             *status,
    void                 **searchParams,
    LALInitSearchParams   *initSearchParams
    )
{
  /* Declare variables */
  StackSlideSearchParams *params;
  UINT2 i;

  INITSTATUS( status, "LALInitSearch", STACKSLIDEC );

  #ifdef DEBUG_STACKSLIDE
  	fprintf(stdout, "\nLALInitSearch\n");
  	fflush(stdout);
  #endif

  /* Attach StackSlideSearchParams to searchParams */
  *searchParams = params = LALMalloc(sizeof( StackSlideSearchParams ));

  #ifdef DEBUG_STACKSLIDE
  	printf("\nargc = %i\n",initSearchParams->argc);
  #endif

/******************************************/
/*                                        */
/* START SECTION: parameters passed as    */
/* arguments.  Those indented are         */
/* computed from these.                   */
/*                                        */
/******************************************/

  #ifdef DEBUG_STACKSLIDE
  	fprintf(stdout, "\nSTART SECTION: parameters passed as arguments\n");
  	fflush(stdout);
  #endif

/*  UINT4   gpsStartTimeSec;            GPS start time of data requested seconds */
/*  UINT4   gpsStartTimeNan;            GPS start time of data requested nanoseconds */
/*  REAL8   sampleRate;                 Sample rate of the time-domain data used to make the frequency-domain input data Blocks */
/*    REAL8   deltaT;                     Time step size in seconds = 1.0/sampleRate */
/*  REAL8   duration;                   Total time being analyzed  */
/*    UINT4   totalNumTimeSamples;        Total Number of time samples = sampleRate*duration */

/*  INT4    numBLKs;                    Number of input BLKs.  Not duration/tBLK if gaps are present */
/*  REAL8   tBLK;                       duration in seconds of BLKs */
/*    INT4    numBLKsPerDuration;         Equals duration/tBLK; but not acual number of blocks if gaps exist in the data */
/*    INT4    nSamplesPerBLK;             Equals sampleRate*tBLK = number of time domain data points used to compute one BLKs */
/*  REAL8   tEffBLK;                    Effective duration of BLK such that dfBLK = 1.0/tEffBLK  */
/*    REAL8   dfBLK;                      Freq resolution of BLK = 1.0/tEffBLK  Could be different from 1/tBLK due to oversampling  */
/*  REAL8   f0BLK;                      Start frequency of the input BLKs */
/*  REAL8   bandBLK;                    Band width of the input BLKs. */
/*  INT4    nBinsPerBLK;                Number of data points in the input BLK in the input frequency band */

/*  INT4    numBLKsPerSTK;              Number of BLKs to use to make one STK */
/*    INT4    numSTKs;                    Number of actual STKs. Not duration/tSTK if gaps are present */
/*    REAL8   tSTK;                       duration in seconds of STKs = tBLK*numBLKsPerSTK */
/*  REAL8   tEffSTK;                    Effective duration of STK such that dfSTK = 1.0/tEffSTK */
/*    REAL8   dfSTK;                      Freq resolution of STK = 1.0/tEffSTK. Could be different from 1/tSTK due to oversampling  */
/*  REAL8   f0STK;                      Start frequency of STKs */
/*  REAL8   bandSTK;                    Band width of the STKs. */
/*  INT4    nBinsPerSTK;                Number of data points in the STKs in the STK frequency band */

/*  INT4    numSTKsPerSUM;              Number of STKs to use to make one SUM (Usually duration/tSTK) */
/*    INT4    numSUMsPerParamSpacePt;   Number of output SUMs per parameter space point = params->duration/params->tSUM. (Usually will = 1) */
/*    REAL8   tSUM;                       duration in seconds of output SUMs = tSTK*numSTKsPerSUM. (Usually = duration) */
/*  REAL8   tEffSUM;                    Effective duration of SUM such that dfSUM = 1.0/tEffSUM */
/*    REAL8   dfSUM;                      Freq resolution of output SUMs.  dfSUM = 1.0/tEffSUM. Could be different from dfSTK in some algorithms?  */
/*  REAL8   f0SUM;                      Start frequency of output SUMs */
/*  REAL8   bandSUM;                    Band width of the output SUMs. */
/*  INT4    nBinsPerSUM;                Number of data points in the output SUMs in the input frequency band */

/*  CHAR    *ifoNickName;               2 character ifoNickName = H1, H2, or L1; note IFO is the site = LHO, LLO, GEO, etc.... */
/*  CHAR    *IFO;                       Identifies the interferometer site = LHO, LLO, GEO, etc.... */
/*  CHAR    *patchName;                 Name of the patch in parameter space of the search (e.g, Galactic Center) */
/*  INT4    patchNumber;                Index of the patch in parameter space of the search (will be -1 until indexing implemented) */

/*  CHAR    tDomainChannel[dbNameLimit];   Time domain channel name used to produce blocks */
/*  CHAR    fDomainChannel[dbNameLimit];   Frequency domain channel name that blocks are stored as */

/*  INT2    inputDataTypeFlag;          0 = SFTs, 1 = PSDs, 2 = F-stat */

/*  INT2    parameterSpaceFlag;         0 = use input delta for each param, 1 = params are input as vectors of data, 2 = use input param metric, 3 = create param metric */
/*    INT4    numParamSpacePts;           Total number of points in the parameter space to cover */
/*    INT4    numSUMsTotal;               Total Number of Sums output = numSUMsPerParamPt*numParamSpacePts */

/*  INT2    stackTypeFlag;              0 means stacks are PSDs from SFTs, 1 means stacks are F-statistic from SFTs */
/*  INT4    Dterms;                     = terms used in the computation of the dirichlet kernel in LALDemod (Default value is 32) */

/*  INT2    thresholdFlag;              How to apply the threshold; 0 = if power above threshold 1 put result in database */
/*  REAL8   threshold1;                 Threshold 1 for put results in the database (could be on power or false alarm rate) */
/*  REAL8   threshold2;                 Threshold 1 for put results in the database (could be on power or false dismisal rate) */

/*  INT2    calibrationFlag;            Flag that specifies what calibration to do; -1 means Blks are already calibrated, 0 means leave uncalibrated; 1 calibrate */

/*  INT2    weightFlag;                 Flag that specifies whether to weight BLKs or STKs with a(t) or b(t).  */

/*  INT2    windowFilterFlag;           Flag that specifies whether any filtering or windowing was done or should be done. (If < 0 then specifies what was done in the time domain) */
/*  REAL8   windowFilterParam1;         1st parameter to use in windowing or filtering */
/*  REAL8   windowFilterParam2;         2nd paramter to use in windowing or filtering */
/*  REAL8   windowFilterParam3;         3rd paramter to use in windowing or filtering */

/*  INT2    normalizationFlag;          Flag that specifies what normalization to do. If < 0 then specifies what normalization was already done. */
/*  REAL8   f0NRM;                      Start frequency to normalize over to create NRMs */
/*  REAL8   bandNRM;                    Band width to normalize over to create NRMs. */
/*  INT4    nBinsPerNRM;                Number of data points to normalize over to create to creat NRMs */

/*  INT2    testFlag;                   Specifies any tests or debugging to do; 0 means no test */

/*  UINT4   numSUMsPerCall;             If > 0 then = number of SUMs (parameter space points) to compute during each call to ApplySearch */
/*  INT2    outputFlag;                 Flag that specifies what to output; if > 0 then will output Sums into frame file */

/*  INT2    unusedFlag1;                Place holder for future flag */
/*  REAL8   unusedParam1;               Place holder for future parameter */
/*  INT2    unusedFlag2;                Place holder for future flag */
/*  REAL8   unusedParam2;               Place holder for future parameter */

/*  REAL8   startRA;                    Starting Right Ascension in radians */
/*  REAL8   stopRA;                     Ending Right Ascension in radians */
/*  REAL8   deltaRA;                    Right Ascension step size in radians */
/*  INT4    numRA;                      Number of RA to compute SUMs for */

/*  REAL8   startDec;                   Starting Declination in radians */
/*  REAL8   stopDec;                    Ending Declination in radians  */
/*  REAL8   deltaDec;                   Declination step size in radians */
/*  INT4    numDec;                     Number of Dec to compute SUMs for */

/*  INT4    numSpinDown;                Number of nonzero spindown parameters */

/*  REAL8   startFDeriv1;               Starting 1st deriv of freq in Hz/s */
/*  REAL8   stopFDeriv1;                Ending 1st deriv of freq in Hz/s */
/*  REAL8   deltaFDeriv1;               1st deriv of freq step size in Hz/s */
/*  INT4    numFDeriv1;                 Number of 1st derivs to compute SUMs for */

/*  REAL8   startFDeriv2;               Starting 2nd deriv of freq in Hz/s^2 */
/*  REAL8   stopFDeriv2;                Ending 2nd deriv of freq in Hz/s^2 */
/*  REAL8   deltaFDeriv2;               2nd deriv of freq step size in Hz/s^2 */
/*  INT4    numFDeriv2;                 Number of 2nd derivs to compute SUMs for */

/*  REAL8   startFDeriv3;               Starting 3rd deriv of freq in Hz/s^3 */
/*  REAL8   stopFDeriv3;                Ending 3rd deriv of freq in Hz/s^3 */
/*  REAL8   deltaFDeriv3;               3rd deriv of freq step size in Hz/s^3 */
/*  INT4    numFDeriv3;                 Number of 3rd derivs to compute SUMs for */

/*  REAL8   startFDeriv4;               Starting 4th deriv of freq in Hz/s^4 */
/*  REAL8   stopFDeriv4;                Ending 4th deriv of freq in Hz/s^4 */
/*  REAL8   deltaFDeriv4;               4th deriv of freq step size in Hz/s^4 */
/*  INT4    numFDeriv4;                 Number of 4th derivs to compute SUMs for */

/*  REAL8   startFDeriv5;               Starting 5th deriv of freq in Hz/s^5 */
/*  REAL8   stopFDeriv5;                Ending 5th deriv of freq in Hz/s^5 */
/*  REAL8   deltaFDeriv5;               5th deriv of freq step size in Hz/s^5 */
/*  INT4    numFDeriv5;                 Number of 5th derivs to compute SUMs for */

 /* Initialize */
 params->ifoNickName = NULL;
 params->IFO = NULL;
 params->patchName = NULL;

 for (i = 1; i < initSearchParams->argc; i++) {

	#ifdef DEBUG_STACKSLIDE
		/* printf("\nargv[%i] = %g\n",i,atof(initSearchParams->argv[i])); */
	#endif

	switch(i) {
		case  1: params->gpsStartTimeSec = (UINT4)atol(initSearchParams->argv[i]); break;
		case  2: params->gpsStartTimeNan = (UINT4)atol(initSearchParams->argv[i]); break;
		case  3: params->sampleRate = (REAL8)atof(initSearchParams->argv[i]); break;
		case  4: params->duration = (REAL8)atof(initSearchParams->argv[i]); break;

		case  5: params->numBLKs = (INT4)atol(initSearchParams->argv[i]); break;
		case  6: params->tBLK = (REAL8)atof(initSearchParams->argv[i]); break;
		case  7: params->tEffBLK = (REAL8)atof(initSearchParams->argv[i]); break;
		case  8: params->f0BLK = (REAL8)atof(initSearchParams->argv[i]); break;
		case  9: params->bandBLK = (REAL8)atof(initSearchParams->argv[i]); break;
		case 10: params->nBinsPerBLK = (INT4)atol(initSearchParams->argv[i]); break;

		case 11: params->numBLKsPerSTK = (INT4)atol(initSearchParams->argv[i]); break;
		case 12: params->tEffSTK = (REAL8)atof(initSearchParams->argv[i]); break;
		case 13: params->f0STK = (REAL8)atof(initSearchParams->argv[i]); break;
		case 14: params->bandSTK = (REAL8)atof(initSearchParams->argv[i]); break;
		case 15: params->nBinsPerSTK = (INT4)atol(initSearchParams->argv[i]); break;

		case 16: params->numSTKsPerSUM = (INT4)atol(initSearchParams->argv[i]); break;
		case 17: params->tEffSUM = (REAL8)atof(initSearchParams->argv[i]); break;
		case 18: params->f0SUM = (REAL8)atof(initSearchParams->argv[i]); break;
		case 19: params->bandSUM = (REAL8)atof(initSearchParams->argv[i]); break;
		case 20: params->nBinsPerSUM = (INT4)atol(initSearchParams->argv[i]); break;

		case 21: params->ifoNickName = (CHAR *) LALMalloc( (strlen( initSearchParams->argv[i] ) + 1) * sizeof(CHAR) );
		         strcpy(params->ifoNickName, initSearchParams->argv[i]); break;
		case 22: params->IFO = (CHAR *) LALMalloc( (strlen( initSearchParams->argv[i] ) + 1) * sizeof(CHAR) );
		         strcpy(params->IFO, initSearchParams->argv[i]); break;
		case 23: params->patchName = (CHAR *) LALMalloc( (strlen( initSearchParams->argv[i] ) + 1) * sizeof(CHAR) );
		         strcpy(params->patchName, initSearchParams->argv[i]); break;
		case 24: params->patchNumber = (INT4)atol(initSearchParams->argv[i]); break;

		case 25: strcpy(params->tDomainChannel, initSearchParams->argv[i]); break;
		case 26: strcpy(params->fDomainChannel, initSearchParams->argv[i]); break;

		case 27: params->inputDataTypeFlag = (INT2)atoi(initSearchParams->argv[i]); break;

		case 28: params->parameterSpaceFlag = (INT2)atoi(initSearchParams->argv[i]); break;

		case 29: params->stackTypeFlag = (INT2)atoi(initSearchParams->argv[i]); break;
		case 30: params->Dterms = (INT4)atoi(initSearchParams->argv[i]); break;

		case 31: params->thresholdFlag = (INT2)atoi(initSearchParams->argv[i]); break;
		case 32: params->threshold1 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 33: params->threshold2 = (REAL8)atof(initSearchParams->argv[i]); break;

		case 34: params->calibrationFlag = (INT2)atoi(initSearchParams->argv[i]); break;

		case 35: params->weightFlag = (INT2)atoi(initSearchParams->argv[i]); break;

		case 36: params->windowFilterFlag = (INT2)atoi(initSearchParams->argv[i]); break;
		case 37: params->windowFilterParam1 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 38: params->windowFilterParam2 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 39: params->windowFilterParam3 = (REAL8)atof(initSearchParams->argv[i]); break;

		case 40: params->normalizationFlag = (INT2)atoi(initSearchParams->argv[i]); break;
		case 41: params->f0NRM = (REAL8)atof(initSearchParams->argv[i]); break;
		case 42: params->bandNRM = (REAL8)atof(initSearchParams->argv[i]); break;
		case 43: params->nBinsPerNRM = (INT4)atol(initSearchParams->argv[i]); break;

		case 44: params->testFlag = (INT2)atoi(initSearchParams->argv[i]); break;

	        case 45: params->numSUMsPerCall = (UINT4)atol(initSearchParams->argv[i]); break;

		case 46: params->outputFlag = (INT2)atoi(initSearchParams->argv[i]); break;

		case 47: params->unusedFlag1 = (INT2)atoi(initSearchParams->argv[i]); break;
		case 48: params->unusedParam1 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 49: params->unusedFlag2 = (INT2)atoi(initSearchParams->argv[i]); break;
		case 50: params->unusedParam2 = (REAL8)atof(initSearchParams->argv[i]); break;

		case 51: params->startRA = (REAL8)atof(initSearchParams->argv[i]); break;
		case 52: params->stopRA = (REAL8)atof(initSearchParams->argv[i]); break;
		case 53: params->deltaRA = (REAL8)atof(initSearchParams->argv[i]); break;
		case 54: params->numRA = (INT4)atol(initSearchParams->argv[i]); break;

		case 55: params->startDec = (REAL8)atof(initSearchParams->argv[i]); break;
		case 56: params->stopDec = (REAL8)atof(initSearchParams->argv[i]); break;
		case 57: params->deltaDec = (REAL8)atof(initSearchParams->argv[i]); break;
		case 58: params->numDec = (INT4)atol(initSearchParams->argv[i]); break;

		case 59: params->numSpinDown = (INT4)atol(initSearchParams->argv[i]); break;

		case 60: params->startFDeriv1 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 61: params->stopFDeriv1 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 62: params->deltaFDeriv1 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 63: params->numFDeriv1 = (INT4)atol(initSearchParams->argv[i]); break;

		case 64: params->startFDeriv2 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 65: params->stopFDeriv2 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 66: params->deltaFDeriv2 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 67: params->numFDeriv2 = (INT4)atol(initSearchParams->argv[i]); break;

		case 68: params->startFDeriv3 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 69: params->stopFDeriv3 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 70: params->deltaFDeriv3 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 71: params->numFDeriv3 = (INT4)atol(initSearchParams->argv[i]); break;

		case 72: params->startFDeriv4 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 73: params->stopFDeriv4 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 74: params->deltaFDeriv4 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 75: params->numFDeriv4 = (INT4)atol(initSearchParams->argv[i]); break;

		case 76: params->startFDeriv5 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 77: params->stopFDeriv5 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 78: params->deltaFDeriv5 = (REAL8)atof(initSearchParams->argv[i]); break;
		case 79: params->numFDeriv5 = (INT4)atol(initSearchParams->argv[i]); break;

	}
  } /* end for (i=1; i<argc; i++)  */

 #ifdef DEBUG_INPUTBLK_CODE
	fprintf(stdout,"\n\nparams->numBLKs = %i \n", params->numBLKs);
	fprintf(stdout,"params->nBinsPerBLK = %i \n", params->nBinsPerBLK);
 	fprintf(stdout,"params->f0BLK = %g \n", params->f0BLK);
  	fprintf(stdout,"params->bandBLK = %g \n", params->bandBLK);
   	fflush(stdout);
 #endif

  params->deltaT = 1.0/params->sampleRate;
  params->totalNumTimeSamples = floor(params->sampleRate*params->duration);

  params->numBLKsPerDuration = floor(params->duration/params->tBLK);  /* equals duration/tBLK; but not acual number of blocks if gaps exist in the data */
  params->nSamplesPerBLK = floor(params->sampleRate*params->tBLK);
  params->dfBLK = 1.0/params->tEffBLK;

  /* params->numSTKs is not computed here. Instead computed when setting up timeStamps. Not duration/tSTK if gaps are present */
  params->tSTK = params->tBLK*params->numBLKsPerSTK;       /* duration in seconds of STKs = tBLK*numBLKsPerSTK */
  params->dfSTK = 1.0/params->tEffSTK;                     /* Freq resolution of STK = 1.0/tEffSTK. Could be different from 1/tSTK due to oversampling  */

  params->tSUM = params->tSTK*params->numSTKsPerSUM; /* duration in seconds of output SUMs = tSTK*numSTKsPerSUM. (Usually = duration) */
  params->numSUMsPerParamSpacePt = params->duration/params->tSUM; /* Number of output SUMs per parameter space point = params->duration/params->tSUM. (Usually will = 1) */
  params->dfSUM = 1.0/params->tEffSUM;  /* Freq resolution of output SUMs.  dfSUM = 1.0/tEffSUM. Could be different from dfSTK in some algorithms?  */

  /* Initialize and set up number of points in the parameter space */
  params->numSkyPosTotal = 0;     /* Total number of Sky positions to cover */
  params->numFreqDerivTotal = 0;  /* Total number of Frequency evolution models to cover */
  params->numParamSpacePts = 0;   /* Total number of points in the parameter space to cover */
  if (params->parameterSpaceFlag < 2) {
     params->numSkyPosTotal = params->numRA*params->numDec;
     for(i=0;i<params->numSpinDown;i++)
	{
	        switch(i) {
		   case  0: params->numFreqDerivTotal = params->numFDeriv1; break;
		   case  1: params->numFreqDerivTotal *= params->numFDeriv2; break;
		   case  2: params->numFreqDerivTotal *= params->numFDeriv3; break;
		   case  3: params->numFreqDerivTotal *= params->numFDeriv4; break;
		   case  4: params->numFreqDerivTotal *= params->numFDeriv5; break;
		}
     }
     if (params->numSpinDown > 0) {
        params->numParamSpacePts = params->numSkyPosTotal*params->numFreqDerivTotal;
     } else {
        params->numParamSpacePts = params->numSkyPosTotal;
     }
  }
  params->numSUMsTotal = params->numSUMsPerParamSpacePt*params->numParamSpacePts;  /* Total Number of Sums = numSUMsPerParamPt*numParamSpacePts */

 #ifdef DEBUG_PARAMETERS
    	fprintf(stdout,"\n");
    	fprintf(stdout,"case  1: params->gpsStartTimeSec = %i \n", params->gpsStartTimeSec);
    	fprintf(stdout,"case  2: params->gpsStartTimeNan = %i \n", params->gpsStartTimeNan);
    	fprintf(stdout,"case  3: params->sampleRate = %g \n", params->sampleRate);
    	fprintf(stdout,"case  4: params->duration = %g \n", params->duration);
    	fprintf(stdout,"\n");
    	fprintf(stdout,"case  5: params->numBLKs = %i \n", params->numBLKs);
    	fprintf(stdout,"case  6: params->tBLK = %g \n", params->tBLK);
    	fprintf(stdout,"case  7: params->tEffBLK = %g \n", params->tEffBLK);
    	fprintf(stdout,"case  8: params->f0BLK = %g \n", params->f0BLK);
    	fprintf(stdout,"case  9: params->bandBLK =  %g \n", params->bandBLK);
    	fprintf(stdout,"case 10: params->nBinsPerBLK = %i \n", params->nBinsPerBLK);
    	fprintf(stdout,"\n");
    	fprintf(stdout,"case 11: params->numBLKsPerSTK = %i \n", params->numBLKsPerSTK);
    	fprintf(stdout,"case 12: params->tEffSTK = %g \n", params->tEffSTK);
    	fprintf(stdout,"case 13: params->f0STK =  %g \n", params->f0STK);
    	fprintf(stdout,"case 14: params->bandSTK =  %g \n", params->bandSTK);
    	fprintf(stdout,"case 15: params->nBinsPerSTK = %i \n", params->nBinsPerSTK);
    	fprintf(stdout,"\n");
    	fprintf(stdout,"case 16: params->numSTKsPerSUM = %i \n", params->numSTKsPerSUM);
    	fprintf(stdout,"case 17: params->tEffSUM = %g \n", params->tEffSUM);
    	fprintf(stdout,"case 18: params->f0SUM = %g \n", params->f0SUM);
    	fprintf(stdout,"case 19: params->bandSUM =  %g \n", params->bandSUM);
    	fprintf(stdout,"case 20: params->nBinsPerSUM = %i \n", params->nBinsPerSUM);
    	fprintf(stdout,"\n");
    	fprintf(stdout,"case 21: params->ifoNickName = %s \n", params->ifoNickName);
    	fprintf(stdout,"case 22: params->IFO = %s \n", params->IFO);
    	fprintf(stdout,"case 23: params->patchName = %s \n", params->patchName);
    	fprintf(stdout,"case 24: params->patchNumber = %i \n", params->patchNumber);
    	fprintf(stdout,"\n");
    	fprintf(stdout,"case 25: params->tDomainChannel = %s \n", params->tDomainChannel);
    	fprintf(stdout,"case 26: params->fDomainChannel = %s \n", params->fDomainChannel);
    	fprintf(stdout,"\n");
    	fprintf(stdout,"case 27: params->inputDataTypeFlag = %i \n", params->inputDataTypeFlag);
    	fprintf(stdout,"\n");
    	fprintf(stdout,"case 28: params->parameterSpaceFlag = %i \n", params->parameterSpaceFlag);
    	fprintf(stdout,"\n");
    	fprintf(stdout,"case 29: params->stackTypeFlag =  %i \n", params->stackTypeFlag);
    	fprintf(stdout,"case 30: params->Dterms = %i \n", params->Dterms);
    	fprintf(stdout,"\n");
    	fprintf(stdout,"case 31: params->thresholdFlag = %i \n", params->thresholdFlag);
    	fprintf(stdout,"case 32: params->threshold1 = %g \n", params->threshold1);
    	fprintf(stdout,"case 33: params->threshold2 = %g \n", params->threshold2);
    	fprintf(stdout,"\n");
    	fprintf(stdout,"case 34: params->calibrationFlag = %i \n", params->calibrationFlag);
    	fprintf(stdout,"\n");
    	fprintf(stdout,"case 35: params->weightFlag = %i \n", params->weightFlag);
    	fprintf(stdout,"\n");
    	fprintf(stdout,"case 36: params->windowFilterFlag = %i \n", params->windowFilterFlag);
    	fprintf(stdout,"case 37: params->windowFilterParam1 = %g \n", params->windowFilterParam1);
    	fprintf(stdout,"case 38: params->windowFilterParam2 = %g \n", params->windowFilterParam2);
    	fprintf(stdout,"case 39: params->windowFilterParam3 = %g \n", params->windowFilterParam3);
    	fprintf(stdout,"\n");
    	fprintf(stdout,"case 40: params->normalizationFlag = %i \n", params->normalizationFlag);
    	fprintf(stdout,"case 41: params->f0NRM = %g \n", params->f0NRM);
    	fprintf(stdout,"case 42: params->bandNRM = %g \n", params->bandNRM);
    	fprintf(stdout,"case 43: params->nBinsPerNRM = %i \n", params->nBinsPerNRM);
    	fprintf(stdout,"\n");
    	fprintf(stdout,"case 44: params->testFlag = %i \n", params->testFlag);
    	fprintf(stdout,"\n");
    	fprintf(stdout,"case 45: params->numSUMsPerCall = %i \n", params->numSUMsPerCall);
    	fprintf(stdout,"\n");
    	fprintf(stdout,"case 46: params->outputFlag = %i \n", params->outputFlag);
    	fprintf(stdout,"\n");
    	fprintf(stdout,"case 47: params->unusedFlag1 = %i \n", params->unusedFlag1);
    	fprintf(stdout,"case 48: params->unusedParam1 = %g \n", params->unusedParam1);
    	fprintf(stdout,"case 49: params->unusedFlag2 = %i \n", params->unusedFlag2);
    	fprintf(stdout,"case 50: params->unusedParam2 = %g \n", params->unusedParam2);
    	fprintf(stdout,"\n");
    	fprintf(stdout,"case 51: params->startRA = %g \n", params->startRA);
    	fprintf(stdout,"case 52: params->stopRA = %g \n", params->stopRA);
    	fprintf(stdout,"case 53: params->deltaRA = %g \n", params->deltaRA);
    	fprintf(stdout,"case 54: params->numRA = %i \n", params->numRA);
    	fprintf(stdout,"\n");
    	fprintf(stdout,"case 55: params->startDec = %g \n", params->startDec);
    	fprintf(stdout,"case 56: params->stopDec =  %g \n", params->stopDec);
    	fprintf(stdout,"case 57: params->deltaDec = %g \n", params->deltaDec);
    	fprintf(stdout,"case 58: params->numDec = %i \n", params->numDec);
    	fprintf(stdout,"\n");
    	fprintf(stdout,"case 59: params->numSpinDown = %i \n", params->numSpinDown);
    	fprintf(stdout,"\n");
    	fprintf(stdout,"case 60: params->startFDeriv1 = %g \n", params->startFDeriv1);
    	fprintf(stdout,"case 61: params->stopFDeriv1 = %g \n", params->stopFDeriv1);
    	fprintf(stdout,"case 62: params->deltaFDeriv1 = %g \n", params->deltaFDeriv1);
	fprintf(stdout,"case 63: params->numFDeriv1 = %i \n", params->numFDeriv1);
    	fprintf(stdout,"\n");
    	fprintf(stdout,"case 64: params->startFDeriv2 = %g \n", params->startFDeriv2);
    	fprintf(stdout,"case 65: params->stopFDeriv2 = %g \n", params->stopFDeriv2);
    	fprintf(stdout,"case 66: params->deltaFDeriv2 = %g \n", params->deltaFDeriv2);
    	fprintf(stdout,"case 67: params->numFDeriv2 = %i \n", params->numFDeriv2);
    	fprintf(stdout,"\n");
    	fprintf(stdout,"case 68: params->startFDeriv3 = %g \n", params->startFDeriv3);
    	fprintf(stdout,"case 69: params->stopFDeriv3 = %g \n", params->stopFDeriv3);
    	fprintf(stdout,"case 70: params->deltaFDeriv3 = %g \n", params->deltaFDeriv3);
	fprintf(stdout,"case 71: params->numFDeriv3 = %i \n", params->numFDeriv3);
    	fprintf(stdout,"\n");
    	fprintf(stdout,"case 72: params->startFDeriv4 = %g \n", params->startFDeriv4);
    	fprintf(stdout,"case 73: params->stopFDeriv4 = %g \n", params->stopFDeriv4);
    	fprintf(stdout,"case 74: params->deltaFDeriv4 = %g \n", params->deltaFDeriv4);
	fprintf(stdout,"case 75: params->numFDeriv4 = %i \n", params->numFDeriv4);
    	fprintf(stdout,"\n");
    	fprintf(stdout,"case 76: params->startFDeriv5 = %g \n", params->startFDeriv5);
    	fprintf(stdout,"case 77: params->stopFDeriv5 = %g \n", params->stopFDeriv5);
    	fprintf(stdout,"case 78: params->deltaFDeriv5 = %g \n", params->deltaFDeriv5);
	fprintf(stdout,"case 79: params->numFDeriv5 = %i \n", params->numFDeriv5);
    	fprintf(stdout,"\n");
	fprintf(stdout,"params->deltaT = %g \n", params->deltaT);
	fprintf(stdout,"params->totalNumTimeSamples = %i \n", params->totalNumTimeSamples);
	fprintf(stdout,"params->numBLKsPerDuration = %i \n", params->numBLKsPerDuration);
	fprintf(stdout,"params->nSamplesPerBLK = %i \n", params->nSamplesPerBLK);
	fprintf(stdout,"params->dfBLK = %g \n", params->dfBLK);
	fprintf(stdout,"params->tSTK = %g \n", params->tSTK);
	fprintf(stdout,"params->dfSTK = %g \n", params->dfSTK);
	fprintf(stdout,"params->tSUM = %g \n", params->tSUM);
	fprintf(stdout,"params->numSUMsPerParamSpacePt = %i \n", params->numSUMsPerParamSpacePt);
	fprintf(stdout,"params->dfSUM = %g \n", params->dfSUM);
	fprintf(stdout,"params->numSkyPosTotal = %i \n", params->numSkyPosTotal);
	fprintf(stdout,"params->numFreqDerivTotal = %i \n", params->numFreqDerivTotal);
	fprintf(stdout,"params->numParamSpacePts = %i \n", params->numParamSpacePts);
	fprintf(stdout,"params->numSUMsTotal = %i \n", params->numSUMsTotal);
    	fprintf(stdout,"\n");
        fflush(stdout);
 #endif

/******************************************/
/*                                        */
/* END SECTION: parameters passed as      */
/* arguments.  Those indented are         */
/* computed from these.                   */
/*                                        */
/******************************************/

/**********************************************/
/*                                            */
/* START SECTION: initialize other parameters */
/*                                            */
/**********************************************/

  #ifdef DEBUG_STACKSLIDE
  	fprintf(stdout, "\nSTART SECTION: initialize other parameters\n");
  	fflush(stdout);
  #endif

  /* Set up name of this DSO */
  params->dsoName = NULL;
  params->dsoName = (CHAR *) LALMalloc( (strlen( "STACKSLIDE" ) + 1) * sizeof(CHAR) );
  strcpy(params->dsoName, "STACKSLIDE");

  params->searchMaster = initSearchParams->nodeClass;
  params->rank         = 0;
  params->numSlaves    = 0;
  params->curSlaves    = 0;
  params->numNodes     = 0;
  params->numDBOutput  = 0;  /* Total number of rows going into all databases is calculated. */

  /* parameters for keeping track of which BLKs, STKs, SUMs to work on; which are done. */
  params->firstK = 0;        /* Index used to keep track of which BLK, STK, or SUM to work on */
  params->lastK = 0;         /* Index used to keep track of which BLK, STK, or SUM to work on */
  params->finishedBLKs = 0;  /* Set equal to true when all BLKS for this job have been found in input data */
  params->finishedSTKs = 0;  /* Set equal to true when all STKs for this job have been created */
  params->finishedSUMs = 0;  /* Set equal to true when all BLKS for this job have been created */

  params->iMinBLK = 0;       /* Index of minimum frequency in BLK band */
  params->iMaxBLK = 0;       /* Index of maximum frequency in BLK band */

  params->edat = NULL;       /* Initialize pointer to ephemeris data */

/**********************************************/
/*                                            */
/* END SECTION: initialize other parameters   */
/*                                            */
/**********************************************/

/**********************************************/
/*                                            */
/* START SECTION: validate parameters         */
/*                                            */
/**********************************************/

  #ifdef DEBUG_STACKSLIDE
  	fprintf(stdout, "\nSTART SECTION: validate parameters\n");
  	fflush(stdout);
  #endif

  if (params->deltaT <= 0.0) {
    ABORT( status, STACKSLIDEH_EDELTAT, STACKSLIDEH_MSGEDELTAT);
  }

  if (params->totalNumTimeSamples <= 0) {
    ABORT( status, STACKSLIDEH_EINPUTN, STACKSLIDEH_MSGEINPUTN);
  }

  if (params->tBLK <= 0.0 || params->tBLK > params->duration) {
    ABORT( status, STACKSLIDEH_ETBLK, STACKSLIDEH_MSGETBLK);
  }

  if (params->nSamplesPerBLK <= 0 || params->nSamplesPerBLK > (INT4)params->totalNumTimeSamples) {
    ABORT( status, STACKSLIDEH_ENBLK, STACKSLIDEH_MSGENBLK);
  }

  if (params->numSUMsPerCall <= 0) {
    ABORT( status, STACKSLIDEH_EMCOHBLKPERCALL, STACKSLIDEH_MSGEMCOHBLKPERCALL);
  }

  if (params->startRA < 0 || params->startRA > (REAL8)LAL_TWOPI) {
    ABORT( status, STACKSLIDEH_ERA, STACKSLIDEH_MSGERA);
  }

  if (params->startDec < -1.0*(REAL8)LAL_PI_2 || params->startDec > (REAL8)LAL_PI_2) {
    ABORT( status, STACKSLIDEH_EDEC, STACKSLIDEH_MSGEDEC);
  }

  /* "Invalid or null ifoNickName" */
  if ( !(params->ifoNickName) ){
    ABORT( status, STACKSLIDEH_EIFONICKNAME, STACKSLIDEH_MSGEIFONICKNAME);
  }

  /* "Invalid or null IFO" */
  if ( !(params->IFO) ){
    ABORT( status, STACKSLIDEH_EIFO, STACKSLIDEH_MSGEIFO);
  }

  /* "Invalid or null Target Name" */
  if ( !(params->patchName) ){
    ABORT( status, STACKSLIDEH_ETARGETNAME, STACKSLIDEH_MSGETARGETNAME);
  }

  /* "Invalid or null channel" */
  if ( strstr(params->tDomainChannel, "NotSet") ){
    ABORT( status, STACKSLIDEH_ECHANNEL, STACKSLIDEH_MSGECHANNEL);
  }

/**********************************************/
/*                                            */
/* END SECTION: validate parameters           */
/*                                            */
/**********************************************/

  RETURN( status );

}
/******************************************/
/*                                        */
/* END FUNCTION: LALInitSearch            */
/*                                        */
/******************************************/

/******************************************/
/*                                        */
/* START FUNCTION: LALConditionData       */
/*                                        */
/******************************************/
void
LALConditionData(
    LALStatus             *status,
    LALSearchInput        *inout,
    void                  *searchParams,
    LALMPIParams	  *mpiParams
    )
{

 StackSlideSearchParams* const params = (StackSlideSearchParams *) searchParams;

 INITSTATUS( status, "LALConditionData", STACKSLIDEC );
 ATTATCHSTATUSPTR (status);

  #ifdef DEBUG_STACKSLIDE
  	fprintf(stdout, "\nSTART FUNCTION: LALConditionData\n");
  	fflush(stdout);
  #endif

 MPI_Comm_rank( *(mpiParams->comm), &params->rank );

 MPI_Comm_size( *(mpiParams->comm), &params->numNodes);

 /* !!!!!!!!!!!!!!!!!!! TO START WITH, WILL RUN CODE ONLY ON THE SEARCH MASTER !!!!!!!!!!!!!!!!! */
 /* !!!!!!! LATER, WILL RUN ON THE SLAVES, AND ADD COMMUNICATION OF RESULTS BACK TO MASTER !!!!! */
 if (params->searchMaster) {

  BOOLEAN inputBLKsFound = 0; /* BLK data was found in inout->sequences[qc1Index].data.complex8 */ /* 07/07/02 gam */
  UINT4 i = 0;      /* all purpose index */
  UINT4 k = 0;      /* another all purpose index */
  UINT4 nData = 0;  /* number of data points from inout->sequences[i].dimensions[0] */

/**********************************************/
/*                                            */
/* START SECTION: find data                   */
/*                                            */
/**********************************************/

  #ifdef DEBUG_STACKSLIDE
  	fprintf(stdout, "\nSTART SECTION: find data\n");
  	fflush(stdout);
  #endif

  for (i = 0; i < inout->numberSequences; ++i)
  {
    if (strstr(inout->sequences[i].comment, "sft"))
    {
        /* NOTE HARD CODED COMMENT ABOVE; SHOULD CHANGE */ /* BLK data was found. Check that this is what was expected and set flags */

        #ifdef DEBUG_INPUTBLK_CODE
    	  fprintf(stdout,"\n FOUND BLK DATA \n");
          fflush(stdout);
        #endif

        inputBLKsFound = 1;   /* Set flag to unpack the data below after allocating memory */

	/* HARD CODE units FOR NOW */
	{
	   LALUnitPair  unitPair;   /* A pair of LALUnits */
	   RAT4         unitPower;  /* Rational number and demoninator minus one to compute power on a LALUnit */
           LALUnit     unitTmp;     /* Temporary place holder for units. */

	   /*  inputDataTypeFlag: 0 = SFTs, 1 = PSDs, 2 = F-stat */
           if (params->inputDataTypeFlag == 2 || params->normalizationFlag < 0) {
	       /* Input data is F-stat or already normalized. */
	       params->unitBLK = lalDimensionlessUnit; /* BLKData is dimensionless */
           } else {
	     /* Input data is unnormalized SFT or PSD data */
             if (params->calibrationFlag < 0) {
                params->unitBLK = lalStrainUnit;  /* Input data is already calibrated */
	     } else {
                params->unitBLK = lalADCCountUnit; /* Input data is uncalibrated */
	     }
  	     unitPower.numerator = -1;
  	     unitPower.denominatorMinusOne = 1;
  	     LALUnitRaise(status->statusPtr, &unitTmp, &lalHertzUnit, &unitPower );
    	     CHECKSTATUSPTR (status);
	     unitPair.unitOne = &(params->unitBLK);
	     unitPair.unitTwo = &unitTmp;
  	     LALUnitMultiply(status->statusPtr, &params->unitBLK, &unitPair);
    	     CHECKSTATUSPTR (status);
             if (params->inputDataTypeFlag == 1) {
	        /* Input data is PSD data; need to square the units of SFTs */
	        unitPair.unitOne = &(params->unitBLK);
	        unitPair.unitTwo = &(params->unitBLK);
  	        LALUnitMultiply(status->statusPtr, &params->unitBLK, &unitPair);
    	        CHECKSTATUSPTR (status);
	     }
	   }
	}

        break;  /* Will unpack BLKs below */

    }
    /* END if (strstr(inout->sequences[i].comment, "sft")) else if ... */
 }
 /* END for (i = 0; i < inout->numberSequences; ++i) */

/**********************************************/
/*                                            */
/* END SECTION: find data                     */
/*                                            */
/**********************************************/

/**********************************************/
/*                                            */
/* START SECTION: unpack BLK data             */
/*                                            */
/**********************************************/

  #ifdef DEBUG_STACKSLIDE
  	fprintf(stdout, "\nSTART SECTION: unpack BLK data\n");
  	fflush(stdout);
  #endif

 if (inputBLKsFound) {

      #ifdef DEBUG_INPUTBLK_CODE
    	fprintf(stdout,"\n START UNPACKING BLK DATA \n\n");
        fflush(stdout);
      #endif

      /* These temporary constants are used below when setting BLK times */
      REAL8 tempReal8tBLK1 = 0.0;
      REAL8 tempReal8tBLK2 = 0.0;
      LIGOTimeGPS tempLIGOTimeGPStBLK;

      params->iMinBLK = 0;
      params->iMaxBLK = params->nBinsPerBLK - 1;

      #ifdef DEBUG_INPUTBLK_CODE
       fprintf(stdout,"\n\niMinBLK = %i, iMaxBLK = %i \n",params->iMinBLK, params->iMaxBLK);
       fprintf(stdout,"numBLKs = %i \n",params->numBLKs);
       fflush(stdout);
      #endif

      params->timeIntervals =(gpsTimeInterval *)LALCalloc(params->numBLKs, sizeof(gpsTimeInterval));

      params->timeStamps=(LIGOTimeGPS *)LALMalloc(params->numBLKs*sizeof(LIGOTimeGPS));

      if (params->inputDataTypeFlag == 0) {
        /* Input data is SFTs */

        /* Allocate memory for the BLKData structure */
    	params->BLKData=(FFT **)LALMalloc(params->numBLKs*sizeof(FFT *));
    	for(i=0;i<params->numBLKs;i++)
    	{
		params->BLKData[i]=(FFT*)LALMalloc(sizeof(FFT));
		params->BLKData[i]->fft=(COMPLEX8FrequencySeries *)LALMalloc(sizeof(COMPLEX8FrequencySeries));
		params->BLKData[i]->fft->data=(COMPLEX8Vector *)LALMalloc(sizeof(COMPLEX8Vector));
		params->BLKData[i]->fft->data->data=(COMPLEX8 *)LALMalloc(params->nBinsPerBLK*sizeof(COMPLEX8));
    	}

         params->whichSTK = 0;  /* which STK does BLK go with  */
         params->lastWhichSTK = -1;  /* Last value of params->whichSTK does BLK go with  */
         params->numSTKs = 0; /* Initialize to 0; will count the actual number of stacks below. Not duration/tSTK if gaps are present */

         /* Loop through sequence again to find BLK data */
	 k = 0;  /* Use this as a counter of BLKs in the next loop. */
         params->firstK = 0;
         for (i = 0; i < inout->numberSequences; ++i)
         {
            if (strstr(inout->sequences[i].comment, "sft")) {
               nData = inout->sequences[i].dimensions[0];
	       params->lastK = params->firstK + nData/params->nBinsPerBLK;
	       if (params->lastK > params->numBLKs) params->lastK = params->numBLKs;
    	       for(k=params->firstK; k<params->lastK; k++)
    	       {
                 memcpy(params->BLKData[k]->fft->data->data, (inout->sequences[i].data.complex8 + (k - params->firstK)*params->nBinsPerBLK), params->nBinsPerBLK*sizeof(COMPLEX8));

                 /* START Assign meta data based on that input from LDAS */
	  	 if (k == params->firstK) {
		 	params->timeStamps[k].gpsSeconds = inout->sequences[i].range.dFreq.gpsStartTimeSec;
		 	params->timeStamps[k].gpsNanoSeconds = inout->sequences[i].range.dFreq.gpsStartTimeNan;

			TimeToFloat(&tempReal8tBLK1, &(params->timeStamps[k]));  /* Save this for k > firstK */

		 	params->timeIntervals[k].startSec = params->timeStamps[k].gpsSeconds;
                 	params->timeIntervals[k].startNan = params->timeStamps[k].gpsNanoSeconds;
                 	params->timeIntervals[k].stopSec = inout->sequences[i].range.dFreq.gpsStopTimeSec;
                 	params->timeIntervals[k].stopNan = inout->sequences[i].range.dFreq.gpsStopTimeNan;

			/* Verify meta data is correct */
     			if (inout->sequences[i].range.dFreq.startFreq != params->f0BLK) {
    				ABORT( status, STACKSLIDEH_ESTARTFREQ, STACKSLIDEH_MSGESTARTFREQ);
			}

     			if (inout->sequences[i].range.dFreq.freqStepSize != params->dfBLK) {
    				ABORT( status, STACKSLIDEH_EFREQSTEPSIZE, STACKSLIDEH_MSGEFREQSTEPSIZE);
			}

                 } else {
		        /* This is not used if BLK data arrives in separate containers for each BLK; only if concatinated together*/
          		tempReal8tBLK2 = tempReal8tBLK1 + ((REAL8)(k - params->firstK))*params->tBLK;
          		FloatToTime(&tempLIGOTimeGPStBLK, &tempReal8tBLK2);
		 	params->timeStamps[k].gpsSeconds = tempLIGOTimeGPStBLK.gpsSeconds;
		 	params->timeStamps[k].gpsNanoSeconds = tempLIGOTimeGPStBLK.gpsNanoSeconds;
          		tempReal8tBLK2 = tempReal8tBLK2 + params->tBLK;
          		FloatToTime(&tempLIGOTimeGPStBLK, &tempReal8tBLK2);
		 	params->timeIntervals[k].startSec = params->timeStamps[k].gpsSeconds;
                 	params->timeIntervals[k].startNan = params->timeStamps[k].gpsNanoSeconds;
                 	params->timeIntervals[k].stopSec = tempLIGOTimeGPStBLK.gpsSeconds;
                 	params->timeIntervals[k].stopNan = tempLIGOTimeGPStBLK.gpsNanoSeconds;
		 }
                 /* END Assign meta data based on that input from LDAS */

		 params->BLKData[k]->fft->epoch=params->timeStamps[k];
		 params->BLKData[k]->fft->f0=params->f0BLK;
		 params->BLKData[k]->fft->deltaF=params->dfBLK;
		 params->BLKData[k]->fft->data->length=params->nBinsPerBLK;

	         /* Which STK does this BKL go with */
	         params->whichSTK = (params->BLKData[k]->fft->epoch.gpsSeconds - params->gpsStartTimeSec)/params->tSTK;
	         if (params->whichSTK > params->lastWhichSTK) {
                     params->numSTKs++;
	         }
                 #ifdef DEBUG_STACKSLIDE
	                fprintf(stdout,"Found BLK # %i, starts at %i; belongs to STK # %i; total STKs so far = %i \n",k,params->BLKData[k]->fft->epoch.gpsSeconds,params->whichSTK,params->numSTKs);
                        fflush(stdout);
                 #endif
                 params->lastWhichSTK = params->whichSTK;
	       }
	       params->firstK = params->lastK;
	    }
            if (params->firstK >= params->numBLKs) {
	      	params->finishedBLKs = 1;
	      	break;
	    }
	 }
         /* END reloop through sequences for BLK data: for (i = 0; i < inout->numberSequences; ++i) */
      } /* END if (params->inputDataTypeFlag == 0) */

      /* Some requested input BLK data is missing */
      if (!params->finishedBLKs) {
    	 ABORT( status, STACKSLIDEH_EMISSINGBLKDATA, STACKSLIDEH_MSGEMISSINGBLKDATA);
      }

      /* Set these times based on time intervals actually found (allow for missing BLKs) */
      params->times.startSec = params->timeIntervals[0].startSec;
      params->times.startNan = params->timeIntervals[0].startNan;
      params->times.stopSec = params->timeIntervals[params->numBLKs - 1].stopSec;
      params->times.stopNan = params->timeIntervals[params->numBLKs -1].stopNan;
      params->times.timeStepSize = params->deltaT;

      #ifdef DEBUG_STACKSLIDE
	 fprintf(stdout,"\nCheck that additional parameters are set up OK: \n");
	 fprintf(stdout,"\nparams->numSTKs = %i \n", params->numSTKs);
         fprintf(stdout,"\ntimes.startSec = %i \n", params->times.startSec);
         fprintf(stdout,"times.startNan = %i \n", params->times.startNan);
         fflush(stdout);
      #endif

      #ifdef DEBUG_INPUTBLK_CODE
    	fprintf(stdout,"\n FINISHED UNPACKING BLK DATA \n\n");
        fflush(stdout);
      #endif
 } else {
      /* "No input data was found" */
      ABORT( status, STACKSLIDEH_ENODATA, STACKSLIDEH_MSGENODATA);
 }

/**********************************************/
/*                                            */
/* END SECTION: unpack BLK data               */
/*                                            */
/**********************************************/

/**********************************************/
/*                                            */
/* START SECTION: unpack ephemeris data       */
/*                                            */
/**********************************************/

  #ifdef DEBUG_STACKSLIDE
  	fprintf(stdout, "\nSTART SECTION: unpack ephemeris data\n");
  	fflush(stdout);
  #endif

      /* Call LALWRAPPERInitBarycenter if INCLUDE_LALWRAPPERINITBARYCENTER; Otherwise call LALInitBarycenter if INCLUDE_LALINITBARYCENTER */
      /* Only one of INCLUDE_LALWRAPPERINITBARYCENTER and INCLUDE_LALINITBARYCENTER should be defined. */
      #ifdef INCLUDE_LALWRAPPERINITBARYCENTER

	  params->edat = (EphemerisData *)LALMalloc(sizeof(EphemerisData));

	   /* Call function to transfer ephemeris data input as ilwd from LALSearchInput structure into LAL structures; based on LALInitBarycenter. */
	  LALWRAPPERInitBarycenter(status->statusPtr, inout, params->edat);
	  CHECKSTATUSPTR (status);

      #endif
      #ifdef INCLUDE_LALINITBARYCENTER
          /* For test purposes use copy of LALInitBarycenter.c from the LAL support package.  Author: Curt Cutler */

          params->edat = (EphemerisData *)LALMalloc(sizeof(EphemerisData));

	  /* NOTE: when using LALInitBarycenter.c for test purposes must hard code input earth and sun file names here. */
	  (*params->edat).ephiles.earthEphemeris = "/home/gmendell/searchcode/share/lal/earth03.dat";
          (*params->edat).ephiles.sunEphemeris = "/home/gmendell/searchcode/share/lal/sun03.dat";

          LALInitBarycenter(status->statusPtr, params->edat);
    	  CHECKSTATUSPTR (status);
     #endif

/**********************************************/
/*                                            */
/* END SECTION: unpack ephemeris data         */
/*                                            */
/**********************************************/

 }
 /* END if (params->searchMaster) */

 RETURN( status );

}
/******************************************/
/*                                        */
/* END FUNCTION: LALConditionData         */
/*                                        */
/******************************************/

/******************************************/
/*                                        */
/* START FUNCTION: LALApplySearch         */
/*                                        */
/******************************************/
void
LALApplySearch(
    LALStatus             *status,
    LALSearchOutput       *output,
    LALSearchInput        *input,
    LALApplySearchParams  *applyParams
    )
{
  StackSlideSearchParams* const params = (StackSlideSearchParams *) applyParams->searchParams;

  UINT4          i = 0;         /* all purpose index */
  UINT4          k = 0;         /* another all purpose index */

  INITSTATUS( status, "LALApplySearch", STACKSLIDEC );
  ATTATCHSTATUSPTR(status);

  #ifdef DEBUG_STACKSLIDE
  	fprintf(stdout, "\nSTART FUNCTION: LALApplySearch\n");
  	fflush(stdout);
  #endif

  if ( !(output) ){
    ABORT( status, STACKSLIDEH_ENULL, STACKSLIDEH_MSGENULL);
  }

  /* !!!!!!!!!!!!!!!!!!! TO START WITH, WILL RUN CODE ONLY ON THE SEARCH MASTER !!!!!!!!!!!!!!!!! */
  /* !!!!!!! LATER, WILL RUN ON THE SLAVES, AND ADD COMMUNICATION OF RESULTS BACK TO MASTER !!!!! */
  if (params->searchMaster) {

    output->numOutput = 0; /* 07/12/02 gam Initialize */

    #ifdef DEBUG_STACKSLIDE
      printf("\nI am the search master.  params->searchMaster = %i \n",params->searchMaster);
      printf("\nRank of this node = %i \n",params->rank);
      printf("\nNumber of nodes this job = %i \n", params->numNodes); /* 07/12/02 gam */
    #endif


/**********************************************/
/*                                            */
/* START SECTION: calibrate BLKs              */
/*                                            */
/**********************************************/

  #ifdef DEBUG_STACKSLIDE
  	fprintf(stdout, "\nSTART SECTION: calibrate BLKs\n");
  	fflush(stdout);
  #endif

   if (params->calibrationFlag > 0) {
      if (params->inputDataTypeFlag == 0) {
        /* Input data is SFTs */

	const LALUnit strainPerCount = {0,{0,0,0,0,0,1,-1},{0,0,0,0,0,0,0}};
	LALUnitPair upair;
	COMPLEX8FrequencySeries response;

	response.data = NULL;
	strncpy( response.name, "response", sizeof( response.name ) );

 	LALCCreateVector( status->statusPtr, &response.data, params->nBinsPerBLK );
  	CHECKSTATUSPTR( status );

	#ifdef DEBUG_CALIBRATEBLKS
 		fprintf(stdout, "Calibrating BLKs \n");
   		fflush(stdout);
	#endif

	/* Copied and modified from src/lalwrapper/test/Test.c */

	for(k=0;k<params->numBLKs;k++) {

		#ifdef DEBUG_CALIBRATEBLKS
 			fprintf(stdout, "Calibrating BLKData[%i]\n",k);
   			fflush(stdout);
		#endif

		response.epoch = params->BLKData[k]->fft->epoch;
  		response.f0 = params->BLKData[k]->fft->f0;
  		response.deltaF = params->BLKData[k]->fft->deltaF;
  		response.sampleUnits = strainPerCount;

  		LALExtractResponse( status->statusPtr, &response, input, NULL );
  		/* CHECKSTATUSPTR( status ); */

		#ifdef DEBUG_CALIBRATEBLKS
 			fprintf(stdout, "Response extracted; will now multipy times BLKData[%i]\n",k);
   			fflush(stdout);
		#endif

		LALCCVectorMultiply( status->statusPtr, params->BLKData[k]->fft->data, params->BLKData[k]->fft->data, response.data );
  		CHECKSTATUSPTR( status );

	} /* END for(k=0;k<params->numBLKs;k++) */

	upair.unitOne = &params->unitBLK;
  	upair.unitTwo = &response.sampleUnits;
  	LALUnitMultiply( status->statusPtr, &params->unitBLK, &upair );
	CHECKSTATUSPTR( status );

 	LALCDestroyVector( status->statusPtr, &response.data );
  	CHECKSTATUSPTR( status );
      } /* END if (params->inputDataTypeFlag == 0) */
   } /* END if (params->calibrationFlag > 0) */

/**********************************************/
/*                                            */
/* END SECTION: calibrate BLKs                */
/*                                            */
/**********************************************/

/**********************************************/
/*                                            */
/* START SECTION: normalize BLKs              */
/*                                            */
/**********************************************/

  #ifdef DEBUG_STACKSLIDE
  	fprintf(stdout, "\nSTART SECTION: normalize BLKs\n");
  	fflush(stdout);
  #endif

   if (params->normalizationFlag == 1) {
      if (params->inputDataTypeFlag == 0) {
        /* Input data is SFTs */

        UINT4 iNRM = 0;
        REAL4 pwSumNRM= 0.0;
	REAL4 blkNRM = 1.0; /* Default value: does not change BLKs. */

        params->iMinNRM = floor((params->f0NRM - params->f0BLK)*params->tBLK + 0.5);    /* Index of mimimum frequency to include when normalizing BLKs */
        params->iMaxNRM = params->iMinNRM + params->nBinsPerNRM - 1;  /* Index of maximum frequency to include when normalizing BLKs */

        #ifdef DEBUG_NORMALIZEBLKS
               fprintf(stdout, "Normalizing BLKData:  params->iMinNRM, params->iMaxNRM = %i, %i\n",params->iMinNRM,params->iMaxNRM);
               fflush(stdout);
        #endif

	for(k=0;k<params->numBLKs;k++) {
            pwSumNRM= 0.0;

	    for(iNRM=params->iMinNRM;iNRM<=params->iMaxNRM;iNRM++)
            {
	        pwSumNRM += params->BLKData[k]->fft->data->data[iNRM].re*params->BLKData[k]->fft->data->data[iNRM].re
		       + params->BLKData[k]->fft->data->data[iNRM].im*params->BLKData[k]->fft->data->data[iNRM].im;
            }
            blkNRM = sqrt(pwSumNRM/((REAL4)(params->nBinsPerNRM))); /* Note NO factor of 2; makes mean BLK power = 1, but NRM is not the one-sided power spectral density */

	    #ifdef DEBUG_NORMALIZEBLKS
                  fprintf(stdout, "Normalizing BLKData[%i], with MeanSpectralAmplitude = %g \n",k, blkNRM);
                  fflush(stdout);
            #endif

	    if (blkNRM > 0.0) {
		blkNRM = 1.0/blkNRM; /* Divide just once; then multiply below. */
		for(i=0;i<params->nBinsPerBLK;i++)
		{
			params->BLKData[k]->fft->data->data[i].re = blkNRM*params->BLKData[k]->fft->data->data[i].re;
			params->BLKData[k]->fft->data->data[i].im = blkNRM*params->BLKData[k]->fft->data->data[i].im;
		}
            } else {
	        /* BLK could be all zeros; just continue */
	    }
	} /* END for(k=0;k<params->numBLKs;k++) */

	params->unitBLK = lalDimensionlessUnit; /* BLKData is now dimensionless */

      } /* END if (params->inputDataTypeFlag == 0) */
   } /* END if (params->normalizationFlag == 1) */

/**********************************************/
/*                                            */
/* END SECTION: normalize BLKs                */
/*                                            */
/**********************************************/

/**********************************************/
/*                                            */
/* START SECTION: set up parameter space      */
/*                                            */
/**********************************************/

  #ifdef DEBUG_STACKSLIDE
  	fprintf(stdout, "\nSTART SECTION: set up parameter space\n");
  	fflush(stdout);
  #endif

 params->skyPosData=(REAL8 **)LALMalloc(params->numSkyPosTotal*sizeof(REAL8 *));
 for(i=0;i<params->numSkyPosTotal;i++)
 {
	params->skyPosData[i] = (REAL8 *)LALMalloc(2*sizeof(REAL8));
 }

 if (params->numSpinDown > 0) {
   params->freqDerivData=(REAL8 **)LALMalloc(params->numFreqDerivTotal*sizeof(REAL8 *));
   for(i=0;i<params->numFreqDerivTotal;i++)
   {
	params->freqDerivData[i] = (REAL8 *)LALMalloc(params->numSpinDown*sizeof(REAL8));
   }
 }

 /* 0 = use input delta for each param, 1 = params are input as vectors of data, 2 = use input param metric, 3 = create param metric */
 if (params->parameterSpaceFlag == 0) {

   INT4 iRA = 0;
   INT4 iDec = 0;
   INT4 iFDeriv1 = 0;
   INT4 iFDeriv2 = 0;
   INT4 iFDeriv3 = 0;
   INT4 iFDeriv4 = 0;
   INT4 iFDeriv5 = 0;

   INT4 repeatFDeriv1every = 0;
   INT4 repeatFDeriv2every = 0;
   INT4 repeatFDeriv3every = 0;
   INT4 repeatFDeriv4every = 0;
   INT4 repeatFDeriv5every = 0;

   for(i=0;i<params->numSpinDown;i++)
   {
        switch(i) {
	   case  0: repeatFDeriv1every = params->numFDeriv1; break;
	   case  1: repeatFDeriv2every = params->numFDeriv2*repeatFDeriv1every; break;
	   case  2: repeatFDeriv3every = params->numFDeriv3*repeatFDeriv2every; break;
	   case  3: repeatFDeriv4every = params->numFDeriv4*repeatFDeriv3every; break;
	   case  4: repeatFDeriv5every = params->numFDeriv5*repeatFDeriv4every; break;
	}
   }

   for(i=0;i<params->numSkyPosTotal;i++)
   {
	iDec = i % params->numDec;
	iRA = floor(i/params->numDec);
	params->skyPosData[i][0] = params->startRA + iRA*params->deltaRA;
	params->skyPosData[i][1] = params->startDec + iDec*params->deltaDec;
   }

   for(i=0;i<params->numFreqDerivTotal;i++)
   {
        for(k=0;k<params->numSpinDown;k++)
	{
		if (k == 0) {
		   iFDeriv1 = i % params->numFDeriv1;
		   params->freqDerivData[i][k] = params->startFDeriv1 + iFDeriv1*params->deltaFDeriv1;
		} else if (k == 1) {
		   iFDeriv2 = floor((i % repeatFDeriv2every)/repeatFDeriv1every);
		   params->freqDerivData[i][k] = params->startFDeriv2 + iFDeriv2*params->deltaFDeriv2;
		} else if (k == 2) {
		   iFDeriv3 = floor((i % repeatFDeriv3every)/repeatFDeriv2every);
		   params->freqDerivData[i][k] = params->startFDeriv3 + iFDeriv3*params->deltaFDeriv3;
		} else if (k == 3) {
		   iFDeriv4 = floor((i % repeatFDeriv4every)/repeatFDeriv3every);
		   params->freqDerivData[i][k] = params->startFDeriv4 + iFDeriv4*params->deltaFDeriv4;
		} else if (k == 4) {
		   iFDeriv5 = floor((i % repeatFDeriv5every)/repeatFDeriv4every);
		   params->freqDerivData[i][k] = params->startFDeriv5 + iFDeriv4*params->deltaFDeriv5;
		} /* END if (k == 0) ELSE ... */
        } /* END for(k=0;k<params->numSpinDown;k++) */
   } /* END for(i=0;i<params->numFreqDerivTotal;i++) */
 } /* END if (params->parameterSpaceFlag == 0) */

/**********************************************/
/*                                            */
/* END SECTION: set up parameter space        */
/*                                            */
/**********************************************/

/**********************************************/
/*                                            */
/* START SECTION: make STKs                   */
/*                                            */
/**********************************************/

  #ifdef DEBUG_STACKSLIDE
  	fprintf(stdout, "\nSTART SECTION: make STKs\n");
  	fflush(stdout);
  #endif

 /* Allocate memory for the STKData structure */
 params->STKData=(REAL4FrequencySeries **)LALMalloc(params->numSTKs*sizeof(REAL4FrequencySeries *));
 for(i=0;i<params->numSTKs;i++)
 {
	params->STKData[i]=(REAL4FrequencySeries *)LALMalloc(sizeof(REAL4FrequencySeries));
	params->STKData[i]->data=(REAL4Vector *)LALMalloc(sizeof(REAL4Vector));
	params->STKData[i]->data->data=(REAL4 *)LALMalloc(params->nBinsPerSTK*sizeof(REAL4));
 }

 if (params->stackTypeFlag == 0) {

    /************************************************/
    /*                                              */
    /* START SUBSECTION: Stacks are PSDs from SFTs  */
    /*                                              */
    /************************************************/

        UINT4 kSTK = -1;  /* index give which STK  */
        UINT4 iSTK = 0;  /* index gives which STK bin */

        #ifdef DEBUG_STACKSLIDE
  	    fprintf(stdout, "\nSTART SUBSECTION: Stacks are PSDs from SFTs\n");
  	    fflush(stdout);
        #endif

        params->whichSTK = 0;  /* which STK does BLK go with  */
        params->lastWhichSTK = -1;  /* Last value of params->whichSTK does BLK go with  */

	params->iMinBLK = floor((params->f0STK-params->f0BLK)*params->tBLK + 0.5); /* Index of mimimum frequency to include when making STKs from BLKs */
        params->iMaxBLK = params->iMinBLK + params->nBinsPerSTK - 1;               /* Index of maximum frequency to include when making STKs from BLKs */

	for(k=0;k<params->numBLKs;k++) {

	    /* Which STK does this BKL go with */
	    params->whichSTK = (params->BLKData[k]->fft->epoch.gpsSeconds - params->gpsStartTimeSec)/params->tSTK;
	    if (params->whichSTK > params->lastWhichSTK) {
	        kSTK++; /* increment which stack we are building */
	    }
	    for(i=params->iMinBLK; i <= params->iMaxBLK; i++)
            {
                iSTK = i - params->iMinBLK;
	        if (params->whichSTK > params->lastWhichSTK) {
		   /* Starting a new STK: initialize */
	           params->STKData[kSTK]->data->data[iSTK] = params->BLKData[k]->fft->data->data[i].re*params->BLKData[k]->fft->data->data[i].re
		       + params->BLKData[k]->fft->data->data[i].im*params->BLKData[k]->fft->data->data[i].im;
                   params->STKData[kSTK]->epoch.gpsSeconds = params->gpsStartTimeSec + params->whichSTK*params->tSTK;
		   params->STKData[kSTK]->epoch.gpsNanoSeconds = params->gpsStartTimeNan;
		   params->STKData[kSTK]->f0=params->f0STK;
		   params->STKData[kSTK]->deltaF=params->dfSTK;
		   params->STKData[kSTK]->data->length=params->nBinsPerSTK;
	        } else {
	           params->STKData[kSTK]->data->data[iSTK] += params->BLKData[k]->fft->data->data[i].re*params->BLKData[k]->fft->data->data[i].re
		       + params->BLKData[k]->fft->data->data[i].im*params->BLKData[k]->fft->data->data[i].im;
		}
            }
            params->lastWhichSTK = params->whichSTK;

	} /* END for(k=0;k<params->numBLKs;k++) */

        params->finishedSTKs = 1;  /* Set equal to true when all STKs for this job have been created */

    /************************************************/
    /*                                              */
    /* END SUBSECTION: Stacks are PSDs from SFTs    */
    /*                                              */
    /************************************************/


 } else if (params->stackTypeFlag == 1) {

    /**************************************************/
    /*                                                */
    /* START SUBSECTION: Stacks are F-stat from SFTs  */
    /*                                                */
    /**************************************************/

        #ifdef DEBUG_STACKSLIDE
  	    fprintf(stdout, "\nSTART SUBSECTION: Stacks are F-stat from SFTs\n");
  	    fflush(stdout);
        #endif

    #ifdef INCLUDE_DEMOD_CODE
        /* stacks are F-statistic from SFTs */

	DemodPar *demParams;          /* Parameters needed for stitching together BLKs to produce F-Stat */
	CSParams *csParams;           /* Container for ComputeSky */
	INT4 iSkyCoh;                 /* For ComputeSky */

        LALDetector cachedDetector;
        BarycenterInput baryinput;
        EarthState earth;
        EmissionTime emit;

        /* Variables for AM correction */
        AMCoeffsParams *amParams;
        AMCoeffs amc;
        LALFrDetector tempDet; /* temporary container to hold detector geometry */

        if (strstr(params->IFO, "LHO")) {
             cachedDetector = lalCachedDetectors[LALDetectorIndexLHODIFF];
        } else if (strstr(params->IFO, "LLO")) {
             cachedDetector = lalCachedDetectors[LALDetectorIndexLLODIFF];
        } else if (strstr(params->IFO, "GEO")) {
             cachedDetector = lalCachedDetectors[LALDetectorIndexGEO600DIFF];
        } else if (strstr(params->IFO, "VIRGO")) {
             cachedDetector = lalCachedDetectors[LALDetectorIndexVIRGODIFF];
        } else if (strstr(params->IFO, "TAMA")) {
             cachedDetector = lalCachedDetectors[LALDetectorIndexTAMA300DIFF];
        } else {
             /* "Invalid or null IFO" */
             ABORT( status, STACKSLIDEH_EIFO, STACKSLIDEH_MSGEIFO);
        }

        baryinput.site.location[0]=cachedDetector.location[0]/LAL_C_SI;
        baryinput.site.location[1]=cachedDetector.location[1]/LAL_C_SI;
        baryinput.site.location[2]=cachedDetector.location[2]/LAL_C_SI;
        baryinput.alpha=params->startRA;
        baryinput.delta=params->startDec;
        baryinput.dInv=0.e0;  /* Inverse distance to the source.  Not sure what 0.0 does. */

	/* BEGIN AMPLITUDE MODULATION */

	/* Allocate space for amParams stucture */
        /* Here, amParams->das is the Detector and Source info */
        amParams = (AMCoeffsParams *)LALMalloc(sizeof(AMCoeffsParams));
        amParams->das = (LALDetAndSource *)LALMalloc(sizeof(LALDetAndSource));
        amParams->das->pSource = (LALSource *)LALMalloc(sizeof(LALSource));
        amParams->baryinput = &baryinput;
        amParams->earth = &earth;
        amParams->edat = params->edat;
        amParams->das->pDetector = &cachedDetector;
        amParams->das->pSource->equatorialCoords.latitude = params->startDec;
        amParams->das->pSource->equatorialCoords.longitude = params->startRA;
        amParams->das->pSource->equatorialCoords.system = COORDINATESYSTEM_EQUATORIAL;
        amParams->das->pSource->orientation = 0.0;  /* The polarization angle. See LALSource documentation. See note below. */
        amParams->polAngle = 0.0; /* Also the polarization angle. */ /* Any angle should work since Jaranowski, Krolak, and Schutz a(t) and b(t) are independent of psi and only need a(t) and b(t) */
        amParams->leapAcc = LALLEAPSEC_STRICT; /* accuracy in def of leap sec */
        amParams->det = (LALFrDetector *)LALMalloc(sizeof(LALFrDetector)); /* detector geometry */
        tempDet = amParams->das->pDetector->frDetector; /* temporary container to hold detector geometry */
	amParams->det = &tempDet; /* Pointer to temporary container with detector geometry */

        /* Allocate space for AMCoeffs */
        amc.a = NULL;
        amc.b = NULL;
        LALSCreateVector(status->statusPtr, &(amc.a), (UINT4)params->numBLKs);
        CHECKSTATUSPTR (status);
        LALSCreateVector(status->statusPtr, &(amc.b), (UINT4)params->numBLKs);
        CHECKSTATUSPTR (status);

       {

          LIGOTimeGPS *midTS;

	  midTS = LALMalloc( params->numBLKs * sizeof( *midTS ) );
          /* TODO: check memory allocation!!! */

          for(k=0; k<params->numBLKs; k++)
          {
	    REAL8 teemp=0.0;

	    TimeToFloat(&teemp, &(params->timeStamps[k]));
	    teemp += (REAL8)0.5*params->tBLK;  /* Go to the time midway through this BLK */
	    FloatToTime(&(midTS[k]), &teemp);
          }

	  /* Compute the AM coefficients */
          LALComputeAM(status->statusPtr, &amc, midTS, amParams);
    	  CHECKSTATUSPTR (status);

          LALFree( midTS );
       }

       /* END AMPLITUDE MODULATION */

        /* Set DemodPar *demParams for LALDemod */
	demParams=(DemodPar *)LALMalloc(sizeof(DemodPar));
	demParams->spinDwnOrder=params->numSpinDown;
	demParams->skyConst=(REAL8 *)LALMalloc((2*params->numSpinDown*(params->numBLKs+1)+2*params->numBLKs+3)*sizeof(REAL8));
	demParams->spinDwn=(REAL8 *)LALMalloc(params->numSpinDown*sizeof(REAL8));
        demParams->amcoe = &amc;
        demParams->f0 = params->f0STK;
	demParams->df = params->dfSTK;
        demParams->SFTno = params->numBLKs; /* Number of BLKs actually input */
        demParams->Dterms = params->Dterms; /* Terms used in the computation of the dirichlet kernel */
        demParams->ifmin = floor(params->f0BLK*params->tBLK + 0.5); /* Index of start frequency would have in BLKs that covered full band from 0 Hz to Nyquist freq. */
        demParams->imax = params->nBinsPerSTK; /* Maximum # of values of F stat to calculate */

       	for(i=0;i<params->numSpinDown;i++)
	{
	        switch(i) {
		   case  0: demParams->spinDwn[i] = params->startFDeriv1; break;
		   case  1: demParams->spinDwn[i] = params->startFDeriv2; break;
		   case  2: demParams->spinDwn[i] = params->startFDeriv3; break;
		   case  3: demParams->spinDwn[i] = params->startFDeriv4; break;
		   case  4: demParams->spinDwn[i] = params->startFDeriv5; break;
		}
	}

        #ifdef DEBUG_DEMOD_CODE
         for(i=0;i<demParams->spinDwnOrder;i++)
         {
      	   fprintf(stdout,"demParams->spinDwn[%i] = %g \n",i,demParams->spinDwn[i]);
      	   fflush(stdout);
         }
        #endif

	/* Allocate space and set quantities for call to ComputeSky() */
	csParams=(CSParams *)LALMalloc(sizeof(CSParams));
	csParams->skyPos=(REAL8 *)LALMalloc(2*sizeof(REAL8));
	csParams->skyPos[0]=params->startRA;
	csParams->skyPos[1]=params->startDec;
	csParams->tGPS=params->timeStamps;
	csParams->spinDwnOrder=params->numSpinDown;
	csParams->mObsSFT=params->numBLKs;
	csParams->tBLK=params->tBLK;
        csParams->edat=params->edat;
        csParams->emit=&emit;
        csParams->earth=&earth;
        csParams->baryinput=&baryinput;

	iSkyCoh = params->patchNumber;  /* Index for sky patch */

        for(i=0;i<(2*params->numSpinDown*(params->numBLKs+1)+2*params->numBLKs+3);i++)
        {
      	  demParams->skyConst[i] = 0.0; /* Initialize */
        }

	/* Call COMPUTESKY() */
	ComputeSky(status->statusPtr, demParams->skyConst, iSkyCoh, csParams);
    	CHECKSTATUSPTR (status);

	/* Deallocate space for ComputeSky parameters */
	LALFree(csParams->skyPos);
	LALFree(csParams);

        #ifdef DEBUG_DEMOD_CODE
          for(i=0;i<(2*params->numSpinDown*(params->numBLKs+1)+2*params->numBLKs+3);i++)
          {
      	    fprintf(stdout,"demParams->skyConst[%i] = %g \n",i,demParams->skyConst[i]);
      	    fflush(stdout);
          }
        #endif

        /**** END COMPUTE demParams ****/

        k = 0; /* Need to set index of which STK */
        #ifdef DEBUG_DEMOD_CODE
           for(i=0;i<params->nBinsPerSTK;i++)
           {
	     params->STKData[k][i] = 0.0;
           }
        #endif

        #ifdef DEBUG_DEMOD_CODE
          for(i=0;i<params->nBinsPerSTK;i++)
          {
	     fprintf(stdout,"initialized params->STKData[%i] f = %g params->STKData data = %g \n",i,params->f0STK+(REAL8)i/params->duration,
		params->STKData[k][i]); /* 06/24/03 gam */
	     fflush(stdout);
          }
         #endif

        /**** CALL LALDemod !!!! ****/
        LALDemod(status->statusPtr, params->STKData[k], params->BLKData, demParams);
        CHECKSTATUSPTR (status);

        #ifdef DEBUG_DEMOD_CODE
          fprintf(stdout,"\nparams->nBinsPerSTK = %i \n",params->nBinsPerSTK);
          fflush(stdout);
          for(i=0;i<params->nBinsPerSTK;i++)
          {
	     fprintf(stdout,"i = %i f = %g fft->data->data[i] = %g \n",i,params->f0STK+(REAL8)i/params->duration,
		params->STKData[k][i]);
	     fflush(stdout);
          }
        #endif

	/* Deallocate demodulation params */
	LALFree(demParams->skyConst);
	LALFree(demParams->spinDwn);
	LALFree(demParams);

        LALSDestroyVector(status->statusPtr, &(amc.a));
    	CHECKSTATUSPTR (status);
        LALSDestroyVector(status->statusPtr, &(amc.b));
    	CHECKSTATUSPTR (status);

        LALFree(amParams->das->pSource);
        LALFree(amParams->das);
        LALFree(amParams->det);
        LALFree(amParams);

	params->finishedSTKs = 1;  /* Set equal to true when all STKs for this job have been created */

        /* LALCheckMemoryLeaks(); */ /* NOT SURE IF I CAN CALL THIS HERE */
     #endif
    /**************************************************/
    /*                                                */
    /* END SUBSECTION: Stacks are F-stat from SFTs    */
    /*                                                */
    /**************************************************/

 } /* END if (params->stackTypeFlag ...) */

/**********************************************/
/*                                            */
/* END SECTION: make STKs                     */
/*                                            */
/**********************************************/

/******************************/
/*                            */
/* START SECTION: make SUMs   */
/*                            */
/******************************/

  #ifdef DEBUG_STACKSLIDE
      fprintf(stdout, "\nSTART SECTION: make SUMs\n");
      fflush(stdout);
  #endif

 /* Allocate memory for the SUMData structure */
 params->SUMData=(REAL4FrequencySeries **)LALMalloc(params->numSUMsTotal*sizeof(REAL4FrequencySeries *));
 for(i=0;i<params->numSUMsTotal;i++)
 {
	params->SUMData[i]=(REAL4FrequencySeries *)LALMalloc(sizeof(REAL4FrequencySeries));
	params->SUMData[i]->data=(REAL4Vector *)LALMalloc(sizeof(REAL4Vector));
	params->SUMData[i]->data->data=(REAL4 *)LALMalloc(params->nBinsPerSUM*sizeof(REAL4));
 }

 /* Allocate memory for the SUMStats structure to hold statistics about eadh sum */
 params->SUMStats=(FreqSeriesPowerStats *)LALMalloc(params->numSUMsTotal*sizeof(FreqSeriesPowerStats));

 {
    /* Declare and variables needed for making SUMs */
    UINT4 iSky = 0;        /* index gives which Sky Position */
    UINT4 iFreqDeriv = 0;  /* index give which Freq Derivs  */
    UINT4 kSUM = 0;        /* index gives which SUM */
    UINT4 iSUM = 0;        /* index gives which SUM bin */

    UINT4 iMinSTK = floor((params->f0SUM-params->f0STK)*params->tSTK + 0.5); /* Index of mimimum frequency to include when making SUMs from STKs */
    UINT4 iMaxSTK = iMinSTK + params->nBinsPerSUM - 1;                       /* Index of maximum frequency to include when making SUMs from STKs */

    /* Code currently assumes  params->numSUMsPerParamSpacePt = params->duration/params->tSUM = 1; need to upgrade to more general case */
    for(kSUM=0;kSUM<params->numSUMsTotal;kSUM++)
    {
      /* Make a SUM for each point in the parameter space */

      iSky = kSUM % params->numSkyPosTotal;  /* RA for this parameter space pt is params->skyPosData[iSky][0], DEC for this parameter space pt is params->skyPosData[iSky][1] */

      if (params->numSpinDown > 0) {
	  iFreqDeriv = floor(kSUM/params->numSkyPosTotal); /* jth Deriv is params->freqDerivData[iFreqDeriv][j] */
      }

      for(k=0;k<params->numSTKs;k++) {
	    for(i=iMinSTK;i<=iMaxSTK; i++) {
                iSUM = i - iMinSTK;
	        if (k==0) {
		   /* Starting a new SUM: initialize */
	           params->SUMData[kSUM]->data->data[iSUM] = params->STKData[k]->data->data[i];
                   params->SUMData[kSUM]->epoch.gpsSeconds = params->gpsStartTimeSec;
		   params->SUMData[kSUM]->epoch.gpsNanoSeconds = params->gpsStartTimeNan;
		   params->SUMData[kSUM]->f0=params->f0SUM;
		   params->SUMData[kSUM]->deltaF=params->dfSUM;
		   params->SUMData[kSUM]->data->length=params->nBinsPerSUM;
	        } else {
	           params->SUMData[kSUM]->data->data[iSUM] += params->STKData[k]->data->data[i];
		}
            }
      } /* END for(k=0;k<params->numSTKs;k++) */
      /* Normalize the SUMs with params->numSTKs*/
      for(i=0;i<params->nBinsPerSUM; i++) {
               params->SUMData[kSUM]->data->data[i] =  params->SUMData[kSUM]->data->data[i]/((REAL4)params->numSTKs);
      }
      /* Compute Stats for this SUM */
      ComputePowerStats( params->SUMData[kSUM]->data, &(params->SUMStats[kSUM]), params->SUMData[kSUM]->f0, params->SUMData[kSUM]->deltaF);
    } /* for(kSUM=0;kSUM<params->numSUMsTotal;kSUM++) */
 }
 params->finishedSUMs = 1;  /* Set equal to true when all BLKS for this job have been created */

/******************************/
/*                            */
/* END SECTION: make SUMs     */
/*                            */
/******************************/

/**********************************************/
/*                                            */
/* START SECTION: output results              */
/*                                            */
/**********************************************/

  #ifdef DEBUG_STACKSLIDE
      fprintf(stdout, "\nSTART SECTION: output results\n");
      fflush(stdout);
  #endif

   if (params->finishedSUMs) {

	/* Next are used when allocating memory in BuildOutput and are also needed in BuildSUMFrameOutput */
	params->numDBOutput = 3;  /* FOR NOW HARD CODE for 1 row in 3 tables: search summary + search summary vars + dperiod tables */
        if (params->outputFlag > 0) {
  	    output->numOutput = 3 + 1; /* FOR NOW HARD CODE */
	} else {
            output->numOutput = 3; /* FOR NOW HARD CODE */
	}

	BuildOutput(status->statusPtr, params, output);
        CHECKSTATUSPTR (status);

        #ifdef INCLUDE_BUILDSUMFRAME_CODE
         if (params->outputFlag > 0) {
             BuildSUMFrameOutput(status->statusPtr, params, output);
	     CHECKSTATUSPTR (status);
	 }
        #endif

        /**********************************************/
        /*                                            */
        /* START SUBSECTION: Deallocate most memory   */
        /*                                            */
        /**********************************************/

         #ifdef DEBUG_STACKSLIDE
             fprintf(stdout, "\nSTART SUBSECTION: Deallocate most memory\n");
             fflush(stdout);
         #endif

         /* Deallocate memory for the SUMStats structure */
	 LALFree(params->SUMStats);

         /* Deallocate memory for the SUMData structure */
         for(i=0;i<params->numSUMsTotal;i++)
         {
	     LALFree(params->SUMData[i]->data->data);
	     LALFree(params->SUMData[i]->data);
	     LALFree(params->SUMData[i]);
         }
         LALFree(params->SUMData);

         /* Deallocate memory for the STKData structure */
         for(i=0;i<params->numSTKs;i++)
         {
	     LALFree(params->STKData[i]->data->data);
	     LALFree(params->STKData[i]->data);
	     LALFree(params->STKData[i]);
         }
         LALFree(params->STKData);

         /* Deallocate memory for the freqDerivData structure */
         if (params->numSpinDown > 0) {
            for(i=0;i<params->numFreqDerivTotal;i++)
            {
	        LALFree(params->freqDerivData[i]);
            }
            LALFree(params->freqDerivData);
         }

	 /* Deallocate memory for the skyPosData structure */
         for(i=0;i<params->numSkyPosTotal;i++)
         {
	     LALFree(params->skyPosData[i]);
         }
         LALFree(params->skyPosData);

	 LALFree(params->timeStamps);

	 LALFree(params->timeIntervals);

	 /* Deallocate BLKData structure */
	 for(i=0;i<params->numBLKs;i++)
	 {
	        LALFree(params->BLKData[i]->fft->data->data);
		LALFree(params->BLKData[i]->fft->data);
		LALFree(params->BLKData[i]->fft);
		LALFree(params->BLKData[i]);
	 }
	 LALFree(params->BLKData);

	 /* Deallocate ephemeris data */
         LALFree(params->edat->ephemS);
         LALFree(params->edat->ephemE);
         LALFree(params->edat);

        /**********************************************/
        /*                                            */
        /* END SUBSECTION: Deallocate memory          */
        /*                                            */
        /**********************************************/

	 output->fracRemaining = 0;
    	 output->notFinished	= 0;

	  #ifdef DEBUG_FRACREMAINING_CODE
  	         fprintf(stdout, "\nfracRemaining = %g \n", output->fracRemaining);
  		 fflush(stdout);
	  #endif

	} else {

	      output->fracRemaining = (REAL4)params->lastK/(REAL4)params->numBLKs;
    	      output->notFinished = 1;

	      #ifdef DEBUG_FRACREMAINING_CODE
  		fprintf(stdout, "\nfracRemaining = %g \n", output->fracRemaining);
  		fflush(stdout);
	      #endif

	} /* END if (params->finishedBLKs) else ... */


  } else {
  /* if NOT (params->searchMaster) */
  /* !!!!!!!!!!!!!!!!!!! TO START WITH, WILL RUN CODE ONLY ON THE SEARCH MASTER !!!!!!!!!!!!!!!!! */
  /* !!!!!!! LATER, WILL RUN ON THE SLAVES, AND ADD COMMUNICATION OF RESULTS BACK TO MASTER !!!!! */

    /* Just continue for now */
    /* Set output data structure */
    output->numOutput 	= 0;
    output->result	= NULL;
    output->fracRemaining = 0;
    output->notFinished	= 0;

  } /* END if (params->searchMaster) else ... */

/**********************************************/
/*                                            */
/* END SECTION: output results                */
/*                                            */
/**********************************************/


  CHECKSTATUSPTR (status);
  DETATCHSTATUSPTR (status);
  RETURN( status );
}
/******************************************/
/*                                        */
/* END FUNCTION: LALApplySearch           */
/*                                        */
/******************************************/

/******************************************/
/*                                        */
/* START FUNCTION: LALFinalizeSearch      */
/*                                        */
/******************************************/
void
LALFinalizeSearch(
    LALStatus             *status,
    void                 **searchParams
    )
{
  StackSlideSearchParams* const params = (StackSlideSearchParams *) *searchParams;

  INITSTATUS( status, "LALFinalizeSearch", STACKSLIDEC );
  ATTATCHSTATUSPTR (status);

  #ifdef DEBUG_STACKSLIDE
      fprintf(stdout, "\nSTART FUNCTION: LALFinalizeSearch\n");
      fflush(stdout);
  #endif

  /* !!!!!!!!!!!!!!!!!!! TO START WITH, WILL RUN CODE ONLY ON THE SEARCH MASTER !!!!!!!!!!!!!!!!! */
  /* !!!!!!! LATER, WILL RUN ON THE SLAVES, AND ADD COMMUNICATION OF RESULTS BACK TO MASTER !!!!! */
  if (params->searchMaster) {

  }

  #ifdef DEBUG_STACKSLIDE
    fprintf(stdout, "\nLALFinalizeSearch, my rank is %i.\n",params->rank);
    fflush(stdout);
  #endif

  /**********************************************/
  /*                                            */
  /* START SECTION: Deallocate remaining memory */
  /*                                            */
  /**********************************************/

   #ifdef DEBUG_STACKSLIDE
        fprintf(stdout, "\nSTART SECTION: Deallocate remaining memory\n");
        fflush(stdout);
   #endif

  LALFree(params->ifoNickName);
  LALFree(params->IFO);
  LALFree(params->patchName);

  LALFree(params->dsoName);

  LALFree(*searchParams);

  /**********************************************/
  /*                                            */
  /* END SECTION: Deallocate remaining memory   */
  /*                                            */
  /**********************************************/

  DETATCHSTATUSPTR (status);
  RETURN(status);

}
/******************************************/
/*                                        */
/* END FUNCTION: LALFinalizeSearch        */
/*                                        */
/******************************************/

/******************************************/
/*                                        */
/* START SECTION: internal functions      */
/*                                        */
/******************************************/

/* Copied from LALDemodTest.c */
static void TimeToFloat(REAL8 *f, LIGOTimeGPS *tgps)
{
  INT4 x, y;

  x=tgps->gpsSeconds;
  y=tgps->gpsNanoSeconds;
  *f=(REAL8)x+(REAL8)y*1.e-9;
}

/* Copied from LALDemodTest.c */
static void FloatToTime(LIGOTimeGPS *tgps, REAL8 *f)
{
  REAL8 temp0, temp2, temp3;
  REAL8 temp1, temp4;

  temp0 = floor(*f);     /* this is tgps.S */
  temp1 = (*f) * 1.e10;
  temp2 = fmod(temp1, 1.e10);
  temp3 = fmod(temp1, 1.e2);
  temp4 = (temp2-temp3) * 0.1;

  tgps->gpsSeconds = (INT4)temp0;
  tgps->gpsNanoSeconds = (INT4)temp4;
}


/* For building output for the database */
/* Copied and modified from Moments.c   */
/* Author: Charlton, Philip; Brady, P R */
/*
 * Function BuildOutput
 *
 * Create a linked list containing the database entries and put it in
 * the search output structure
 */
static void
BuildOutput(LALStatus*            const status,
	    const StackSlideSearchParams* const params,
	    LALSearchOutput*      const output)
{

  UINT4  i = 0;        /* all purpose index */

  dataBase* dbEntry = 0;
  outPut* outputArray = 0;

  UINT4 stringLength; /* Copied from StochasticBuildOutput.c */
  dataBase  *prev;    /* Copied from StochasticBuildOutput.c */

  /* Define these temporary constants used below when setting in_end_time and in_end_time_ns */
  REAL8 tempReal8tBLK1 = 0.0;
  LIGOTimeGPS tempLIGOTimeGPStBLK;

  /* macro to attach status pointer */
  INITSTATUS( status, "BuildOutput", STACKSLIDEC );
  ATTATCHSTATUSPTR( status );

  #ifdef DEBUG_BUILDOUTPUT_CODE
    	fprintf(stdout,"\nIn BuildOutput: output->params->numDBOutput = %i \n",params->numDBOutput);
    	fprintf(stdout,"\nIn BuildOutput: output->numOutput = %i \n",output->numOutput);
	fflush(stdout);
  #endif

  outputArray = output->result = (outPut *) LALCalloc( output->numOutput, sizeof(outPut) );
  if ( ! outputArray )
  {
    ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
  }

     /************************************************/
     /*                                              */
     /* START SECTION: Write to search summary table */
     /*                                              */
     /************************************************/

     /* Write to the search summary table.  Copied and modified from StochasticBuildOutput.c */

     /* this is the same for every output */

      outputArray[0].search         = periodic;
      outputArray[0].significant    = 1;
      outputArray[0].states         = 0;
      outputArray[0].optional       = 0;
      outputArray[0].indexNumber    = 0;

      /* Initiate linked list */
      dbEntry = outputArray[0].results = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->previous = NULL;

     /* Shared object name */

     strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
     strncpy( dbEntry->columnName, "shared_object", dbNameLimit );

     dbEntry->type       = char_s_ptr;
     dbEntry->numberRows = 1;

     /* This prevents extra \0s from being written to the strings */

     stringLength = strlen("stackslide");

     dbEntry->rows.chars = (CHAR *) LALMalloc( stringLength * dbEntry->numberRows * sizeof(CHAR) );
     if ( ! dbEntry->rows.chars )
     {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }
     strncpy( dbEntry->rows.chars,"stackslide", stringLength*sizeof(CHAR) );

     dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
     if ( ! dbEntry->rowDimensions )
     {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }
     *(dbEntry->rowDimensions) = stringLength;

     prev = dbEntry;

     /* LAL CVS tag */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }
     dbEntry->previous = prev;

     strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
     strncpy( dbEntry->columnName, "lal_cvs_tag", dbNameLimit );

     dbEntry->type       = char_s_ptr;
     dbEntry->numberRows = 1;

     /* This prevents extra \0s from being written to the strings */

     stringLength = strlen(LAL_CVS_TAG);

     dbEntry->rows.chars =
       (CHAR *) LALMalloc( stringLength * dbEntry->numberRows * sizeof(CHAR) );
     if ( ! dbEntry->rows.chars )
     {
       ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }
     strncpy( dbEntry->rows.chars, LAL_CVS_TAG, stringLength*sizeof(CHAR) );

     dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
     if ( ! dbEntry->rowDimensions )
     {
       ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }
     *(dbEntry->rowDimensions) = stringLength;

     prev = dbEntry;

     /* LALWrapper CVS tag */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }
     dbEntry->previous = prev;

     strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
     strncpy( dbEntry->columnName, "lalwrapper_cvs_tag", dbNameLimit );

     dbEntry->type       = char_s_ptr;
     dbEntry->numberRows = 1;

     /* This prevents extra \0s from being written to the strings */

     stringLength = strlen(LAL_WRAPPER_CVS_TAG);

     dbEntry->rows.chars =
       (CHAR *) LALMalloc( stringLength * dbEntry->numberRows * sizeof(CHAR) );
     if ( ! dbEntry->rows.chars )
     {
       ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }
     strncpy( dbEntry->rows.chars, LAL_WRAPPER_CVS_TAG, stringLength*sizeof(CHAR) );

     dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
     if ( ! dbEntry->rowDimensions )
     {
       ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }
     *(dbEntry->rowDimensions) = stringLength;

     prev = dbEntry;

     /* start time seconds */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }
     dbEntry->previous = prev;

     strncpy( dbEntry->tableName, "search_summary", dbNameLimit);
     strncpy( dbEntry->columnName, "in_start_time", dbNameLimit );

     dbEntry->type       = int_4s;
     dbEntry->numberRows = 1;
     dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
     if ( ! dbEntry->rows.int4s )
     {
       ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }
     dbEntry->rowDimensions = NULL;
     *(dbEntry->rows.int4s) = params->gpsStartTimeSec; /* Use input start time; actual start time for out_start_time below */

     prev = dbEntry;

     /* start time nanoseconds */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }
     dbEntry->previous = prev;

     strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
     strncpy( dbEntry->columnName, "in_start_time_ns", dbNameLimit );

     dbEntry->type       = int_4s;
     dbEntry->numberRows = 1;
     dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
     if ( ! dbEntry->rows.int4s )
     {
       ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }
     dbEntry->rowDimensions = NULL;
     *(dbEntry->rows.int4s) = params->gpsStartTimeNan; /* Use input start time; actual start time for out_start_time below */

     prev = dbEntry;

     /* end time seconds */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }
     dbEntry->previous = prev;

     strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
     strncpy( dbEntry->columnName, "in_end_time", dbNameLimit );

     dbEntry->type       = int_4s;
     dbEntry->numberRows = 1;
     dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
     if ( ! dbEntry->rows.int4s )
     {
       ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }
     dbEntry->rowDimensions = NULL;
     /* Compute requested end time which may differ from actual end time due to missing BLKs (gaps). */
     tempReal8tBLK1 = ((REAL8)params->gpsStartTimeSec) + 1.0e-9*((REAL8)params->gpsStartTimeNan) + params->duration;  /* Code currently assumes  params->numSUMsPerParamSpacePt = params->duration/params->tSUM = 1; need to upgrade to more general case */
     FloatToTime(&tempLIGOTimeGPStBLK, &tempReal8tBLK1);
     *(dbEntry->rows.int4s) = tempLIGOTimeGPStBLK.gpsSeconds;

     prev = dbEntry;

     /* end time nanoseconds */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }
     dbEntry->previous = prev;

     strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
     strncpy( dbEntry->columnName, "in_end_time_ns", dbNameLimit );

     dbEntry->type       = int_4s;
     dbEntry->numberRows = 1;
     dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
     if ( ! dbEntry->rows.int4s )
     {
       ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }
     dbEntry->rowDimensions = NULL;
     *(dbEntry->rows.int4s) = tempLIGOTimeGPStBLK.gpsNanoSeconds;

     prev = dbEntry;

     /* start time seconds */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }
     dbEntry->previous = prev;

     strncpy( dbEntry->tableName, "search_summary", dbNameLimit);
     strncpy( dbEntry->columnName, "out_start_time", dbNameLimit );

     dbEntry->type       = int_4s;
     dbEntry->numberRows = 1;
     dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
     if ( ! dbEntry->rows.int4s )
     {
       ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }
     dbEntry->rowDimensions = NULL;
     *(dbEntry->rows.int4s) = params->times.startSec;

     prev = dbEntry;

     /* start time nanoseconds */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }
     dbEntry->previous = prev;

     strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
     strncpy( dbEntry->columnName, "out_start_time_ns", dbNameLimit );

     dbEntry->type       = int_4s;
     dbEntry->numberRows = 1;
     dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
     if ( ! dbEntry->rows.int4s )
     {
       ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }
     dbEntry->rowDimensions = NULL;
     *(dbEntry->rows.int4s) = params->times.startNan;

     prev = dbEntry;

     /* end time seconds */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }
     dbEntry->previous = prev;

     strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
     strncpy( dbEntry->columnName, "out_end_time", dbNameLimit );

     dbEntry->type       = int_4s;
     dbEntry->numberRows = 1;
     dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
     if ( ! dbEntry->rows.int4s )
     {
       ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }
     dbEntry->rowDimensions = NULL;
     *(dbEntry->rows.int4s) = params->times.stopSec;

     prev = dbEntry;

     /* end time nanoseconds */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }
     dbEntry->previous = prev;

     strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
     strncpy( dbEntry->columnName, "out_end_time_ns", dbNameLimit );

     dbEntry->type       = int_4s;
     dbEntry->numberRows = 1;
     dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
     if ( ! dbEntry->rows.int4s )
     {
       ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }
     dbEntry->rowDimensions = NULL;
     *(dbEntry->rows.int4s) = params->times.stopNan;

     dbEntry->next = NULL;

  #ifdef DEBUG_BUILDOUTPUT_CODE
      fprintf(stdout,"\nFINISHED OUTPUT FOR SEARCH SUMMARY TABLE\n");
      fflush(stdout);
  #endif

     /************************************************/
     /*                                              */
     /* END SECTION: Write to search summary table   */
     /*                                              */
     /************************************************/

     /**********************************************************/
     /*                                                        */
     /* START SECTION: Write to search summary variables table */
     /*                                                        */
     /**********************************************************/

      /* this is the same for every output */

      outputArray[1].search         = periodic;
      outputArray[1].significant    = 1;
      outputArray[1].states         = 0;
      outputArray[1].optional       = 0;
      outputArray[1].indexNumber    = 0;

      /* Initiate linked list */
      dbEntry = outputArray[1].results = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->previous = NULL;

     /* Write the search summary to the database */

     /* name */

     strncpy( dbEntry->tableName, "search_summvars", dbNameLimit );
     strncpy( dbEntry->columnName, "name", dbNameLimit );

     dbEntry->type       = char_s_ptr;
     dbEntry->numberRows = 1;

     /* This prevents extra \0s from being written to the strings */

     stringLength = strlen("stackslide");

     dbEntry->rows.chars = (CHAR *) LALMalloc( stringLength * dbEntry->numberRows * sizeof(CHAR) );
     if ( ! dbEntry->rows.chars )
     {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }
     strncpy( dbEntry->rows.chars,"stackslide", stringLength*sizeof(CHAR) );

     dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
     if ( ! dbEntry->rowDimensions )
     {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }
     *(dbEntry->rowDimensions) = stringLength;

     prev = dbEntry;

     /* string */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }
     dbEntry->previous = prev;

     strncpy( dbEntry->tableName, "search_summvars", dbNameLimit );
     strncpy( dbEntry->columnName, "string", dbNameLimit );

     dbEntry->type       = char_s_ptr;
     dbEntry->numberRows = 1;

     /* This prevents extra \0s from being written to the strings */

     stringLength = strlen("snr");

     dbEntry->rows.chars = (CHAR *) LALMalloc( stringLength * dbEntry->numberRows * sizeof(CHAR) );
     if ( ! dbEntry->rows.chars )
     {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }
     strncpy( dbEntry->rows.chars,"snr", stringLength*sizeof(CHAR) );

     dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
     if ( ! dbEntry->rowDimensions )
     {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }
     *(dbEntry->rowDimensions) = stringLength;

     prev = dbEntry;

     /* value */
     dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
     if ( ! dbEntry )
     {
       ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }
     dbEntry->previous = prev;

     strncpy( dbEntry->tableName, "search_summvars", dbNameLimit );
     strncpy( dbEntry->columnName, "value", dbNameLimit );

     dbEntry->type       = real_8;
     dbEntry->numberRows = 1;
     dbEntry->rows.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
     if ( ! dbEntry->rows.real8 )
     {
       ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }
     dbEntry->rowDimensions = NULL;
     if (params->SUMStats[0].pwStdDev > 0.0 && (params->SUMStats[0].pwMax - params->SUMStats[0].pwMean) >= 0.0) {
      	 *(dbEntry->rows.real8) = (REAL8)(sqrt((params->SUMStats[0].pwMax - params->SUMStats[0].pwMean)/params->SUMStats[0].pwStdDev));
     } else {
      	 *(dbEntry->rows.real8) = -1.0;   /* This case should not happen except in test data; hard code for now */
     }

     dbEntry->next = NULL;

     /**********************************************************/
     /*                                                        */
     /* START SECTION: Write to search summary variables table */
     /*                                                        */
     /**********************************************************/

  /*********************************************/
  /*                                           */
  /* START SECTION: Write to DPERIODIC table   */
  /*                                           */
  /*********************************************/

  for (i = 2; i < params->numDBOutput; i++) {

      outputArray[i].search         = periodic;
      outputArray[i].significant    = 1;   /* 1 - IS significant */
      outputArray[i].states         = 0;
      outputArray[i].optional       = 0;

      /* Just set equal to zero for now */
      outputArray[i].indexNumber = 0;

      /* Put entry into the SNGL_DPERIODIC database table */
      /* DPERIODIC = DIRECTED PERIODIC */

      /* Initiate linked list */
      dbEntry = outputArray[i].results = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->previous = NULL;

      /* ifo */
      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "ifo" );

      dbEntry->type       = char_s_ptr;
      dbEntry->numberRows = 1;
      stringLength = strlen(params->ifoNickName);
      dbEntry->rows.chars = (CHAR *) LALMalloc( stringLength * sizeof(CHAR) );
      if ( ! dbEntry->rows.chars )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      strncpy(dbEntry->rows.chars, params->ifoNickName, stringLength*sizeof(CHAR)); /* CAN ONLY BE TWO CHARACTERS */
      dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
      if ( ! dbEntry->rowDimensions )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      *(dbEntry->rowDimensions) = stringLength;

      /* search */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "search" );

      dbEntry->type       = char_s_ptr;
      dbEntry->numberRows = 1;
      stringLength = strlen("stackslide");
      dbEntry->rows.chars = (CHAR *) LALMalloc( stringLength * sizeof(CHAR) );
      if ( ! dbEntry->rows.chars )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      strncpy(dbEntry->rows.chars, "stackslide",stringLength*sizeof(CHAR));
      dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
      if ( ! dbEntry->rowDimensions )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      *(dbEntry->rowDimensions) = stringLength;

      /* start_time */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "start_time" );

      dbEntry->type       = int_4s;
      dbEntry->numberRows = 1;
      dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
      if ( ! dbEntry->rows.int4s )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->rowDimensions = NULL;
      *(dbEntry->rows.int4s) = params->times.startSec;

     /* start_time_ns */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "start_time_ns" );

      dbEntry->type       = int_4s;
      dbEntry->numberRows = 1;
      dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
      if ( ! dbEntry->rows.int4s )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->rowDimensions = NULL;
      *(dbEntry->rows.int4s) = params->times.startNan;

     /* end_time */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "end_time" );

      dbEntry->type       = int_4s;
      dbEntry->numberRows = 1;
      dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
      if ( ! dbEntry->rows.int4s )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->rowDimensions = NULL;
      *(dbEntry->rows.int4s) = params->times.stopSec;

     /* end_time_ns */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "end_time_ns" );

      dbEntry->type       = int_4s;
      dbEntry->numberRows = 1;
      dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
      if ( ! dbEntry->rows.int4s )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->rowDimensions = NULL;
      *(dbEntry->rows.int4s) = params->times.stopNan;

      /* duration */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "duration" );

      dbEntry->type       = real_4;
      dbEntry->numberRows = 1;
      dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
      if ( ! dbEntry->rows.real4 )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->rowDimensions = NULL;
      *(dbEntry->rows.real4) = (REAL4)(params->duration);

      /* target_name */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "target_name" );

      dbEntry->type       = char_s_ptr;
      dbEntry->numberRows = 1;
      stringLength = strlen(params->patchName );
      dbEntry->rows.chars = (CHAR *) LALMalloc( stringLength * sizeof(CHAR) );
      if ( ! dbEntry->rows.chars )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      strncpy(dbEntry->rows.chars, params->patchName, stringLength*sizeof(CHAR));
      dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
      if ( ! dbEntry->rowDimensions )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      *(dbEntry->rowDimensions) = stringLength;

      /* sky_ra */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "sky_ra" );

      dbEntry->type       = real_8;
      dbEntry->numberRows = 1;
      dbEntry->rows.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
      if ( ! dbEntry->rows.real8 )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->rowDimensions = NULL;
      *(dbEntry->rows.real8) = params->startRA;

      /* sky_dec */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "sky_dec" );

      dbEntry->type       = real_8;
      dbEntry->numberRows = 1;
      dbEntry->rows.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
      if ( ! dbEntry->rows.real8 )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->rowDimensions = NULL;
      *(dbEntry->rows.real8) = params->startDec;

      /* frequency */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "frequency" );

      dbEntry->type       = real_8;
      dbEntry->numberRows = 1;
      dbEntry->rows.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
      if ( ! dbEntry->rows.real8 )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->rowDimensions = NULL;
      *(dbEntry->rows.real8) = (REAL8) params->SUMStats[0].freqPWMax;

      /* amplitude */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "amplitude" );

      dbEntry->type       = real_4;
      dbEntry->numberRows = 1;
      dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
      if ( ! dbEntry->rows.real4 )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->rowDimensions = NULL;
      *(dbEntry->rows.real4) = (REAL4) sqrt(params->SUMStats[0].pwMax);

      /* phase */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "phase" );

      dbEntry->type       = real_4;
      dbEntry->numberRows = 1;
      dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
      if ( ! dbEntry->rows.real4 )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->rowDimensions = NULL;
      *(dbEntry->rows.real4) = 0.0;

      /* snr */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "snr" );

      dbEntry->type       = real_4;
      dbEntry->numberRows = 1;
      dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
      if ( ! dbEntry->rows.real4 )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->rowDimensions = NULL;
      if (params->SUMStats[0].pwStdDev > 0.0 && (params->SUMStats[0].pwMax - params->SUMStats[0].pwMean) >= 0.0) {
       	 *(dbEntry->rows.real4) = (REAL4)(sqrt((params->SUMStats[0].pwMax - params->SUMStats[0].pwMean)/params->SUMStats[0].pwStdDev));
      } else {
       	 *(dbEntry->rows.real4) = -1.0;   /* This case should not happen except in test data; hard code for now */
      }

      /* confidence */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "confidence" );

      dbEntry->type       = real_4;
      dbEntry->numberRows = 1;
      dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
      if ( ! dbEntry->rows.real4 )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->rowDimensions = NULL;
      if (params->SUMStats[0].pwMean > 0.0) {
       	 *(dbEntry->rows.real4) = (REAL4)(exp(-1.0*params->SUMStats[0].pwMax/params->SUMStats[0].pwMean));  /* Really want something like Brady & Creighton Eq. 3.1 */
      } else {
       	 *(dbEntry->rows.real4) = -1.0;   /* This case should not happen except in test data; hard code for now */
      }

      /* stat_name */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "stat_name" );

      dbEntry->type       = char_s_ptr;
      dbEntry->numberRows = 1;

      stringLength = strlen("Max Power");
      dbEntry->rows.chars = (CHAR *) LALMalloc( stringLength * sizeof(CHAR) );
      if ( ! dbEntry->rows.chars )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      strncpy(dbEntry->rows.chars, "Max Power",stringLength*sizeof(CHAR));
      dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
      if ( ! dbEntry->rowDimensions )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      /* Don't include terminating null */
      *(dbEntry->rowDimensions) = stringLength;

      /* stat_value */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "stat_value" );

      dbEntry->type       = real_4;
      dbEntry->numberRows = 1;
      dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
      if ( ! dbEntry->rows.real4 )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->rowDimensions = NULL;
      *(dbEntry->rows.real4) = (REAL4) params->SUMStats[0].pwMax;

      /* stat_prob */
      dbEntry->next = (dataBase *) LALMalloc( sizeof(dataBase) );
      if ( ! dbEntry->next )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->next->previous = dbEntry;
      dbEntry = dbEntry->next;

      sprintf( dbEntry->tableName, "sngl_dperiodic" );
      sprintf( dbEntry->columnName, "stat_prob" );

      dbEntry->type       = real_4;
      dbEntry->numberRows = 1;
      dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
      if ( ! dbEntry->rows.real4 )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      dbEntry->rowDimensions = NULL;
      if (params->SUMStats[0].pwMean > 0.0) {
       	 *(dbEntry->rows.real4) = (REAL4)(exp(-1.0*params->SUMStats[0].pwMax/params->SUMStats[0].pwMean));  /* Really want something like Brady & Creighton Eq. 3.1 */
      } else {
       	 *(dbEntry->rows.real4) = -1.0;   /* This case should not happen except in test data; hard code for now */
      }

      /* Terminate linked list */
      dbEntry->next = NULL;

  } /* END for (i = 0; i < output->numOutput; i++) */

  /* macro to detach the status pointer */
  DETATCHSTATUSPTR( status );

  RETURN(status);
}
/* END BuildOutput */

/* Based on StochasticBuildOutput.c written by Anderson, W.G., Romano, J.D., and Whelan, J.T. */
#ifdef INCLUDE_BUILDSUMFRAME_CODE
static void BuildSUMFrameOutput(LALStatus* const status,
	    const StackSlideSearchParams*    const params,
	    LALSearchOutput*            const output)
{
   outPut* outputArray;
   multiDimData*   mddEntry;
   CHARVector     *unitString;

   LALUnitPair  unitPair;   /* A pair of LALUnits */
   LALUnit     unitSUM;     /* Temporary place holder for units. */

   dcHistory *  historyEntry;

   UINT4 k = 0;    /* index used with SUMData */
   UINT4 iOut = 0; /* index used with outputArray */

   INITSTATUS( status, "BuildDemodFramesOutput", STACKSLIDEC );
   ATTATCHSTATUSPTR (status);

   outputArray = output->result;

   /* Set the dimensions from the output SUMs  */
   if (params->stackTypeFlag == 0) {
      unitPair.unitOne = &(params->unitBLK);
      unitPair.unitTwo = &(params->unitBLK);
      LALUnitMultiply(status->statusPtr, &unitSUM, &unitPair);
      CHECKSTATUSPTR (status);
   }

  for (k = 0; k < 1; k++) {

      iOut =  k + params->numDBOutput;

      outputArray[iOut].search         = periodic;
      outputArray[iOut].significant    = 1;   /* 1 - IS significant */
      outputArray[iOut].states         = 0;
      outputArray[iOut].optional       = 0;
      outputArray[iOut].indexNumber = 0;

     mddEntry = outputArray[iOut].optional = (multiDimData *) LALCalloc (1, sizeof(multiDimData));
     if (!mddEntry)
     {
      ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }

     /* write the name and domain of the data */
     strncpy (mddEntry->name, "SUM_output", maxMultiDimName);

     unitString = NULL;
     TRY( LALCHARCreateVector(status->statusPtr, &unitString, LALUnitTextSize), status );
     TRY( LALUnitAsString(status->statusPtr, unitString, &(unitSUM)), status );
     strncpy (mddEntry->units, unitString->data, maxMultiDimUnits);
     TRY( LALCHARDestroyVector(status->statusPtr, &unitString), status );
     mddEntry->space = freqD;

     #ifdef DEBUG_BUILDSUMFRAMEOUTPUT_CODE
  	fprintf(stdout, "\ngpsStartTimeSec= %i \n", (UINT4)params->timeIntervals[0].startSec);
  	fprintf(stdout, "\ngpsStartTimeNan = %i \n",(UINT4)params->timeIntervals[0].startNan);
  	fprintf(stdout, "\ngpsStopTimeSec = %i \n", (UINT4)params->timeIntervals[params->numBLKs - 1].stopSec);
  	fprintf(stdout, "\ngpsStopTimeNan = %i \n", (UINT4)params->timeIntervals[params->numBLKs - 1].stopNan);
  	fflush(stdout);
     #endif

     /* THIS ONLY WORKS WHEN THERE IS ONLY ONE SUM, WHICH IS TRUE FOR NOW; ALSO USED IN HISTORY BELOW */
     /* OTHERWISE INTERVALS MUST BE CHANGED TO BE SUM INTERVALS */
     mddEntry->range.dFreq.gpsStartTimeSec = (UINT4)params->timeIntervals[0].startSec;
     mddEntry->range.dFreq.gpsStartTimeNan = (UINT4)params->timeIntervals[0].startNan;
     mddEntry->range.dFreq.gpsStopTimeSec = (UINT4)params->timeIntervals[params->numBLKs - 1].stopSec;
     mddEntry->range.dFreq.gpsStopTimeNan = (UINT4)params->timeIntervals[params->numBLKs -1].stopNan;

     #ifdef DEBUG_BUILDSUMFRAMEOUTPUT_CODE
  	fprintf(stdout, "\nstartFreq = %f \n", params->f0SUM);
  	fprintf(stdout, "\nstopFreq = %f  \n", params->f0SUM + (REAL8)(params->nBinsPerSUM - 1) * params->dfSUM);
  	fprintf(stdout, "\nfreqStepSize = %f \n",params->dfSUM);
  	fflush(stdout);
     #endif

     mddEntry->range.dFreq.startFreq = params->f0SUM;
     mddEntry->range.dFreq.stopFreq  = params->f0SUM + (REAL8)(params->nBinsPerSTK - 1) * params->dfSUM;
     mddEntry->range.dFreq.freqStepSize = params->dfSUM;

     mddEntry->range.dFreq.numberSamples = params->nBinsPerSUM;

     /* data type is REAL4 */
     mddEntry->type = real_4;

     /* size of the data */
     mddEntry->numberDimensions = 1;
     mddEntry->dimensions = (UINT4 *) LALMalloc (sizeof(UINT4));
     if (!(mddEntry->dimensions))
     {
      ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }
     *(mddEntry->dimensions) = (UINT4)params->nBinsPerSUM;

      /* Start history linked list */
      historyEntry = mddEntry->history = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      historyEntry->previous = NULL;

      /* Use ifdef below to remove items from history list */
#ifdef HISTORY_test
      /* dso_name */
      sprintf(historyEntry->name, "dso_name" );
      sprintf(historyEntry->units, " " );

      historyEntry->type = char_s_ptr;
      historyEntry->numberValues = strlen( params->dsoName );
      historyEntry->value.chars = (CHAR *) LALMalloc( (strlen( params->dsoName ) + 1) * sizeof(CHAR) );
      if ( ! historyEntry->value.chars )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      strcpy(historyEntry->value.chars, params->dsoName);
#endif

      /* RA */
      sprintf(historyEntry->name, "RA" );
      sprintf(historyEntry->units, "radians" );

      historyEntry->type = real_8;
      historyEntry->numberValues = 1;
      historyEntry->value.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
      if ( ! historyEntry->value.real8 )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      *(historyEntry->value.real8) = (REAL8)(params->startRA);

      /* Dec */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "Dec" );
      sprintf(historyEntry->units, "radians" );

      historyEntry->type = real_8;
      historyEntry->numberValues = 1;
      historyEntry->value.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
      if ( ! historyEntry->value.real8 )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      *(historyEntry->value.real8) = (REAL8)(params->startDec);

      /* freqDeriv1 */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "freqDeriv1" );
      sprintf(historyEntry->units, "Hz2" );

      historyEntry->type = real_8;
      historyEntry->numberValues = 1;
      historyEntry->value.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
      if ( ! historyEntry->value.real8 )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      *(historyEntry->value.real8) = (REAL8)(params->startFDeriv1);

      /* freqDeriv2 */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "freqDeriv2" );
      sprintf(historyEntry->units, "Hz3" );

      historyEntry->type = real_8;
      historyEntry->numberValues = 1;
      historyEntry->value.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
      if ( ! historyEntry->value.real8 )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      *(historyEntry->value.real8) = (REAL8)(params->startFDeriv2);

      /* freqDeriv3 */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "freqDeriv3" );
      sprintf(historyEntry->units, "Hz4" );

      historyEntry->type = real_8;
      historyEntry->numberValues = 1;
      historyEntry->value.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
      if ( ! historyEntry->value.real8 )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      *(historyEntry->value.real8) = (REAL8)(params->startFDeriv3);

      /* freqDeriv4 */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "freqDeriv4" );
      sprintf(historyEntry->units, "Hz5" );

      historyEntry->type = real_8;
      historyEntry->numberValues = 1;
      historyEntry->value.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
      if ( ! historyEntry->value.real8 )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      *(historyEntry->value.real8) = (REAL8)(params->startFDeriv4);

      /* freqDeriv5 */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "freqDeriv5" );
      sprintf(historyEntry->units, "Hz6" );

      historyEntry->type = real_8;
      historyEntry->numberValues = 1;
      historyEntry->value.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
      if ( ! historyEntry->value.real8 )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      *(historyEntry->value.real8) = (REAL8)(params->startFDeriv5);

      /* power_max */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;


      sprintf(historyEntry->name, "power_max" );
      sprintf(historyEntry->units, "power" );

      historyEntry->type = real_4;
      historyEntry->numberValues = 1;
      historyEntry->value.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
      if ( ! historyEntry->value.real4 )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      *(historyEntry->value.real4) = (REAL4) params->SUMStats[0].pwMax;

      /* freq_pwmax */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "freq_pwmax" );
      sprintf(historyEntry->units, "Hz" );

      historyEntry->type = real_8;
      historyEntry->numberValues = 1;
      historyEntry->value.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
      if ( ! historyEntry->value.real8 )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      *(historyEntry->value.real8) = params->SUMStats[0].freqPWMax;

     /* power_mean */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "power_mean" );
      sprintf(historyEntry->units, "power" );

      historyEntry->type = real_4;
      historyEntry->numberValues = 1;
      historyEntry->value.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
      if ( ! historyEntry->value.real4 )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      *(historyEntry->value.real4) = (REAL4)(params->SUMStats[0].pwMean);

     /* power_stddev */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "power_stddev" );
      sprintf(historyEntry->units, "power" );

      historyEntry->type = real_4;
      historyEntry->numberValues = 1;
      historyEntry->value.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
      if ( ! historyEntry->value.real4 )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      *(historyEntry->value.real4) = (REAL4)(params->SUMStats[0].pwStdDev);

      /* startSec */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "startSec" );
      sprintf(historyEntry->units, "s" );

      historyEntry->type = int_4s;
      historyEntry->numberValues = 1;
      historyEntry->value.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
      if ( ! historyEntry->value.int4s )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      *(historyEntry->value.int4s) = params->timeIntervals[0].startSec;

      /* startNan */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "startNan" );
      sprintf(historyEntry->units, "ns" );

      historyEntry->type = int_4s;
      historyEntry->numberValues = 1;
      historyEntry->value.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
      if ( ! historyEntry->value.int4s )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      *(historyEntry->value.int4s) = params->timeIntervals[0].startNan;

      /* sample_rate */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "sample_rate" );
      sprintf(historyEntry->units, "hz" );

      historyEntry->type = real_8;
      historyEntry->numberValues = 1;
      historyEntry->value.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
      if ( ! historyEntry->value.real8 )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      *(historyEntry->value.real8) = params->sampleRate;

      /* duration */
      historyEntry->next = (dcHistory *) LALMalloc( sizeof(dcHistory) );
      if ( ! historyEntry->next  )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      historyEntry->next->previous = historyEntry;
      historyEntry = historyEntry->next;

      sprintf(historyEntry->name, "duration" );
      sprintf(historyEntry->units, "sec" );

      historyEntry->type = real_8;
      historyEntry->numberValues = 1;
      historyEntry->value.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
      if ( ! historyEntry->value.real8 )
      {
        ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
      }
      *(historyEntry->value.real8) = (REAL8)(params->duration);

     historyEntry->next = NULL;
      /* End history linked list */

     /* allocate memory for the data */
     mddEntry->data.real4 = (REAL4 *) LALCalloc (params->nBinsPerSTK, sizeof(REAL4));
     if (!(mddEntry->data.real4))
     {
      ABORT( status, STACKSLIDEH_EALOC, STACKSLIDEH_MSGEALOC );
     }

     /* memcpy (mddEntry->data.real8, params->STKData[k]->fft->data->data, params->nBinsPerSTK * sizeof(REAL8) ); */

     memcpy (mddEntry->data.real4, params->SUMData[k]->data->data, params->nBinsPerSUM * sizeof(REAL4) );

  } /* end for (k = 0; k < 1; k++) */

   /* normal exit */
   DETATCHSTATUSPTR (status);
   RETURN (status);
}
/* END BuildSUMFrameOutput */
#endif

/* Calculate stats associated with frequency series */
void ComputePowerStats( const REAL4Vector *fsVec, FreqSeriesPowerStats *stats, REAL8 fStart, REAL8 df )
{
    UINT4 i = 0;         /* all purpose index */
    INT4 ipwMax = 0;     /* Index max power */
    REAL8 pw = 0.0;      /* power in one bin */
    REAL8 pwSum= 0.0;    /* Sum of power */
    REAL8 pwSum2 = 0.0;  /* Sum of power squared */

    stats->pwMax = 0.0;  /* Initialize */

    for(i=0;i<fsVec->length;i++) {
		pw = (REAL8)fsVec->data[i];

	        pwSum += pw;
	        pwSum2 += pw*pw;

		/* Update maximum power */
		if(stats->pwMax < pw)
		{
			stats->pwMax=pw;
			ipwMax=i;
		}
    }
    stats->freqPWMax = fStart + ((REAL8)ipwMax)*df; /* Freq associated with max power */
    stats->pwMean = pwSum/((REAL8)fsVec->length);   /* Mean power */
    /* Standard Deviation of the power = sqrt of the variance */
    stats->pwStdDev = pwSum2/((REAL8)fsVec->length) - stats->pwMean*stats->pwMean;
    if ( stats->pwStdDev > 0.0 ) {
    	stats->pwStdDev = sqrt( stats->pwStdDev );
    } else {
        stats->pwStdDev = -1.0; /* Should not happen except due to round off error in test data */
    }
}

#ifdef INCLUDE_LALINITBARYCENTER
 /* Copied from LALInitBarycenter.c in the LAL support package.  Author: Curt Cutler */
 void LALInitBarycenter(LALStatus *stat, EphemerisData *edat)
 {

    FILE *fp1, *fp2; /* fp1 is table of Earth location; fp2 is for Sun*/

    INT4 j; /*dummy index*/
    INT4 gpsYr;  /*gpsYr + leap is the time on the GPS clock
                          at first instant of new year, UTC; equivalently
                          leap is # of leap secs added between Jan.6, 1980 and
                          Jan. 2 of given year */

/*  07/01/02 gam Comment out since LALINITBARYCENTERC not defined */
/*     INITSTATUS(stat,"LALInitBarycenter",LALINITBARYCENTERC);
    ATTATCHSTATUSPTR(stat); */

/*  07/01/02 gam  Change for local stand-alone code */
    /* fp1 = LALOpenDataFile(edat->ephiles.earthEphemeris);
    fp2 = LALOpenDataFile(edat->ephiles.sunEphemeris); */
    fp1 = fopen(edat->ephiles.earthEphemeris, "r");
    fp2 = fopen(edat->ephiles.sunEphemeris, "r");

    /*
    fp1 = fopen(edat->ephiles.earthEphemeris,"r");
    fp2 = fopen(edat->ephiles.sunEphemeris,"r");
    */

    /* CHECK THAT fp1 and fp2 are not NULL: */
    if ( ( fp1 == NULL ) || ( fp2 == NULL ) ) {
      /*  07/01/02 gam Comment out since LALINITBARYCENTERC not defined */
      /* ABORT (stat, LALINITBARYCENTERH_EOPEN, LALINITBARYCENTERH_MSGEOPEN); */
      return;
    }

/*reading first line of each file */

	 fscanf(fp1,"%d %le %d\n", &gpsYr, &edat->dtEtable, &edat->nentriesE);
	 fscanf(fp2,"%d %le %d\n", &gpsYr, &edat->dtStable, &edat->nentriesS);

     edat->ephemE  = (PosVelAcc *)LALMalloc(edat->nentriesE*sizeof(PosVelAcc));
     edat->ephemS  = (PosVelAcc *)LALMalloc(edat->nentriesS*sizeof(PosVelAcc));

/*first column in earth.dat or sun.dat is gps time--one long integer
  giving the number of secs that have ticked since start of GPS epoch
  +  on 1980 Jan. 6 00:00:00 UTC
*/
       for (j=0; j < edat->nentriesE; ++j){

	 /* check return value of fscanf */
	 fscanf(fp1,"%le %le %le %le %le %le %le %le %le %le\n",
		&edat->ephemE[j].gps,&edat->ephemE[j].pos[0],
		&edat->ephemE[j].pos[1],&edat->ephemE[j].pos[2],
		&edat->ephemE[j].vel[0],&edat->ephemE[j].vel[1],
		&edat->ephemE[j].vel[2],&edat->ephemE[j].acc[0],
		&edat->ephemE[j].acc[1],
		&edat->ephemE[j].acc[2] );
       }

       for (j=0; j < edat->nentriesS; ++j){
	 fscanf(fp2,"%le %le %le %le %le %le %le %le %le %le\n",
		&edat->ephemS[j].gps,&edat->ephemS[j].pos[0],
		&edat->ephemS[j].pos[1],&edat->ephemS[j].pos[2],
		&edat->ephemS[j].vel[0],&edat->ephemS[j].vel[1],
		&edat->ephemS[j].vel[2],&edat->ephemS[j].acc[0],
		&edat->ephemS[j].acc[1],
		&edat->ephemS[j].acc[2] );
       }


/*Test to make sure last entries for gpsE,S are
  reasonable numbers; specifically, checking that they are
  of same order as t2000
  --within factor e
Machine dependent: to be fixed!

    if ( (fabs(log(1.e0*(edat->ephemE[edat->nentriesE -1].gps)/t2000)) > 1.e0)
        ||(fabs(log(1.e0*(edat->ephemS[edat->nentriesS -1].gps)/t2000)) > 1.e0) ){
      LALFree(edat->ephemE);
      LALFree(edat->ephemS);
      ABORT(stat,LALINITBARYCENTERH_EEPHFILE, LALINITBARYCENTERH_MSGEEPHFILE);
    }
*/
	fclose(fp1);
        fclose(fp2);

	/*curt: is below the right return???*/
        /*  07/01/02 gam Comment out since LALINITBARYCENTERC not defined */
	/*
	DETATCHSTATUSPTR(stat);
	RETURN(stat);
	*/
 }
#endif

#ifdef INCLUDE_LALWRAPPERINITBARYCENTER
 /* Function to transfer ephemeris data input as ilwd from LALSearchInput structure into LAL structures; based on LALInitBarycenter. */
 void LALWRAPPERInitBarycenter(LALStatus *stat, LALSearchInput *inout, EphemerisData *edat)
 {

    /* Author: Greg Mendell; based on LALInitBarycenter. Started: 07/10/02. */

    BOOLEAN earthDataFound = 0;
    BOOLEAN sunDataFound = 0;

    INT4 i; /*dummy index*/
    INT4 j; /*dummy index*/

    /* Loop through sequence again to find BLK data */
    for (i = 0; i < inout->numberSequences; ++i)
    {
         if (strstr(inout->sequences[i].name, "earth")) {

		earthDataFound = 1;

		edat->dtEtable = inout->sequences[i].range.dTime.timeStepSize; /* time step between entries */

		edat->nentriesE = inout->sequences[i].dimensions[1]; /* number of entries */

                #ifdef DEBUG_LALWRAPPERINITBARYCENTER
			fprintf(stdout, "\nFound earth data; dtEtable = %g, nentriesE = %i \n", edat->dtEtable, edat->nentriesE);
			fflush(stdout);
		#endif

     	        edat->ephemE = (PosVelAcc *)LALMalloc(edat->nentriesE*sizeof(PosVelAcc));

                for (j=0; j < edat->nentriesE; ++j){

		     /* initialize time for first entry or increment with time step from previous entry */
                     if (j == 0) {
		         edat->ephemE[j].gps = inout->sequences[i].range.dTime.startSec;
		     } else {
		         edat->ephemE[j].gps = edat->ephemE[j-1].gps + edat->dtEtable;
		     }

		     /* successive 9 values are 3D position, velocity, and acceleration */
		     edat->ephemE[j].pos[0] = inout->sequences[i].data.real4[0 + j*9];
		     edat->ephemE[j].pos[1] = inout->sequences[i].data.real4[1 + j*9];
		     edat->ephemE[j].pos[2] = inout->sequences[i].data.real4[2 + j*9];
		     edat->ephemE[j].vel[0] = inout->sequences[i].data.real4[3 + j*9];
		     edat->ephemE[j].vel[1] = inout->sequences[i].data.real4[4 + j*9];
		     edat->ephemE[j].vel[2] = inout->sequences[i].data.real4[5 + j*9];
		     edat->ephemE[j].acc[0] = inout->sequences[i].data.real4[6 + j*9];
		     edat->ephemE[j].acc[1] = inout->sequences[i].data.real4[7 + j*9];
		     edat->ephemE[j].acc[2] = inout->sequences[i].data.real4[8 + j*9];

                     #ifdef DEBUG_LALWRAPPERINITBARYCENTER
			fprintf(stdout, "\nEarth pos,vel,and acc data for GPS time = %f: %19.16f %19.16f %19.16f %23.16e %23.16e %23.16e %23.16e %23.16e %23.16e \n",edat->ephemE[j].gps,
			        edat->ephemE[j].pos[0],edat->ephemE[j].pos[1],edat->ephemE[j].pos[2],
				edat->ephemE[j].vel[0],edat->ephemE[j].vel[1],edat->ephemE[j].vel[2],
				edat->ephemE[j].acc[0],edat->ephemE[j].acc[1],edat->ephemE[j].acc[2]);
			fflush(stdout);
		     #endif

                }

	    } else if (strstr(inout->sequences[i].name, "sun")) {

		sunDataFound = 1;

                edat->dtStable = inout->sequences[i].range.dTime.timeStepSize; /* time step between entries */

		edat->nentriesS = inout->sequences[i].dimensions[1]; /* number of entries */

                #ifdef DEBUG_LALWRAPPERINITBARYCENTER
			fprintf(stdout, "\nFound sun data; dtStable = %g, nentriesS = %i \n", edat->dtStable, edat->nentriesS);
			fflush(stdout);
		#endif

	        edat->ephemS = (PosVelAcc *)LALMalloc(edat->nentriesS*sizeof(PosVelAcc));

                for (j=0; j < edat->nentriesS; ++j){

		     /* initialize time for first entry or increment with time step from previous entry */
                     if (j == 0) {
		         edat->ephemS[j].gps = inout->sequences[i].range.dTime.startSec;
		     } else {
		         edat->ephemS[j].gps = edat->ephemS[j-1].gps + edat->dtStable;
		     }

		     /* successive 9 values are 3D position, velocity, and acceleration */
		     edat->ephemS[j].pos[0] = inout->sequences[i].data.real4[0 + j*9];
		     edat->ephemS[j].pos[1] = inout->sequences[i].data.real4[1 + j*9];
		     edat->ephemS[j].pos[2] = inout->sequences[i].data.real4[2 + j*9];
		     edat->ephemS[j].vel[0] = inout->sequences[i].data.real4[3 + j*9];
		     edat->ephemS[j].vel[1] = inout->sequences[i].data.real4[4 + j*9];
		     edat->ephemS[j].vel[2] = inout->sequences[i].data.real4[5 + j*9];
		     edat->ephemS[j].acc[0] = inout->sequences[i].data.real4[6 + j*9];
		     edat->ephemS[j].acc[1] = inout->sequences[i].data.real4[7 + j*9];
		     edat->ephemS[j].acc[2] = inout->sequences[i].data.real4[8 + j*9];

                     #ifdef DEBUG_LALWRAPPERINITBARYCENTER
			fprintf(stdout, "\nSun pos,vel,and acc data for GPS time = %f: %19.16f %19.16f %19.16f %23.16e %23.16e %23.16e %23.16e %23.16e %23.16e \n",edat->ephemS[j].gps,
			        edat->ephemS[j].pos[0],edat->ephemS[j].pos[1],edat->ephemS[j].pos[2],
				edat->ephemS[j].vel[0],edat->ephemS[j].vel[1],edat->ephemS[j].vel[2],
				edat->ephemS[j].acc[0],edat->ephemS[j].acc[1],edat->ephemS[j].acc[2]);
			fflush(stdout);
		     #endif

                }

         }

	 if (earthDataFound && sunDataFound) {
                #ifdef DEBUG_LALWRAPPERINITBARYCENTER
			fprintf(stdout, "\nFound earth and sun data.\n");
			fflush(stdout);
		#endif
	 	break;
	 }
    }
    /* END for (i = 0; i < inout->numberSequences; ++i) */
 }
 /* END void LALWRAPPERInitBarycenter(LALStatus *stat, LALSearchInput *inout, EphemerisData *edat) */
#endif

/******************************************/
/*                                        */
/* END SECTION: internal functions        */
/*                                        */
/******************************************/
