/********************************** <lalVerbatim file="TSApplySearchCV">
Author: Charlie Torres
$Id: TrackSearchApplySearch.c,v 1.1 2003/10/08 15:05:12 jolien Exp $
**************************************************** </lalVerbatim> */

#include <lal/LALStdlib.h>
#include <lal/LALDatatypes.h>
#include <lal/ExcessPower.h>
#include <lal/TimeFreq.h>
#include <lal/Comm.h>
#include <lal/AVFactories.h>
#include <lal/BurstSearch.h>
#include <lal/TSData.h>
#include <lal/TSSearch.h>
#include <lal/TrackSearch.h>
#include <lal/Window.h>
#include "TSExch.h"
#include "TrackSearchApplySearch.h"


#define TRUE 1
#define FALSE 0

#define PRINTAVERAGE 1

NRCSID( TSAPPLYSEARCHC, "TrackSearch $Id: TrackSearchApplySearch.c,v 1.1 2003/10/08 15:05:12 jolien Exp $");

static void CreateEventListFromCurveList(LALStatus*,TrackSearchOut*,CreateTimeFreqIn,TSSearchParams,TrackSearchEvent**);
static void DestroyEventListFromCurveList(LALStatus*,TrackSearchEvent**);

void
LALApplySearch(
    LALStatus             *status,
    LALSearchOutput       *output,
    LALSearchInput        *input,
    LALApplySearchParams  *applyParams
    )
{
  TSSearchParams             *params;
  INT4                        myRank;
  InitExchParams              initExchParams;
  TrackSearchEvent           *event;
  TrackSearchEventList        eventList;
  UINT4                       stringLength=0;
  /* Variables to create TFMaps */
  CreateTimeFreqIn                    tfInputs; /* Input structure for map */
  TimeFreqRep                        *tfmap = NULL; /* Pointer to new map */   
  REAL4Vector                        *SIGNAL = NULL;
  TrackSearchOut                     outputCurves;
  TimeFreqParam                      *autoparams = NULL; /* Code prepared params */
  TrackSearchParams                  inputParams; /*Tracksearch Params*/
  INT4                               j; /* counter */
  LALWindowParams                    windowParams;
  UINT4                              tmpDutyCycle;
  INITSTATUS (status, "LALApplySearch", TSAPPLYSEARCHC);
  ATTATCHSTATUSPTR (status);

  /* check inputs to Apply Search */
  ASSERT (output, status, TRACKSEARCHAPPLYSEARCHH_ENULL, 
	  TRACKSEARCHAPPLYSEARCHH_MSGENULL);
  ASSERT ( !(output->result), status, TRACKSEARCHAPPLYSEARCHH_ENNUL, 
	  TRACKSEARCHAPPLYSEARCHH_MSGENNUL);
  ASSERT (applyParams, status, TRACKSEARCHAPPLYSEARCHH_ENULL, 
	  TRACKSEARCHAPPLYSEARCHH_MSGENULL);
  ASSERT (applyParams->mpiParams, status, TRACKSEARCHAPPLYSEARCHH_ENULL,
	  TRACKSEARCHAPPLYSEARCHH_MSGENULL);
  ASSERT (applyParams->searchParams, status, TRACKSEARCHAPPLYSEARCHH_ENULL,
	  TRACKSEARCHAPPLYSEARCHH_MSGENULL);
  
  /* cast params into tracksearch datatype */
  params = (TSSearchParams *) applyParams->searchParams;


  /*
   * Search Master
   */

  if (params->searchMaster)
    {
      printf("I'm the Search Master in ApplySearch routines\n");
    /* only do this the first time the master is called */
      if (!params->numSlaves)
	{
      /* allocate memory and check for success */
	  params->numSlaves = (UINT4 *) LALMalloc( sizeof(UINT4) );
	  memset( params->numSlaves, 0, sizeof(UINT4) );
	  if(!params->numSlaves)
	    {
	      ABORT( status, TRACKSEARCHAPPLYSEARCHH_EALOC, TRACKSEARCHAPPLYSEARCHH_MSGEALOC );
	    }
	    /* get the number of search slaves */
	  MPI_Comm_size( *(applyParams->mpiParams->comm), params->numSlaves );
	  /**/printf("MPI Process Count:%d\n",*(params->numSlaves));
	  *(params->numSlaves) -= 1;
	}
      /* set the number of events to zero */
    params->numEvents = 0;

    /* initialize the event list */
    /* what is the form of tracksearch event list ???? */    
      stringLength =  TRACKSEARCHAPPLYSEARCHH_STRLENGTH;
      printf("Allocate Channel name ram next\n");
      eventList.channel = (CHAR *) LALMalloc(stringLength * sizeof(CHAR));
      printf("Done Allocating getting ready to copy\n");
      strncpy (eventList.channel, params->channelName, stringLength);
      printf("Done copying data\n");
    }

  /* Line 90 Power ApplySearch.c */

  /*
   * Determine my rank Slave,SearchMaster,WrapperMaster
   */

  printf("Entering Call to MPI_Comm\n");
  MPI_Comm_rank( *(applyParams->mpiParams->comm), &myRank );
  printf("Done Call to MPI_Comm\n");
  /* check that the master is running on node rank zero */
  if ( params->searchMaster && myRank != 0 )
  {
    printf("Aborting 1, Value searchMaster %d...myRank %d\n",params->searchMaster,myRank);
    ABORT( status, TRACKSEARCHAPPLYSEARCHH_ERANK, TRACKSEARCHAPPLYSEARCHH_MSGERANK );
  }

  /* and that the slave is not running on node rank zero */
  if ( !params->searchMaster && myRank == 0 )
  {
    printf("Aborting 2, Value searchMaster %d...myRank %d\n",params->searchMaster,myRank);
    ABORT( status, TRACKSEARCHAPPLYSEARCHH_ERANK, TRACKSEARCHAPPLYSEARCHH_MSGERANK );
  }
 
  initExchParams.mpiComm   = *(applyParams->mpiParams->comm);
  initExchParams.myProcNum = myRank;

  /* Finished MPI related error checking */


  /*
   * Search Starts Here
   */
  printf("Starting Search Exchanges\n");
  /* SEARCH MASTER */
  /* Line 120 ApplySearch.c */
 if ( params->searchMaster )
  {
    printf("There are slaves available :%d\n",*(params->numSlaves));
    /* if there are slaves */ 
    if ( *(params->numSlaves) )
    {
      ExchParams *thisExch = NULL;

      /* wait for a message */
      /* REMOVE AFTER DEBUG */
      printf("Calling LALInitExchange\n");
      LALInitializeExchange (status->statusPtr, &thisExch, NULL, &initExchParams);
      CHECKSTATUSPTR (status);
      
      printf("Setting up exchange data types at the switch statement\n");
      /* case the exchange type */
      switch (thisExch->exchObjectType)
      {
        
        /* how many segments slave should analyze */
        case ExchDataList:
	  /* %% */params->currentSeg = params->dataSegVec->length+1;
	  /* %% */tmpDutyCycle = params->currentSeg;
	  /* Remove above line for development */
          /* if there is more data to analyze  */
          if ( (params->currentSeg) < params->dataSegVec->length )
          {
            tmpDutyCycle=params->currentSeg; /* Do only one segment the first non-assigned segment left*/
            /* tell the slave how many segments */
            /* these two exchanges should be melded into one */
	    /* only send 1 segment per request largest size requested */
            LALExchangeUINT4 (status->statusPtr, &tmpDutyCycle, thisExch);
	    printf("I have segment %i and now assigned segment %i\n",params->currentSeg,tmpDutyCycle);
            CHECKSTATUSPTR (status);
            /* increment to the next segment number to be analyzed */
            params->currentSeg++;
          }
          else
          {
            /* tell the slave that there is no more data */
	    /* tmpDutyCycle will exceed params->NumSeg */
            LALExchangeUINT4 (status->statusPtr, &tmpDutyCycle, thisExch);
            CHECKSTATUSPTR (status);
          }
          break;
  
        /* exchange event list */
        case ExchTSEvent:
          /* get the number of events */
          LALExchangeUINT4 (status->statusPtr, &(params->numEvents), thisExch);
          CHECKSTATUSPTR (status);

          /* if there are events */
          if ( params->numEvents > 0 )
          {
            TrackSearchEventList outputParams; /*For use in linked list of output data*/

            /* set number of outputParams */
            outputParams.numEventMaps=params->numEvents;

            /* get the event list and stick them into temporary memory */
	    /* this event list will correlated to a particular map */
	    /* one event map returned per call to return results */
	    /*ToDo*/
	    /*consider doing this as a LALExchangeTSEventList with length 1 */
	    /* The actual length is obtained in the previous mpi call */
	    LALExchangeTSEventList(status->statusPtr,&event,1,thisExch);
            CHECKSTATUSPTR (status);

            /* point burstEventList to events */
            outputParams.eventlist = event;
            
            /* build the wrapper's output structure */
	    /*To do*/
            BuildTrackSearchOutput (status->statusPtr, output, 
				    &(outputParams),params);

            CHECKSTATUSPTR (status);
            
            /* free the temporary memory containin the events */
            while (event)
            {
              TrackSearchEvent *thisEvent;
              thisEvent = event;
              outputParams.eventlist  = outputParams.eventlist->nextEvent;
              LALFree( thisEvent );
            }
          }
          /* if there are no events set output to zero */
          else
          {
            output->numOutput     = 0;
            output->result        = NULL;
          }
          break;
      

        /* a slave is finished */
        case ExchFinished:
          --*(params->numSlaves);
          break;


        /* unrecognized type */
        default: 
          ABORT( status, TRACKSEARCHAPPLYSEARCHH_EUEXT, TRACKSEARCHAPPLYSEARCHH_MSGEUEXT );
      }
      /* end case exhange type */

      /* finish the exchange */
      LALFinalizeExchange (status->statusPtr, &thisExch);
      CHECKSTATUSPTR (status);

      /* set fraction remaining */
      output->fracRemaining = 1.0 - 
        ((REAL4)(params->currentSeg) / 
        (REAL4)(params->dataSegVec->length));
      /* BEGIN KLUDGE:  do we need this any more? */
      if ( output->fracRemaining < 0.001 ) { output->fracRemaining = 0.01; } 
      /* END KLUDGE */
    }
    else
    {
      /* there are no slaves left,  so finish */
      output->fracRemaining = 0;
      output->notFinished   = 0;
    }
    printf("Reach end of Master Loop\n");
  }

  /*
   *
   * if I'm a slave
   *
   */
 else
   {
     UINT4                   tmpDutyCycle;
     ExchParams              exchEvents;
     ExchParams              exchDutyCycle;
     ExchParams              exchFinished;
     ExchParams             *thisExch = NULL;
     
     /* printing message that this is slave process running */
     printf("I'm a slave node staring LALInitialize Exchange\n");
     
     /* exchange finished message */
     exchFinished.exchObjectType  = ExchFinished;
     exchFinished.send            = 0;  /* master sends */
     exchFinished.numObjects      = 0;
     exchFinished.partnerProcNum  = 0;  /* master */
     
     /* exchange duty cycle information */
     exchDutyCycle.exchObjectType = ExchDataList;
     exchDutyCycle.send           = 0; /* master sends */
     exchDutyCycle.numObjects     = 1; /* i send */
     exchDutyCycle.partnerProcNum = 0; /* master */
     
     /* exchange event information */
     exchEvents.exchObjectType = ExchTSEvent;
     exchEvents.send           = 1; /* i send */
     exchEvents.numObjects     = 1; /* i send */
     exchEvents.partnerProcNum = 0; /* master */
     
     
     /* find out if there is more data to analyze */ 
     LALInitializeExchange (status->statusPtr, &thisExch, 
			    &exchDutyCycle, &initExchParams);
     CHECKSTATUSPTR (status);
     
     LALExchangeUINT4 (status->statusPtr, &tmpDutyCycle, thisExch);
     CHECKSTATUSPTR (status);
     
     /* if there are no segments,  tell the master we're outta here!  */ 
     if (tmpDutyCycle > params->NumSeg )
       {
	 
	 LALFinalizeExchange (status->statusPtr, &thisExch);
	 CHECKSTATUSPTR (status);
	 
	 /* this is the slave shutdown handshake */
	 /* commenting out the following shutdown handshaking */
	 LALInitializeExchange (status->statusPtr, &thisExch, 
				&exchFinished, &initExchParams);
	 CHECKSTATUSPTR (status);
	 
	 LALFinalizeExchange (status->statusPtr, &thisExch);
	 CHECKSTATUSPTR (status);

	 /* make sure to tell the wrapperAPI that we're done */
	 output->notFinished   = 0;
	 
	 DETATCHSTATUSPTR (status);
	 RETURN (status);
	 
       }
     else
       {
	 /* there are segments,  so find out the next one to analyze */ 
	 printf("Doing Nothing Right Here\n");
	 /*	 LALExchangeUINT4 (status->statusPtr, &(params->currentSeg), thisExch);
		 CHECKSTATUSPTR (status);*/
       }
     
     /* finish the exchange */
     LALFinalizeExchange (status->statusPtr, &thisExch);
     CHECKSTATUSPTR (status);
     
     /* 
      *
      * This is the main engine of the tracksearch method
      *
      */
     
     
     /* Assign inputs to proper structure */
     inputParams.allocFlag = 1; /* Flag to prepare structure inside LALTrackSearch */
     
     /* Typecast the data to proper type */
     SIGNAL = (REAL4Vector*) params->dataSegVec->dataSeg[(INT4)params->currentSeg].data; 
     /*Get these statements working after some debugging*/
     tfInputs.type = Spectrogram; /*params.TransformType;*/
     tfInputs.fRow = 64;/*64*/ /*INT4 params.FreqBins;*/
     /* This looks like we get only 32 useful points from this call why?*/
     /* Remeber to setup the appropriate params in tracksearch.schema.in*/
     tfInputs.tCol = 100; /*INT4 params.TimeBins;*/
     tfInputs.wlengthT = 33; /*params.windowsize;*/
     tfInputs.wlengthF = 33; /*params.windowsize;*/
     params->FreqBins = tfInputs.fRow;
     params->TimeBins = tfInputs.tCol;
     params->windowsize = tfInputs.wlengthT;
     LALCreateTimeFreqParam(status->statusPtr,&autoparams,&tfInputs);
     CHECKSTATUSPTR(status);
     LALCreateTimeFreqRep(status->statusPtr,&tfmap,&tfInputs);
     CHECKSTATUSPTR(status);
     for (j=0;j<tfmap->tCol;j++)
       {
	 tfmap->timeInstant[j]=j;
       }
     /*insert case statement to do various tf methods */
     /*now just spectrgram*/
     /* We are going to set the windows right now */
     windowParams.length = autoparams->windowT->length;
     windowParams.type = Welch;
     LALWindow(status->statusPtr,autoparams->windowT,&(windowParams));
     LALTfrSp(status->statusPtr,SIGNAL,tfmap,autoparams);
     CHECKSTATUSPTR(status);
     /*Setting Input Params some are hardwired guesses for debug*/
     inputParams.sigma=params->LineWidth;
     inputParams.high=params->StartThresh;
     inputParams.low=params->LinePThresh;
     inputParams.height=(tfmap->fRow/2)+1;
     inputParams.width=tfmap->tCol;
     printf("Sending Parameters to the Search Routine\n");
     printf("Hi %e\nLow %e\nHeight %i\nWidth %i\n",inputParams.high,inputParams.low,inputParams.height,inputParams.width);
     /*Where should curve memory space allocation occur??*/
     printf("Calling the LAL Module\n");
     printf("Outputing the Map to Disk\n");
     /* Try temp data output function NOT LDAS Complient*/
     /* %% WriteMap(*tfmap,*SIGNAL); */
     /**/
     LALSignalTrackSearch(status->statusPtr,&outputCurves,tfmap,&inputParams);
     CHECKSTATUSPTR (status);
     printf("Done with the LAL Module call\n");
     /* To be stripped before submission */
     printf("Outputing RAW curve information\n");
     /* */
     /* Outputing Curves to File */
     /* %% WriteCurves(outputCurves); */
     /* Add Subroutine to prepare raw curve data and trim of schema arguements*/
     /* We call the Routine CurveCriteria */
     /* Send it double pointer to destroy structure and reallocate */
     /* only curves that meet search criteria send in length and power */
     /* criteria to edit the data structures */
     LALDestroyTimeFreqParam(status->statusPtr,&autoparams);
     CHECKSTATUSPTR(status);
     LALDestroyTimeFreqRep(status->statusPtr,&tfmap);
     CHECKSTATUSPTR(status);
     /*Unallocate other temporary memory as well */
     /* Move event information to a LALwrapper usable format */
     /* Tell master how many events to expect */
     LALInitializeExchange (status->statusPtr, &thisExch, 
			    &exchEvents, &initExchParams);
     CHECKSTATUSPTR (status);
     params->numEvents = outputCurves.numberOfCurves;
     params->numEvents = 0; /* %% Remove this line for development */
     LALExchangeUINT4 (status->statusPtr, &(params->numEvents), thisExch);
     CHECKSTATUSPTR (status);
     if ( params->numEvents > 0 )
       {
	 /* send the master the event list */
	 /* this routine should decompose the curvelist as individual events*/
	 TrackSearchEvent      *EventsFound = NULL;
	 printf("Sending %d Events\n",params->numEvents);
	 CreateEventListFromCurveList(status->statusPtr,&outputCurves,tfInputs,*params,&EventsFound);
	 CHECKSTATUSPTR(status);
	 /* Now cycle throught and transmit all event */
	 /* %% LALExchangeTSEventList(status->statusPtr, &(EventsFound), 
	    params->numEvents, thisExch);
	    CHECKSTATUSPTR (status); */
	    /* free the temporary memory containin the events */
	 while (EventsFound)
	   {
	     TrackSearchEvent *thisEvent;
	     thisEvent = EventsFound;
	     /*      Event = burstEvent->nextEvent;*/
	     LALFree( thisEvent );
	   }
	 /*Repeat loop for all curves*/
	 DestroyEventListFromCurveList(status->statusPtr,&EventsFound);
	 CHECKSTATUSPTR(status);
       }
     
     LALFinalizeExchange (status->statusPtr, &thisExch);
     CHECKSTATUSPTR (status);
     
     
     /* I think it is set in the end by the search master */
     /* Dont know if this is right */
     output->numOutput     = 0;
     output->result        = NULL;
     output->fracRemaining = 0;
     output->notFinished   = 1;
     
   }
 
 /* Not sure where this goes exactly Freeing eventList.channel */
 LALFree(eventList.channel);
 DETATCHSTATUSPTR (status);
 RETURN (status);
}

void CreateEventListFromCurveList(
				  LALStatus           *status,
				  TrackSearchOut      *mapcurves,
				  CreateTimeFreqIn     auxmapinfo,
				  TSSearchParams       searchparam,
				  TrackSearchEvent   **events
				  )
{
  INT4                 i,j; /*Counter*/
  TrackSearchEvent    *currentEvent = NULL;
  TrackSearchEvent     localEvents;
  i = mapcurves->numberOfCurves;
  /*  ASSERT(!*mapcurves,status,TRACKSEARCHAPPLYSEARCHH_ELOCF,TRACKSEARCHAPPLYSEARCHH_MSGELOCF);*/
  
  *events = (TrackSearchEvent *) LALMalloc (i * sizeof(TrackSearchEvent));
  /*  Make sure that the allocation was succesful */
  if ( !(&events) ){
    ABORT (status, TRACKSEARCHAPPLYSEARCHH_EALOC, TRACKSEARCHAPPLYSEARCHH_MSGEALOC);
  }
  for (i=0;i < mapcurves->numberOfCurves;i++)
    {
      localEvents.mapStartTime     = searchparam.GPSstart;
      localEvents.durationPoints   = searchparam.TimeLengthPoints;
      localEvents.FreqBins         = searchparam.FreqBins;
      localEvents.TimeBins         = searchparam.TimeBins;
      localEvents.window           = searchparam.window;
      localEvents.windowsize       = searchparam.windowsize;
      localEvents.overlap    = 0;  /* Correct this later */
      localEvents.TstartPixel      = mapcurves->curves[i].row[0];
      localEvents.FstartPixel      = mapcurves->curves[i].col[0];
      localEvents.TstopPixel       = mapcurves->curves[i].row[mapcurves->curves[i].n];
      localEvents.FstopPixel       = mapcurves->curves[i].col[mapcurves->curves[i].n];
      localEvents.junction         = mapcurves->curves[i].junction;
      localEvents.power            = mapcurves->curves[i].totalPower;
      for (j=0;j<=mapcurves->curves[i].n;j++)
	{
	  if (mapcurves->curves[i].depth[j] >= localEvents.peakPixelPower)
	    {
	      localEvents.peakPixelPower = mapcurves->curves[i].depth[j];
	      localEvents.peakPixelT = mapcurves->curves[i].row[j];
	      localEvents.peakPixelF = mapcurves->curves[i].col[j];
	    };
	};
      currentEvent=*events;
      **events=localEvents;
      (*events)++;
      currentEvent->nextEvent=*events;
    };
};

void DestroyEventListFromCurveList(
				   LALStatus         *status,
				   TrackSearchEvent  **events
				   )
     
{
  LALFree(events);
};


/****************************************************************************/
/* Non-ldas complient code to be stripped after testing */
/****************************************************************************/


void WriteMap(
	      TimeFreqRep       map,
	      REAL4Vector       signal
	      )

{
  INT4         i;
  INT4         j;
  INT4         k;
  FILE        *fp;
  FILE        *fp2;
  FILE        *fp3;

  fp = fopen("/home/charlie/research.d/matlabAnalysis.d/OutMap.dat","w");
  fp2 = fopen("/home/charlie/research.d/matlabAnalysis.d/OutMap2.dat","w");
  fp3 = fopen("/home/charlie/research.d/matlabAnalysis.d/OutSignal.dat","w");
  fprintf(fp,"Number of Time Columns: %i\n",map.tCol);
  fprintf(fp,"Number of Freq Rows:    %i\n",map.fRow);
  fprintf(fp,"Time Instants to follow\n");
  for (i = 0;i < map.tCol;i++)
    {
      fprintf(fp,"%d\n",map.timeInstant[i]);
    }
  fprintf(fp,"Freq Instants to follow\n");
  for (i = 0;i < (map.fRow/2+1);i++)
    {
      fprintf(fp,"%e\n",map.freqBin[i]);
    }
  fprintf(fp,"Raw Map ouput to follow space delimited carriage return marks end of row\n\n");
  k = 0;
  i = 0;
  j = 0;
  for (i = 0;i < map.tCol;i++)
    {
      for (j = 0;j <(map.fRow/2+1);j++)
      {
	/*fprintf(fp,"%f\n",*(map.map)[k]);*/
	fprintf(fp2,"%f\n",(&map)->map[i][j]);
	/*printf("k:%i,i:%i,j:%i  ",k,i,j);*/
	k++;
      };
    };
  for (i=0;i < signal.length;i++)
    {
      fprintf(fp3,"%f\n",signal.data[i]);
    }
  fclose(fp);
  fclose(fp2);
};

void WriteCurves(
		 TrackSearchOut      outCurve
		 )
{
  INT4         i;
  INT4         j;
  FILE        *fp;

  fp = fopen("/home/charlie/research.d/matlabAnalysis.d/OutCurve.dat","w");
  fprintf(fp,"Total Number of Curves:%i\n",outCurve.numberOfCurves);
  fprintf(fp,"\n");
  for (i = 0;i < outCurve.numberOfCurves;i++)
    {
      fprintf(fp,"Curve      #:%i\n",i);
      fprintf(fp,"Points      :%i\n",outCurve.curves[i].n);
      fprintf(fp,"Junction    :%c\n",outCurve.curves[i].junction);
      fprintf(fp,"Total Power :%f\n",outCurve.curves[i].totalPower);
      fprintf(fp,"Time      Freq      Depth\n");
      for (j = 0;j < outCurve.curves[i].n;j++)
	{
	  fprintf(fp,"%e    %e     %e\n",outCurve.curves[i].col[j],outCurve.curves[i].row[j],outCurve.curves[i].depth[j]);
	}
    }
  fclose(fp);
};

/* %% Is a marker to be in place for commitment to repository not during */
/* code development.  Will not be needed when done */
