/********************************** <lalVerbatim file="TSConditionDataCV">
Author: Charlie Torres
$Id: TrackSearchConditionData.c,v 1.2 2003/10/08 19:48:24 jolien Exp $
**************************************************** </lalVerbatim> */

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <unistd.h>
#include <lal/LALStdlib.h>
#include <lal/SeqFactories.h>
#include <lal/TimeFreq.h>
#include <lal/TrackSearch.h>
#include <lal/TSSearch.h>
#include <lal/TSData.h>
#include "TrackSearchConditionData.h"

NRCSID( TRACKSEARCHCONDITIONDATAC , "TrackSearch $Id: TrackSearchConditionData.c,v 1.2 2003/10/08 19:48:24 jolien Exp $" );

void LALConditionData(
		      LALStatus        *status,
		      LALSearchInput   *inout, /*What is this type*/
		      void             *searchParams,
		      LALMPIParams     *mpiParams
		      )
{
  INT4                 i; /* Counter */
  REAL4               *tempmddc = NULL; /* MultiDim Channel Data */
  INT4                 indexmddc = 0;   /* Index for MultiDim Data Channel */
  INT8                 dataTimeNS = 0;  /* Time in Secs for data stretch time */
  COMPLEX8            *tempmddr = NULL; /* MultiDim Response Data */
  REAL4               *tempmdds = NULL; /* MultiDim Spectrum Data */
  INT4                 indexmddr = 0;   /* Index for MultiDim Data Response */
  INT4                 indexmdds = 0;   /* Index for MultiDim Data Spectrum */
  TSSearchParams      *params = NULL; /* location of search params */
  REAL4Vector         *TData = NULL;
  COMPLEX8Vector      *TResponse = NULL;
  REAL4Vector         *TSpectrum = NULL;

  INITSTATUS (status,"TrackSearchConditionData",TRACKSEARCHCONDITIONDATAC);
  printf("Attaching Status Pointer\n");
  ATTATCHSTATUSPTR(status);

  /* Recast the unknown type to our custom search type */
  params = (TSSearchParams *) searchParams;
  /*
   * Search Master doesn't condition the data.
   */
  printf("Starting the TS Condition Data Module my node class is %d\n",params->searchMaster);
  if (params->searchMaster)
    {
      printf("I'm the search master returning\n");
      DETATCHSTATUSPTR(status);
      RETURN(status);
    }

  /*
   * If we didn't already return then this is a slave node.
   * Time to condition the data for our search.
   */
  printf("I'm the Slave and Conditioning the Data\n");
  ASSERT (searchParams, status,TRACKSEARCHCONDITIONDATAH_ENULL, TRACKSEARCHCONDITIONDATAH_MSGENULL);

  /*
   *
   * Loop to read out inout structure so that we condition the data and
   * pack into the appropriate LAL datatypes.
   *
   */

  for (i = 0; i < inout->numberSequences; i++)
    {
      printf("Splitting the data into chunks LOOP Count:%i\n",i);
      if (strstr( inout->sequences[i].name, INPUTNAME_CHANNEL))
	{
	  indexmddc = i;
	  tempmddc = inout->sequences[i].data.real4;

	  dataTimeNS = 1000000000L * 
            (INT8) inout->sequences[i].range.dTime.startSec;
	  dataTimeNS += (INT8) inout->sequences[i].range.dTime.startNan;
	}
      if ( strstr( inout->sequences[i].name, INPUTNAME_RESPONSE ) )
	{
	  indexmddr = i;
	  tempmddr   = inout->sequences[i].data.complex8;
	}
      if ( strstr( inout->sequences[i].name, INPUTNAME_SPECTRUM ) )
	{
	  indexmdds = i;
	  tempmdds   = inout->sequences[i].data.real4;
	}
    }
  /*
   * Error check all inout->sequences should be unique.  (No Repeated Types)
   */
  if (!tempmddc) {
    ABORT(status,TRACKSEARCHCONDITIONDATAH_EINPUT, TRACKSEARCHCONDITIONDATAH_MSGEINPUT);
  };

  if (!tempmddr) {
    ABORT(status,TRACKSEARCHCONDITIONDATAH_EINPUT, TRACKSEARCHCONDITIONDATAH_MSGEINPUT);
  };

  if (!tempmdds) {
    ABORT(status,TRACKSEARCHCONDITIONDATAH_EINPUT, TRACKSEARCHCONDITIONDATAH_MSGEINPUT);
  };
  /* What should be be checking Here the following is inappropriate */
  /* We should actually be checking against expected data segments */
  /*UINT4  NumSeg;*//* Number of segments length TLP */
  if (inout->sequences[indexmddc].dimensions[0] <=
      (params->NumSeg)) /*altered*/
    {
      ABORT(status,TRACKSEARCHCONDITIONDATAH_EDATZ,TRACKSEARCHCONDITIONDATAH_MSGEDATZ);
    }

  /*
   * Copy the MultiDim Incoming Data into the appropriate location in the 
   * TSSearch Datatype variable (params)
   */
  /* 
   * Copy the Channel Data into the params variable from MultiDimData
   */

  /* 2003Jun19 Note that right now we need to break up the tempXXXX data */
  /* for multiple tfmap searches Currently we are doing only one map */
  /* Call various LAL allocates for data copying */
  /* The only structure that doesn't need to be allocated apriori */
  /* is the one passed via TSSearch Params structure for data */
  /* Allocation of Local Variable to hold spectra and response functions */
  LALCCreateVector(status->statusPtr,&TResponse,params->TimeLengthPoints/2+1);
  LALCreateVector(status->statusPtr,&TSpectrum,params->TimeLengthPoints/2+1);
  LALCreateVector(status->statusPtr,&TData,params->TimeLengthPoints);

  /*we assume that data comes as three large sets 
    data,response,spectra*/

  memcpy((TData->data),tempmddc,(params->TimeLengthPoints) *sizeof(REAL4));
  TData->length = params->TimeLengthPoints;
  /* Copy Spectrum Function */
  /* Check for onesided or two sided spectra */
  /* This is for onesided spectra */
  memcpy((TSpectrum->data), tempmdds,
	 (params->TimeLengthPoints/2 +1)*sizeof(REAL4) );
  TSpectrum->length = params->TimeLengthPoints/2+1;
  /* Commented out next few lines 
     dummyseg->TSSearchSpectrum->epoch.gpsSeconds = 
     ( inout->sequences[indexmdds].range.dTime.startSec );
     
     dummyseg->TSSearchSpectrum->epoch.gpsNanoSeconds = 
     ( inout->sequences[indexmdds].range.dTime.startNan );
  */
  /*  dummyseg->TSSearchSpectrum->f0 = 0.0; */
  /* Copy the Response function */
  memcpy((TResponse->data), tempmddr, 
	 (params->TimeLengthPoints/2 +1)*sizeof(COMPLEX8) ); 
  TResponse->length = params->TimeLengthPoints/2+1;
  /* Commented out next few lines
     dummyseg->TSSearchResponse->deltaT = 
     inout->sequences[indexmddr].range.dFreq.freqStepSize;
     
     dummyseg->TSSearchResponse->epoch.gpsSeconds = 
     ( inout->sequences[indexmddr].range.dTime.startSec );
     
     dummyseg->TSSearchResponse->epoch.gpsNanoSeconds = 
     ( inout->sequences[indexmddr].range.dTime.startNan );  
      */
  /*  dummyseg->TSSearchResponse->f0 = 0.0; */
  /* Compare to lines 223 from POWER DSO Condition Data */
  /* Prepare the overwhiting here in the frequency domain */
  /* This should be in done in the frequency domain */
  /* We will only keep the data set after manipulation */
  TrackSearchPrep(status,TData,TResponse,TSpectrum,params->dataSegVec,params->NumSeg,params->SegLengthPoints);

  printf("Exiting Condition Data\n");
  printf("My Status Code: %d\n",status->statusCode);
  DETATCHSTATUSPTR (status);
  RETURN (status);
}

void TrackSearchPrep(
		     LALStatus           *status,
		     REAL4Vector         *TSSearchData,
		     COMPLEX8Vector      *TSSearchResponse,
		     REAL4Vector         *TSSearchSpectrum,
		     TSSegmentVector     *PreparedData,
		     UINT4               SegsToCreate,
		     UINT4               DataSegsLength
		     )
{
  INT4            i;
  INT4            j;
  INT4            k;
  /* We won't do anything yet */
  if (!TSSearchData)
    {
      ABORT(status,TRACKSEARCHCONDITIONDATAH_ESUBR,TRACKSEARCHCONDITIONDATAH_MSGESUBR);
    }
  if (!TSSearchResponse) 
    {
      ABORT(status,TRACKSEARCHCONDITIONDATAH_ESUBR,TRACKSEARCHCONDITIONDATAH_MSGESUBR);
    }
  if (!TSSearchSpectrum)
    {
      ABORT(status,TRACKSEARCHCONDITIONDATAH_ESUBR,TRACKSEARCHCONDITIONDATAH_MSGESUBR);
    }
  /* Not doing a damn thing */
  /* Copy and split the data up */
  k=0;
  for (i=0;i<SegsToCreate;i++)
    {
      for (j=0;j<DataSegsLength;j++)
	{
	  PreparedData->dataSeg[i].data->data[j]=TSSearchData->data[k];
	  k++;
	};
      PreparedData->dataSeg[i].data->length = DataSegsLength;
    };
  PreparedData->length = SegsToCreate;
  return;
};
