/********************************** <lalVerbatim file="TSInitSearchCV">A
Author: Charlie Torres
$Id: TrackSearchInitSearch.c,v 1.1 2003/10/08 15:05:12 jolien Exp $
**************************************************** </lalVerbatim> */

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <unistd.h>
#include <lal/TSData.h>
#include <lal/LALStdlib.h>
#include <lal/TimeFreq.h>
#include <lal/TrackSearch.h>
#include <lal/TSSearch.h> 
#include "TrackSearchInitSearch.h"
#include "LALWrapperInterface.h"

NRCSID(TSINITSEARCHC,"$Id: TrackSearchInitSearch.c,v 1.1 2003/10/08 15:05:12 jolien Exp $");

void LALInitSearch(
		   LALStatus             *status,
		   void                 **searchParams,
		   LALInitSearchParams   *initSearchParams
		   )
{
  TSCreateParams      memoryParams;
  TSSearchParams     *params;


  INITSTATUS( status, "LALInitSearch", TSINITSEARCHC );
  ATTATCHSTATUSPTR(status);

  /*
   *Sleep for Attaching DDD 
   */

#ifdef TRACKSEARCH_SO_ATTACH_DDD
  {
   unsigned int doze = 30;
   pid_t myPID;
   myPID = getpid( );
   fprintf( stdout, "pid %d sleeping for %d seconds\n", myPID, doze );
   fflush( stdout );
   sleep( doze );
   fprintf( stdout, "pid %d awake\n", myPID );
   fflush( stdout );
  }
#endif

  /*
   * Check Input, Output and Params Structures for proper grammer
   */

  /* Is SearchParams NULL it should not be. */
  if (!(initSearchParams)) {
    ABORT(status, TS_NULL_POINTER, TS_MSG_NULL_POINTER);
  }

  /* Is *searchParams NULL it should be. */
  if (*searchParams) {
    ABORT(status, TRACKSEARCHINITSEARCHH_ENNUL, TRACKSEARCHINITSEARCHH_MSGENNUL);
  }

  /* The argv member of initSearchParams should not be null */
  if (!(initSearchParams->argv)) {
    ABORT(status, TRACKSEARCHINITSEARCHH_ENULL,TRACKSEARCHINITSEARCHH_MSGENULL);
  }

  /* The argc field initSearchParams should state that there are 8 arguments */
  if (!(initSearchParams->argc = 9)) {
    ABORT(status, TRACKSEARCHINITSEARCHH_EARGS, TRACKSEARCHINITSEARCHH_MSGEARGS);
  }

  /* The zeroth field of the argv structure should contain "-filterparams" */
  fprintf(stdout,"Arg[0]: %s\n",initSearchParams->argv[0]);
  fprintf(stdout,"Arg[1]: %s ",initSearchParams->argv[1]);
  fprintf(stdout,"Arg[2]: %s ",initSearchParams->argv[2]);
  fprintf(stdout,"Arg[3]: %s ",initSearchParams->argv[3]);
  fprintf(stdout,"Arg[4]: %s ",initSearchParams->argv[4]);
  fprintf(stdout,"Arg[5]: %s ",initSearchParams->argv[5]);
  fprintf(stdout,"Arg[6]: %s ",initSearchParams->argv[6]);
  fprintf(stdout,"Arg[7]: %s ",initSearchParams->argv[7]);
  fprintf(stdout,"Arg[8]: %s\n",initSearchParams->argv[8]);
  if ((strcmp(initSearchParams->argv[0],"-filterparams"))) {
    ABORT(status,TRACKSEARCHINITSEARCHH_EZARG,TRACKSEARCHINITSEARCHH_MSGEZARG);
  }
     
      /* Check first arg GPS Start Time */
  if (atoi(initSearchParams->argv[1]) < 0) {
    ABORT(status,TRACKSEARCHINITSEARCHH_EGPST,TRACKSEARCHINITSEARCHH_MSGEGPST);
  }

  /* Time Series Stretch Must be > zero */
  if (atoi(initSearchParams->argv[2]) < 1) {
    ABORT(status,TRACKSEARCHINITSEARCHH_ENUMZ,TRACKSEARCHINITSEARCHH_MSGENUMZ);
  }

  /* Must specifiy Spectragram Type string */
  if (!((atoi(initSearchParams->argv[3]) > 0 )||(atoi(initSearchParams->argv[3]) < 6))) {
    ABORT(status,TRACKSEARCHINITSEARCHH_EWIND,TRACKSEARCHINITSEARCHH_MSGEWIND);
  }

  /* Pixel width for curves >0 <03 */
  if (!((atoi(initSearchParams->argv[4]) <= 3) || (atoi(initSearchParams->argv[3]) > 0))) {
    ABORT(status, TRACKSEARCHINITSEARCHH_EWIDE, TRACKSEARCHINITSEARCHH_MSGEWIDE);
  }

  /* 2nd derv line start point threshold */
  if (!(atoi(initSearchParams->argv[5]) >= 0)) {
    ABORT(status, TRACKSEARCHINITSEARCHH_ECSPT,TRACKSEARCHINITSEARCHH_MSGECSPT);
  }

  /* 2nd derv line point threshold check, must be less than start threshold, but greater than zero */
  if (!((atof(initSearchParams->argv[6]) < atof(initSearchParams->argv[5])) && (atof(initSearchParams->argv[6]) >0))) {
    ABORT(status, TRACKSEARCHINITSEARCHH_ECLPT,TRACKSEARCHINITSEARCHH_MSGECLPT);
  }

  /* Check for valid minimum curve length argument */
  if ((atoi(initSearchParams->argv[7]) < 2)) {
    ABORT(status, TRACKSEARCHINITSEARCHH_ELINE,TRACKSEARCHINITSEARCHH_MSGELINE);
  }

  /* Check for valid total power desired in curve */
  if (((atoi(initSearchParams->argv[8]) <=0))) {
    ABORT(status, TRACKSEARCHINITSEARCHH_EPOWL,TRACKSEARCHINITSEARCHH_MSGEPOWL);
  }

  /* Check for Channel Name to use */
  if (((strlen(initSearchParams->argv[9]) == 0))) {
    ABORT(status,TRACKSEARCHINITSEARCHH_ECHNL,TRACKSEARCHINITSEARCHH_MSGECHNL);
  }

  /* Done Checking Input Parameters for DSO */

  /*
   *
   * Memory Allocation
   *
   */
  printf("My Node rank is: %d\n",(initSearchParams->nodeClass));
  *searchParams = params = LALMalloc(sizeof(TSSearchParams));
  /* Params must be nonNull after memory allocation */
  if(!params) {
    ABORT(status,TRACKSEARCHINITSEARCHH_EALOC,TRACKSEARCHINITSEARCHH_MSGEALOC);
  }

  /* Allocate Memory to Store Channel Name */
  printf("This is the channel %s\n",initSearchParams->argv[9]);
  /**/
  params->channelName = (CHAR *) LALMalloc(sizeof(CHAR)*(strlen(initSearchParams->argv[9])+1));
  if (!params->channelName) {
    LALFree(params);
    ABORT(status,TRACKSEARCHINITSEARCHH_EALOC,TRACKSEARCHINITSEARCHH_MSGEALOC);
  }
  
  /*
   *
   * Initialize Search Parameters
   *
   */

  /* Setup GPS Start Time (Accurate to 1sec) */
  params->GPSstart.gpsSeconds = atoi(initSearchParams->argv[1]);
  params->GPSstart.gpsNanoSeconds = 0;

  /* Setup GPS Length (Accurate to 1sec) */
  params->SegLengthPoints = atoi(initSearchParams->argv[2]);
  params->Tlength.gpsSeconds = 0;
  params->Tlength.gpsNanoSeconds = 0;

  /*  params->Tlength.gpsSeconds = atoi(initSearchParams->argv[2]);
      params->Tlength.gpsNanoSeconds = 0; */

  /* Use the channel sampling rate to determine total number of */
  /* points requested , Also need to get the number of datasegments*/
  /* to break the incoming time series into */
  params->SamplingRate=1024; /* Not for Production use channel sets this */
  params->NumSeg = 10; /* Not Real Expected Value or Number of maps to make*/
  params->TimeLengthPoints = params->SegLengthPoints * params->NumSeg;
  /* Choose Transform Type Via Single Character */
  params->TransformType = atoi(initSearchParams->argv[3]);

  /* Set pixel width of curve */
  params->LineWidth = atoi(initSearchParams->argv[4]);

  /* Set Line Start Threshold */
  params->StartThresh = atof(initSearchParams->argv[5]);

  /* Set Line Point Threshold */
  params->LinePThresh = atof(initSearchParams->argv[6]);

  /* Set minium curve length */
  params->MinLength = atoi(initSearchParams->argv[7]);

  /* Set min power threshold */
  params->MinPower = atof(initSearchParams->argv[8]);

  /* Set Channel Name to Use */
  strcpy(params->channelName,initSearchParams->argv[9]);

  /* Done Setting Search Parameters */

  /*
   *
   * Initialize all data carrying members
   * Allocate the RAM neede to TSSearch Params structures data
   *
   */
  params->currentSeg=0;
  params->numSlaves = NULL;
  params->dataSegVec = NULL;
  /* Allocate memory for TF maps to send over the TrackSearchApplySearch */
  /* The TSSearch params structure needs this memory */
  /*  memoryParams.dataSegmentPoints = tempmddc->length; */
  memoryParams.dataSegmentPoints = params->TimeLengthPoints;
  memoryParams.numberDataSegments = params->NumSeg;
  /* Initialize the member dataSegVec */
  LALCreateTSDataSegmentVector(status->statusPtr,
			   &(params->dataSegVec),
			   &memoryParams);

  /* Done Initializing */
  /* Set Node Class */
  /* Allocate the memory required for the search here instead */
  params->searchMaster = initSearchParams->nodeClass;
  printf("I'm done Initializing Search Code\n");
  printf("My node Class is %d\n",params->searchMaster);
  /* Return from Initialize Search */
  printf("My Status Code: %d\n",status->statusCode);
  DETATCHSTATUSPTR (status);
  RETURN (status);
}  

