/*************** <lalVerbatim file="WaveHV"> ****************
Author: Klimenko, S. and Yakushin, I.
$Id: Wave.h,v 1.10 2003/07/27 20:43:39 igor Exp $
**************** </lalVerbatim> *************************************/

#ifndef _WAVE_H
#define _WAVE_H

#include <lal/LALRCSID.h>
#include <LALWrapperInterface.h>
#include <lal/LALStdlib.h>
#include <lal/LALWavelet.h>
#include <time.h>

NRCSID(WAVEH,"$Id");

/***** <lalLaTeX> *****

\section{Header \texttt{Wave.h}}

Provides the LAL-wrapper interface routines for the \texttt{waveburst} shared
object.

\subsection*{Synopsis}
\begin{verbatim}
#include <Wave.h>
\end{verbatim}


\subsection*{Error conditions}
\input{WaveHErrTab}

****** </lalLaTeX> *****/




#define MAX_NAME_LENGTH 256
#define MAX_NUMBER_OF_OUT_OPTIONS 100
#define MAX_OUT_OPTION_NAME 10

/*************** <lalErrTable file="WaveHErrTab"> ***********/

#define WAVEH_ENULL 1
#define WAVEH_ENSEQ 2
#define WAVEH_EUNITS 3
#define WAVEH_ETYPES 5
#define WAVEH_ESPACES 6
#define WAVEH_EDIMS 7

#define WAVEH_MSGENULL "Null pointer."
#define WAVEH_MSGENSEQ "Number of sequences is not 2"
#define WAVEH_MSGEUNITS "Inconsistent units"
#define WAVEH_MSGETYPES "Inconsistent data types"
#define WAVEH_MSGESPACES "Inconsistent sequence type"
#define WAVEH_MSGEDIMS "Inconsistent dimensions"

/*************** </lalErrTable> ************************************/

typedef struct 
tagWaveParams
{
  /*input data*/
  Wavelet *oneIn;
  Wavelet *twoIn;

  REAL4FrequencySeries *onePSD;
  REAL4FrequencySeries *twoPSD;

  /*percentile transform parameters*/
  REAL4 nonZeroFraction;

  /*pixelMixer parameters*/
  BOOLEAN usePixelMixer;
  UINT4 seed;

  /*pixelSwap parameters*/
  BOOLEAN usePixelSwapOne;
  BOOLEAN usePixelSwapTwo;

  /*coincidence parameters*/
  INT4 timeWindowPixels;
  INT4 freqWindowPixels;
  INT4 coincidenceLevel;
  REAL4 minAmp4SinglePixels;
  REAL4 minAmp4ClusterExtension;

  /*clustering parameters*/
  BOOLEAN aura;
  UINT4 minClusterSize;
  UINT4 maxClusterSize;
  REAL4 minLikelihood;
  REAL4 maxLikelihood;
  REAL4 minCorrelation;
  REAL4 maxCorrelation;
  REAL4 minPower;
  REAL4 maxPower;
  REAL4 minEnergy;
  REAL4 maxEnergy;

  /*simulation parameters*/
  UINT4 simulationType;

  /*number of LALApply calls*/
  INT4 ncalls;

  CHAR nameOne[MAX_NAME_LENGTH];
  CHAR nameTwo[MAX_NAME_LENGTH];

  CHAR comment[MAX_NAME_LENGTH];

  LIGOTimeGPS start;
  LIGOTimeGPS stop;

  /*output data from different stages of computation*/

  OutputPercentileWavelet *oneOutPT;
  OutputPercentileWavelet *twoOutPT;

  OutputPixelMixerWavelet *oneOutPM;
  OutputPixelMixerWavelet *twoOutPM;

  OutputPixelMixerWavelet *oneOutPM1;
  OutputPixelMixerWavelet *oneOutPM2;
  OutputPixelMixerWavelet *twoOutPM1;
  OutputPixelMixerWavelet *twoOutPM2;

  OutputPixelSwapWavelet *oneOutPS;
  OutputPixelSwapWavelet *twoOutPS;

  OutputCoincidenceWavelet *coincidenceOut;
  OutputCoincidenceWavelet *coincidenceOutPS;
  OutputCoincidenceWavelet *coincidenceOutPM;
  OutputCoincidenceWavelet *coincidenceOutPM1;
  OutputCoincidenceWavelet *coincidenceOutPM2;

  OutputClusterWavelet *oneOutCluster;
  OutputClusterWavelet *oneOutClusterPS;
  OutputClusterWavelet *oneOutClusterPM;
  OutputClusterWavelet *oneOutClusterPM1;
  OutputClusterWavelet *oneOutClusterPM2;

  OutputClusterWavelet *twoOutCluster;
  OutputClusterWavelet *twoOutClusterPS;
  OutputClusterWavelet *twoOutClusterPM;
  OutputClusterWavelet *twoOutClusterPM1;
  OutputClusterWavelet *twoOutClusterPM2;

  /*
    Print out options:
    oi  -   original wavelet for channel i
    pi  -   results of percentile transform for channel i
    pmi -   results of applying pixelMixer to channel i
    psi -   results of applying pixelSwap to channel i
    coi -   results of applying coincidence with the other channel to channel i 
    cli -   results of running cluster and clusterProperties for channel i
    blobP - put a blob with percentile amplitudes to the database
    blobO - pub a blob with original amplitudes to the database
    default: everything false except for cl1 and cl2
  */

  BOOLEAN o1;
  BOOLEAN o2;
  BOOLEAN p1;
  BOOLEAN p2;
  BOOLEAN co1;
  BOOLEAN co2;
  BOOLEAN cl1;
  BOOLEAN cl2;
  BOOLEAN blobP;
  BOOLEAN blobO;
  /* PT, PS, PM */
  CHAR outputToFile[10];
  LALSearchInput *input;

  BOOLEAN reuseCluster;

  REAL8 strain;
  REAL8 timeShift;

  BOOLEAN noise_rms_flag;
  REAL4 calibration_max_freq;

} WaveParams;


typedef struct tagLALSummaryDBWaveInput
{
  WaveParams *my;
  UINT4 nevents;
  UINT4 nevents_gw_1;
  UINT4 nevents_gw_2;
  UINT4 nevents_swp_1;
  UINT4 nevents_swp_2;
  UINT4 nevents_pm_1;
  UINT4 nevents_pm1_1;
  UINT4 nevents_pm2_1;
  UINT4 nevents_pm_2;
  UINT4 nevents_pm1_2;
  UINT4 nevents_pm2_2;
  UINT4 nnodes;
  INT4 *counter;
  CHAR comment[LALNameLength];
  REAL8 strain;
  REAL8 timeShift;
} LALSummaryDBWaveInput;

typedef struct tagLALClustersDBWaveInput
{
  WaveParams *my;
  ClusterWavelet *w;
  ClusterWavelet *theOther;
  INT4 *counter;
  CHAR comment[LALNameLength];
  UINT4 startEventID;
} LALClustersDBWaveInput;

typedef struct tagLALBlobsDBWaveInput
{
  WaveParams *my;
  ClusterWavelet *w;
  ClusterWavelet *theOther;
  INT4 *counter;
  CHAR comment[LALNameLength];
  UINT4 startEventID;
} LALBlobsDBWaveInput;

typedef struct tagLALClustersTSWaveInput
{
  WaveParams *my;
  REAL4TimeSeries *ts;
  INT4 *counter;
  CHAR comment[LALNameLength];
} LALClustersTSWaveInput;

typedef struct tagLALClustersParametersWaveInput
{
  WaveParams *my;
  ClusterWavelet *w1;
  ClusterWavelet *w2;
  INT4 *counter;
  CHAR comment[LALNameLength];
} LALClustersParametersWaveInput;

typedef struct tagLALClustersMediansWaveInput
{
  WaveParams *my;
  ClusterWavelet *w;
  INT4 *counter;
  CHAR comment[LALNameLength];
} LALClustersMediansWaveInput;

typedef struct tagLALClustersVolumesWaveInput
{
  WaveParams *my;
  ClusterWavelet *w;
  INT4 *counter;
  CHAR comment[LALNameLength];
} LALClustersVolumesWaveInput;

typedef struct tagLALClustersListWaveInput
{
  WaveParams *my;
  ClusterWavelet *w;
  INT4 *counter;
  CHAR comment[LALNameLength];
} LALClustersListWaveInput;

typedef struct tagLALClustersPMaskWaveInput
{
  WaveParams *my;
  ClusterWavelet *w;
  INT4 *counter;
  CHAR comment[LALNameLength];
} LALClustersPMaskWaveInput;

void LALBuildOutputWave(LALStatus *status,
			WaveParams *my, 
			LALSearchOutput *output
			);

void LALSummaryDBWave(LALStatus *status,
		      LALSummaryDBWaveInput *input,
		      LALSearchOutput *output
		      );

void LALClustersDBWave(LALStatus *status,
		       LALClustersDBWaveInput *input,
		       LALSearchOutput *output
		       );

void LALBlobsDBWave(LALStatus *status,
		    LALBlobsDBWaveInput *input,
		    LALSearchOutput *output
		    );

void LALClustersTSWave(LALStatus *status,
		       LALClustersTSWaveInput *input,
		       LALSearchOutput *output
		       );

void LALClustersParametersWave(LALStatus *status,
			       LALClustersParametersWaveInput *input,
			       LALSearchOutput *output
			       );

void LALClustersMediansWave(LALStatus *status,
			    LALClustersMediansWaveInput *input,
			    LALSearchOutput *output
			    );

void LALClustersVolumesWave(LALStatus *status,
			    LALClustersVolumesWaveInput *input,
			    LALSearchOutput *output
			    );

void LALClustersListWave(LALStatus *status,
			 LALClustersListWaveInput *input,
			 LALSearchOutput *output
			 );

void LALClustersPMaskWave(LALStatus *status,
			  LALClustersPMaskWaveInput *input,
			  LALSearchOutput *output
			  );

void LALSelectWave(LALStatus *status,
		   WaveParams *my); 

void LALSelectWave1(LALStatus *status,
		    WaveParams *my); 

void LALSelectWave2(LALStatus *status,
		    WaveParams *my); 


/**** <lalLaTeX> *****
\vfill{\footnotesize\input{WaveHV}}
%\newpage\input{WaveC}
***** </lalLaTeX> ***/


#endif /* _WAVEH */
