/**** <lalVerbatim file="BuildDBHV">
 * $Id: BuildDB.h,v 1.7 2003/01/23 19:50:19 jolien Exp $
 **** </lalVerbatim> */
/**** <lalLaTeX>
 *
 * \section{Header \texttt{BuildDB.h}}
 * \label{s:BuildDB.h}
 *
 * Provides LAL functions for building database entries.
 *
 * \subsection*{Synopsis}
 * \begin{verbatim}
 * #include <BuildDB.h>
 * \end{verbatim}
 *
 * \noindent This header covers LAL routines that are used
 * to build \texttt{dataBase} entries to be returned to the
 * \texttt{wrapperAPI} by \texttt{LALApplySearch()}.
 *
 **** </lalLaTeX> */

#include <lal/LALDatatypes.h>
#include <wrapperInterfaceDatatypes.h>
#define BUILDBH_LDAS_0_6

/**** <lalLaTeX>
 * \subsection*{Error conditions}
 **** </lalLaTeX> */
/**** <lalErrTable> */
#define BUILDDBH_ENULL 001
#define BUILDDBH_ENNUL 002
#define BUILDDBH_ESIZE 004
#define BUILDDBH_EALOC 010
#define BUILDDBH_EROWS 020
#define BUILDDBH_ETYPE 040
#define BUILDDBH_MSGENULL "Null pointer"
#define BUILDDBH_MSGENNUL "Non-null pointer"
#define BUILDDBH_MSGESIZE "Invalid data length"
#define BUILDDBH_MSGEALOC "Memory allocation error"
#define BUILDDBH_MSGEROWS "Invalid number of rows"
#define BUILDDBH_MSGETYPE "Invalid datatype"
/**** </lalErrTable> */

/**** <lalLaTeX>
 *
 * \subsection*{Structures}
 *
 * \subsubsection*{Type \texttt{LALBLOB}}
 *
 **** </lalLaTeX> */
/**** <lalVerbatim> */
typedef struct
tagLALBLOB
{
  UINT4  size;
  void  *data;
}
LALBLOB;
/**** </lalVerbatim> */
/**** <lalLaTeX>
 *
 * LAL container for a Binary Large OBject (BLOB).  The fields are
 * \begin{description}
 * \item[\texttt{size}] The size, in bytes, of the BLOB.
 * \item[\texttt{data}] Pointer to the memory containing the BLOB.
 * \end{description}
 *
 * \subsubsection*{Type \texttt{SearchSummaryInfo}}
 *
 **** </lalLaTeX> */
/**** <lalVerbatim> */
typedef struct
tagSearchSummaryInfo
{
  const char *dsoName;
  const char *comment;
  LIGOTimeGPS inputStartTime;
  LIGOTimeGPS inputEndTime;
  LIGOTimeGPS outputStartTime;
  LIGOTimeGPS outputEndTime;
  UINT4       numEvents;
  UINT4       numNodes;
}
SearchSummaryInfo;
/**** </lalVerbatim> */
/**** <lalLaTeX>
 *
 * Summary information about a search required by the function
 * \texttt{LALCreateSearchSummaryDataBase()} to populate the
 * \texttt{search\_summary} database table.  The fields are:
 * \begin{description}
 * \item[\texttt{dsoName}] The name of the DSO (no more than 64 characters).
 * \item[\texttt{comment}] User comment (no more than 240 characters).
 * \item[\texttt{inputStartTime}] The GPS start time of the input data.
 * \item[\texttt{inputEndTime}] The GPS end time of the input data.
 * \item[\texttt{outputStartTime}] The GPS start time of the data actually
 *      analyzed.
 * \item[\texttt{outputEndTime}] The GPS end time of the data actually analyzed.
 * \item[\texttt{numEvents}] The number of events found.
 * \item[\texttt{numNodes}] The number of nodes this job used.
 * \end{description}
 *
 * \subsubsection*{Type \texttt{SearchSummaryVariables}}
 *
 **** </lalLaTeX> */
/**** <lalVerbatim> */

typedef struct
tagSearchSummaryVariables
{
  const char  *name;
  const char  *string;
#ifdef BUILDBH_LDAS_0_6
  const REAL8 *value;
#else
  const REAL4 *value;
#endif
}
SearchSummaryVariables;
/**** </lalVerbatim> */
/**** <lalLaTeX>
 *
 * Summary variables generated by a search required by the function
 * \texttt{LALCreateSearchSummvarsDataBase()} to populate the
 * \texttt{search\_summvars} database table.  The fields are:
 * \begin{description}
 * \item[\texttt{name}] The name of the variable.
 * \item[\texttt{string}] The string associated with the variable (NULL to omit)
 * \item[\texttt{value}] The floating point value of the variable (NULL to omit)
 * \end{description}
 *
 * \vfill{\footnotesize\input{BuildDBHV}}
 * \newpage\input{BuildDBC}
 *
 **** </lalLaTeX> */

void
LALCreateBLOB(
    LALStatus   *status,
    LALBLOB    **output,
    const void  *input,
    UINT4        size
    );

void
LALDestroyBLOB(
    LALStatus  *status,
    LALBLOB   **blob
    );

void
LALCreateNextDBEntry( 
    LALStatus   *status,
    dataBase   **output,
    const CHAR  *table,
    const CHAR  *column
    );

void
LALCreateDBEntryData( 
    LALStatus *status,
    dataBase  *output,
    datatype   type,
    UINT4      rows
    );

void
LALCreateDBEntryBLOB(
    LALStatus *status,
    dataBase  *output,
    LALBLOB   *blob,
    UINT4      rows
    );

void
LALCreateDBEntryUBLOB(
    LALStatus *status,
    dataBase  *output,
    LALBLOB   *blob,
    UINT4      rows
    );

void
LALCreateSearchSummaryDataBase(
    LALStatus          *status,
    dataBase          **output,
    SearchSummaryInfo  *info
    );

void
LALCreateSearchSummvarsDataBase(
    LALStatus               *status,
    dataBase               **output,
    SearchSummaryVariables  *info
    );
