/**** <lalVerbatim file="ExtractSeriesHV">
 * $Id: ExtractSeries.h,v 1.6 2003/02/07 17:11:53 jolien Exp $
 **** </lalVerbatim> */
/**** <lalLaTeX>
 *
 * \section{Header \texttt{ExtractSeries.h}}
 * \label{s:ExtractSeries.h}
 *
 * Provides LAL functions for extracting time and frequency series from
 * a \texttt{LALSearchInput} structure.
 *
 * \subsection*{Synopsis}
 * \begin{verbatim}
 * #include <ExtractSeries.h>
 * \end{verbatim}
 *
 * \noindent This header covers LAL routines that are used to extract
 * time and frequency series from the \texttt{multiDimData} sequences
 * in a \texttt{LALSearchInput} structure.  These routines are typically
 * called in \texttt{LALConditionData()}.
 *
 **** </lalLaTeX> */

#include <LALWrapperInterface.h>

/**** <lalLaTeX>
 * \subsection*{Error conditions}
 **** </lalLaTeX> */
/**** <lalErrTable> */
#define EXTRACTSERIES_ENULL 001
#define EXTRACTSERIES_ENNUL 002
#define EXTRACTSERIES_EZSEQ 004
#define EXTRACTSERIES_EMTCH 010
#define EXTRACTSERIES_ETYPE 020
#define EXTRACTSERIES_ESERT 040

#define EXTRACTSERIES_MSGENULL "Null pointer"
#define EXTRACTSERIES_MSGENNUL "Non-null pointer"
#define EXTRACTSERIES_MSGEZSEQ "Zero sequences"
#define EXTRACTSERIES_MSGEMTCH "No matching sequence found"
#define EXTRACTSERIES_MSGETYPE "Wrong data type"
#define EXTRACTSERIES_MSGESERT "Wrong series type"
/**** </lalErrTable> */

/**** <lalLaTeX>
 * \vfill{\footnotesize\input{ExtractSeriesHV}}
 * \newpage\input{ExtractSeriesC}
 * \newpage\input{CalibrationC}
 **** </lalLaTeX> */

void
LALFindSequenceByName(
    LALStatus       *status,
    multiDimData   **output,
    LALSearchInput  *input,
    const CHAR      *regex
    );

void
LALFindSequenceByComment(
    LALStatus       *status,
    multiDimData   **output,
    LALSearchInput  *input,
    const CHAR      *regex
    );

void
LALExtractCOMPLEX16TimeSeries(
    LALStatus       *status,
    COMPLEX16TimeSeries *series,
    multiDimData *input
    );

void
LALExtractCOMPLEX8TimeSeries(
    LALStatus       *status,
    COMPLEX8TimeSeries *series,
    multiDimData *input
    );

void
LALExtractREAL8TimeSeries(
    LALStatus       *status,
    REAL8TimeSeries *series,
    multiDimData *input
    );

void
LALExtractREAL4TimeSeries(
    LALStatus       *status,
    REAL4TimeSeries *series,
    multiDimData *input
    );

void
LALExtractUINT8TimeSeries(
    LALStatus       *status,
    UINT8TimeSeries *series,
    multiDimData *input
    );

void
LALExtractUINT4TimeSeries(
    LALStatus       *status,
    UINT4TimeSeries *series,
    multiDimData *input
    );

void
LALExtractUINT2TimeSeries(
    LALStatus       *status,
    UINT2TimeSeries *series,
    multiDimData *input
    );

void
LALExtractINT8TimeSeries(
    LALStatus       *status,
    INT8TimeSeries *series,
    multiDimData *input
    );

void
LALExtractINT4TimeSeries(
    LALStatus       *status,
    INT4TimeSeries *series,
    multiDimData *input
    );

void
LALExtractINT2TimeSeries(
    LALStatus       *status,
    INT2TimeSeries *series,
    multiDimData *input
    );

void
LALExtractCOMPLEX16FrequencySeries(
    LALStatus       *status,
    COMPLEX16FrequencySeries *series,
    multiDimData *input
    );

void
LALExtractCOMPLEX8FrequencySeries(
    LALStatus       *status,
    COMPLEX8FrequencySeries *series,
    multiDimData *input
    );

void
LALExtractREAL8FrequencySeries(
    LALStatus       *status,
    REAL8FrequencySeries *series,
    multiDimData *input
    );

void
LALExtractREAL4FrequencySeries(
    LALStatus       *status,
    REAL4FrequencySeries *series,
    multiDimData *input
    );

void
LALExtractUINT8FrequencySeries(
    LALStatus       *status,
    UINT8FrequencySeries *series,
    multiDimData *input
    );

void
LALExtractUINT4FrequencySeries(
    LALStatus       *status,
    UINT4FrequencySeries *series,
    multiDimData *input
    );

void
LALExtractUINT2FrequencySeries(
    LALStatus       *status,
    UINT2FrequencySeries *series,
    multiDimData *input
    );

void
LALExtractINT8FrequencySeries(
    LALStatus       *status,
    INT8FrequencySeries *series,
    multiDimData *input
    );

void
LALExtractINT4FrequencySeries(
    LALStatus       *status,
    INT4FrequencySeries *series,
    multiDimData *input
    );

void
LALExtractINT2FrequencySeries(
    LALStatus       *status,
    INT2FrequencySeries *series,
    multiDimData *input
    );

void
LALExtractResponse(
    LALStatus               *status,
    COMPLEX8FrequencySeries *output,
    LALSearchInput          *input,
    const CHAR              *ifo
    );
